@@ -13,7 +13,7 @@
  *                         All rights reserved.
  * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2013-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
@@ -25,10 +25,10 @@
  *
  */
 
-#include <src/include/pmix_config.h>
-#include <pmix_server.h>
-#include <src/include/types.h>
-#include <src/include/pmix_globals.h>
+#include "src/include/pmix_config.h"
+#include "../include/pmix_server.h"
+#include "src/include/types.h"
+#include "src/include/pmix_globals.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,8 +42,6 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
-#include PMIX_EVENT_HEADER
-
 #include "src/class/pmix_list.h"
 #include "src/util/pmix_environ.h"
 #include "src/util/output.h"
@@ -266,9 +264,9 @@ int main(int argc, char **argv)
 
     /* setup to see sigchld on the forked tests */
     PMIX_CONSTRUCT(&children, pmix_list_t);
-    event_assign(&handler, pmix_globals.evbase, SIGCHLD,
-                 EV_SIGNAL|EV_PERSIST,wait_signal_callback, &handler);
-    event_add(&handler, NULL);
+    pmix_event_assign(&handler, pmix_globals.evbase, SIGCHLD,
+                      EV_SIGNAL|EV_PERSIST,wait_signal_callback, &handler);
+    pmix_event_add(&handler, NULL);
 
     /* see if we were passed the number of procs to run or
      * the executable to use */
@@ -424,7 +422,7 @@ static void set_namespace(int nprocs, char *ranks, char *nspace,
     char hostname[PMIX_MAXHOSTNAMELEN];
     pmix_status_t rc;
     myxfer_t myxfer;
-    size_t i;
+    size_t i = 0;
 
     gethostname(hostname, sizeof(hostname));
 
@@ -486,6 +484,7 @@ static void set_namespace(int nprocs, char *ranks, char *nspace,
 
     PMIx_server_register_nspace(nspace, nprocs, x->info, x->ninfo,
                                 cbfunc, x);
+
 }
 
 static void errhandler(size_t evhdlr_registration_id,
@@ -871,7 +870,7 @@ static void wait_signal_callback(int fd, short event, void *arg)
     pid_t pid;
     wait_tracker_t *t2;
 
-    if (SIGCHLD != event_get_signal(sig)) {
+    if (SIGCHLD != pmix_event_get_signal(sig)) {
         return;
     }
 
