@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
@@ -12,16 +12,14 @@
  *
  * $HEADER$
  */
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
-#include <src/include/types.h>
-#include <src/include/pmix_stdint.h>
-#include <src/include/pmix_socket_errno.h>
+#include "src/include/pmix_stdint.h"
+#include "src/include/pmix_socket_errno.h"
 
-#include <pmix.h>
-#include <pmix_common.h>
-#include <pmix_server.h>
-#include <pmix_rename.h>
+#include "include/pmix.h"
+#include "include/pmix_common.h"
+#include "include/pmix_server.h"
 
 #include "src/threads/threads.h"
 #include "src/util/argv.h"
@@ -120,7 +118,16 @@ static void acb(pmix_status_t status,
                 void *release_cbdata)
 {
     pmix_cb_t *cb = (pmix_cb_t*)cbdata;
+    size_t n;
+
     cb->status = status;
+    if (0 < ninfo) {
+        PMIX_INFO_CREATE(cb->info, ninfo);
+        cb->ninfo = ninfo;
+        for (n=0; n < ninfo; n++) {
+            PMIX_INFO_XFER(&cb->info[n], &info[n]);
+        }
+    }
     if (NULL != release_fn) {
         release_fn(release_cbdata);
     }
@@ -128,7 +135,8 @@ static void acb(pmix_status_t status,
 }
 
 PMIX_EXPORT pmix_status_t PMIx_Job_control(const pmix_proc_t targets[], size_t ntargets,
-                                           const pmix_info_t directives[], size_t ndirs)
+                                           const pmix_info_t directives[], size_t ndirs,
+                                           pmix_info_t **results, size_t *nresults)
 {
     pmix_cb_t cb;
     pmix_status_t rc;
@@ -158,6 +166,14 @@ PMIX_EXPORT pmix_status_t PMIx_Job_control(const pmix_proc_t targets[], size_t n
     /* wait for the operation to complete */
     PMIX_WAIT_THREAD(&cb.lock);
     rc = cb.status;
+    if (0 < cb.ninfo) {
+        if (NULL != results && NULL != nresults) {
+            *results = cb.info;
+            *nresults = cb.ninfo;
+            cb.info = NULL;
+            cb.ninfo = 0;
+        }
+    }
     PMIX_DESTRUCT(&cb);
 
     pmix_output_verbose(2, pmix_globals.debug_output,
@@ -279,7 +295,8 @@ PMIX_EXPORT pmix_status_t PMIx_Job_control_nb(const pmix_proc_t targets[], size_
 }
 
 PMIX_EXPORT pmix_status_t PMIx_Process_monitor(const pmix_info_t *monitor, pmix_status_t error,
-                                               const pmix_info_t directives[], size_t ndirs)
+                                               const pmix_info_t directives[], size_t ndirs,
+                                               pmix_info_t **results, size_t *nresults)
 {
     pmix_cb_t cb;
     pmix_status_t rc;
@@ -309,6 +326,12 @@ PMIX_EXPORT pmix_status_t PMIx_Process_monitor(const pmix_info_t *monitor, pmix_
     /* wait for the operation to complete */
     PMIX_WAIT_THREAD(&cb.lock);
     rc = cb.status;
+    if (0 < cb.ninfo) {
+        *results = cb.info;
+        *nresults = cb.ninfo;
+        cb.info = NULL;
+        cb.ninfo = 0;
+    }
     PMIX_DESTRUCT(&cb);
 
     pmix_output_verbose(2, pmix_globals.debug_output,
@@ -366,7 +389,7 @@ PMIX_EXPORT pmix_status_t PMIx_Process_monitor_nb(const pmix_info_t *monitor, pm
     PMIX_RELEASE_THREAD(&pmix_global_lock);
 
     /* if the monitor is PMIX_SEND_HEARTBEAT, then send it */
-    if (0 == strncmp(monitor->key, PMIX_SEND_HEARTBEAT, PMIX_MAX_KEYLEN)) {
+    if (PMIX_CHECK_KEY(monitor, PMIX_SEND_HEARTBEAT)) {
         msg = PMIX_NEW(pmix_buffer_t);
         if (NULL == msg) {
             return PMIX_ERR_NOMEM;
