@@ -11,7 +11,9 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2018      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,7 +21,7 @@
  * $HEADER$
  */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
 
 #ifdef HAVE_STRING_H
@@ -31,10 +33,10 @@
 #include <stdlib.h>
 #endif
 
-#include <pmix_common.h>
-#include <pmix_rename.h>
+#include "include/pmix_common.h"
 
 #include "src/include/pmix_globals.h"
+#include "src/common/pmix_attributes.h"
 
 PMIX_EXPORT const char* PMIx_Proc_state_string(pmix_proc_state_t state)
 {
@@ -228,6 +230,16 @@ PMIX_EXPORT const char* pmix_command_string(pmix_cmd_t cmd)
             return "IOF PUSH";
         case PMIX_IOF_PULL_CMD:
             return "IOF PULL";
+        case PMIX_GROUP_CONSTRUCT_CMD:
+            return "GROUP CONSTRUCT";
+        case PMIX_GROUP_JOIN_CMD:
+            return "GROUP JOIN";
+        case PMIX_GROUP_INVITE_CMD:
+            return "GROUP INVITE";
+        case PMIX_GROUP_LEAVE_CMD:
+            return "GROUP LEAVE";
+        case PMIX_GROUP_DESTRUCT_CMD:
+            return "GROUP DESTRUCT";
         default:
             return "UNKNOWN";
     }
@@ -241,25 +253,96 @@ PMIX_EXPORT const char* PMIx_IOF_channel_string(pmix_iof_channel_t channel)
 {
     size_t cnt=0;
 
-    memset(answer, 0, sizeof(answer));
     if (PMIX_FWD_STDIN_CHANNEL & channel) {
-        strncpy(&answer[cnt], "STDIN ", strlen("STDIN "));
+        strcpy(&answer[cnt], "STDIN ");
         cnt += strlen("STDIN ");
     }
     if (PMIX_FWD_STDOUT_CHANNEL & channel) {
-        strncpy(&answer[cnt], "STDOUT ", strlen("STDOUT "));
+        strcpy(&answer[cnt], "STDOUT ");
         cnt += strlen("STDOUT ");
     }
     if (PMIX_FWD_STDERR_CHANNEL & channel) {
-        strncpy(&answer[cnt], "STDERR ", strlen("STDERR "));
+        strcpy(&answer[cnt], "STDERR ");
         cnt += strlen("STDERR ");
     }
     if (PMIX_FWD_STDDIAG_CHANNEL & channel) {
-        strncpy(&answer[cnt], "STDDIAG ", strlen("STDDIAG "));
+        strcpy(&answer[cnt], "STDDIAG ");
         cnt += strlen("STDDIAG ");
     }
     if (0 == cnt) {
-        strncpy(&answer[cnt], "NONE", strlen("NONE"));
+        strcpy(&answer[cnt], "NONE");
     }
     return answer;
 }
+
+PMIX_EXPORT const char* PMIx_Job_state_string(pmix_job_state_t state)
+{
+    switch(state) {
+        case PMIX_JOB_STATE_UNDEF:
+            return "UNDEFINED";
+        case PMIX_JOB_STATE_PREPPED:
+            return "PREPPED FOR LAUNCH";
+        case PMIX_JOB_STATE_LAUNCH_UNDERWAY:
+            return "LAUNCHING";
+        case PMIX_JOB_STATE_RUNNING:
+            return "RUNNING";
+        case PMIX_JOB_STATE_SUSPENDED:
+            return "SUSPENDED";
+        case PMIX_JOB_STATE_CONNECTED:
+            return "CONNECTED";
+        case PMIX_JOB_STATE_UNTERMINATED:
+            return "UNTERMINATED";
+        case PMIX_JOB_STATE_TERMINATED:
+            return "TERMINATED";
+        case PMIX_JOB_STATE_TERMINATED_WITH_ERROR:
+            return "TERMINATED WITH ERROR";
+      default:
+            return "UNKNOWN";
+    }
+}
+
+PMIX_EXPORT const char* PMIx_Get_attribute_string(char *attribute)
+{
+    return pmix_attributes_lookup(attribute);
+}
+
+PMIX_EXPORT const char* PMIx_Get_attribute_name(char *attrstring)
+{
+    return pmix_attributes_reverse_lookup(attrstring);
+}
+
+PMIX_EXPORT const char* PMIx_Link_state_string(pmix_link_state_t state)
+{
+    switch(state) {
+        case PMIX_LINK_STATE_UNKNOWN:
+            return "UNKNOWN";
+        case PMIX_LINK_DOWN:
+            return "INACTIVE";
+        case PMIX_LINK_UP:
+            return "ACTIVE";
+        default:
+            return "UNKNOWN";
+    }
+}
+
+const char* PMIx_Device_type_string(pmix_device_type_t type)
+{
+    switch(type) {
+        case PMIX_DEVTYPE_UNKNOWN:
+            return "UNKNOWN";
+        case PMIX_DEVTYPE_BLOCK:
+            return "BLOCK";
+        case PMIX_DEVTYPE_GPU:
+            return "GPU";
+        case PMIX_DEVTYPE_NETWORK:
+            return "NETWORK";
+        case PMIX_DEVTYPE_OPENFABRICS:
+            return "OPENFABRICS";
+        case PMIX_DEVTYPE_DMA:
+            return "DMA";
+        case PMIX_DEVTYPE_COPROC:
+            return "COPROCESSOR";
+        default:
+            return "UNKNOWN";
+    }
+}
