@@ -10,7 +10,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2020      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,11 +22,11 @@
 #ifndef PMIX_EVENT_H
 #define PMIX_EVENT_H
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 #include "src/include/types.h"
 #include PMIX_EVENT_HEADER
 
-#include <pmix_common.h>
+#include "include/pmix_common.h"
 #include "src/class/pmix_list.h"
 #include "src/util/output.h"
 
@@ -122,6 +123,7 @@ typedef struct pmix_event_chain_t {
     bool timer_active;
     bool nondefault;
     bool endchain;
+    bool cached;
     pmix_proc_t source;
     pmix_data_range_t range;
     /* When generating events, callers can specify
@@ -138,7 +140,11 @@ typedef struct pmix_event_chain_t {
     size_t nallocated;
     pmix_info_t *results;
     size_t nresults;
+    pmix_info_t *interim;
+    size_t ninterim;
     pmix_event_hdlr_t *evhdlr;
+    pmix_op_cbfunc_t opcbfunc;
+    void *cbdata;
     pmix_op_cbfunc_t final_cbfunc;
     void *final_cbdata;
 } pmix_event_chain_t;
@@ -147,30 +153,30 @@ PMIX_CLASS_DECLARATION(pmix_event_chain_t);
 /* prepare a chain for processing by cycling across provided
  * info structs and translating those supported by the event
  * system into the chain object*/
-pmix_status_t pmix_prep_event_chain(pmix_event_chain_t *chain,
-                                    const pmix_info_t *info, size_t ninfo,
-                                    bool xfer);
+PMIX_EXPORT pmix_status_t pmix_prep_event_chain(pmix_event_chain_t *chain,
+                                                const pmix_info_t *info, size_t ninfo,
+                                                bool xfer);
 
 /* invoke the error handler that is registered against the given
  * status, passing it the provided info on the procs that were
  * affected, plus any additional info provided by the server */
-void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain);
+PMIX_EXPORT void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain);
 
-bool pmix_notify_check_range(pmix_range_trkr_t *rng,
-                             const pmix_proc_t *proc);
+PMIX_EXPORT bool pmix_notify_check_range(pmix_range_trkr_t *rng,
+                                         const pmix_proc_t *proc);
 
-bool pmix_notify_check_affected(pmix_proc_t *interested, size_t ninterested,
-                                pmix_proc_t *affected, size_t naffected);
+PMIX_EXPORT bool pmix_notify_check_affected(pmix_proc_t *interested, size_t ninterested,
+                                            pmix_proc_t *affected, size_t naffected);
 
 
 /* invoke the server event notification handler */
-pmix_status_t pmix_server_notify_client_of_event(pmix_status_t status,
-                                                 const pmix_proc_t *source,
-                                                 pmix_data_range_t range,
-                                                 const pmix_info_t info[], size_t ninfo,
-                                                 pmix_op_cbfunc_t cbfunc, void *cbdata);
+PMIX_EXPORT pmix_status_t pmix_server_notify_client_of_event(pmix_status_t status,
+                                                             const pmix_proc_t *source,
+                                                             pmix_data_range_t range,
+                                                             const pmix_info_t info[], size_t ninfo,
+                                                             pmix_op_cbfunc_t cbfunc, void *cbdata);
 
-void pmix_event_timeout_cb(int fd, short flags, void *arg);
+PMIX_EXPORT void pmix_event_timeout_cb(int fd, short flags, void *arg);
 
 #define PMIX_REPORT_EVENT(e, p, r, f)                                                   \
     do {                                                                                \
@@ -209,7 +215,7 @@ void pmix_event_timeout_cb(int fd, short flags, void *arg);
             }                                                                           \
             /* if this is lost-connection-to-server, then we let it go to */            \
             /* the default event handler - otherwise, we don't */                       \
-            if (PMIX_ERR_LOST_CONNECTION_TO_SERVER != (e) &&                            \
+            if (PMIX_ERR_LOST_CONNECTION != (e) &&                            \
                 PMIX_ERR_UNREACH != (e)) {                                              \
                 ch->ninfo = 1;                                                          \
                 ch->nallocated = 3;                                                     \
@@ -246,8 +252,11 @@ void pmix_event_timeout_cb(int fd, short flags, void *arg);
             ch->info = info;                                                            \
             ch->ninfo = ninfo - 2;                                                      \
             /* reset the timer */                                                       \
-            pmix_event_del(&ch->ev);                                                    \
+            if (ch->timer_active) {                                                     \
+                pmix_event_del(&ch->ev);                                                \
+            }                                                                           \
             PMIX_POST_OBJECT(ch);                                                       \
+            ch->timer_active = true;                                                    \
             pmix_event_add(&ch->ev, &pmix_globals.event_window);                        \
         }                                                                               \
     } while(0)
