@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2017-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
@@ -11,15 +11,15 @@
  *
  * $HEADER$
  */
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
-#include <pmix.h>
-#include <pmix_common.h>
-#include <pmix_server.h>
-#include <pmix_rename.h>
+#include "include/pmix.h"
+#include "include/pmix_common.h"
+#include "include/pmix_server.h"
 
 #include "src/threads/threads.h"
 #include "src/util/error.h"
+#include "src/util/name_fns.h"
 #include "src/util/output.h"
 
 #include "src/mca/bfrops/bfrops.h"
@@ -32,6 +32,10 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
                                             pmix_data_range_t range,
                                             const pmix_info_t info[], size_t ninfo,
                                             pmix_op_cbfunc_t cbfunc, void *cbdata);
+static void progress_local_event_hdlr(pmix_status_t status,
+                                      pmix_info_t *results, size_t nresults,
+                                      pmix_op_cbfunc_t cbfunc, void *thiscbdata,
+                                      void *notification_cbdata);
 
 /* if we are a client, we call this function to notify the server of
  * an event. If we are a server, our host RM will call this function
@@ -51,8 +55,8 @@ PMIX_EXPORT pmix_status_t PMIx_Notify_event(pmix_status_t status,
         return PMIX_ERR_INIT;
     }
 
-    if (PMIX_PROC_IS_SERVER(pmix_globals.mypeer) &&
-        !PMIX_PROC_IS_LAUNCHER(pmix_globals.mypeer)) {
+    if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer) ||
+        PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
         PMIX_RELEASE_THREAD(&pmix_global_lock);
 
         pmix_output_verbose(2, pmix_server_globals.event_output,
@@ -67,11 +71,15 @@ PMIX_EXPORT pmix_status_t PMIx_Notify_event(pmix_status_t status,
         if (PMIX_SUCCESS != rc && PMIX_OPERATION_SUCCEEDED != rc) {
             PMIX_ERROR_LOG(rc);
         }
-        return rc;
+        if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer) &&
+            !PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
+            return rc;
+        }
+        PMIX_ACQUIRE_THREAD(&pmix_global_lock);
     }
 
     /* if we aren't connected, don't attempt to send */
-    if (!pmix_globals.connected) {
+    if (!pmix_globals.connected && PMIX_RANGE_PROC_LOCAL != range) {
         PMIX_RELEASE_THREAD(&pmix_global_lock);
         return PMIX_ERR_UNREACH;
     }
@@ -93,6 +101,7 @@ PMIX_EXPORT pmix_status_t PMIx_Notify_event(pmix_status_t status,
 static void notify_event_cbfunc(struct pmix_peer_t *pr, pmix_ptl_hdr_t *hdr,
                                 pmix_buffer_t *buf, void *cbdata)
 {
+    (void)hdr;
     pmix_status_t rc, ret;
     int32_t cnt = 1;
     pmix_cb_t *cb = (pmix_cb_t*)cbdata;
@@ -110,7 +119,7 @@ static void notify_event_cbfunc(struct pmix_peer_t *pr, pmix_ptl_hdr_t *hdr,
     PMIX_RELEASE(cb);
 }
 
-static pmix_status_t notify_event_cache(pmix_notify_caddy_t *cd)
+pmix_status_t pmix_notify_event_cache(pmix_notify_caddy_t *cd)
 {
     pmix_status_t rc;
     int j;
@@ -219,8 +228,12 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
     /* setup for our own local callbacks */
     chain = PMIX_NEW(pmix_event_chain_t);
     chain->status = status;
-    pmix_strncpy(chain->source.nspace, pmix_globals.myid.nspace, PMIX_MAX_NSLEN);
-    chain->source.rank = pmix_globals.myid.rank;
+    chain->range = range;
+    if (NULL == source) {
+        PMIX_LOAD_PROCID(&chain->source, pmix_globals.myid.nspace, pmix_globals.myid.rank);
+    } else {
+        PMIX_LOAD_PROCID(&chain->source, source->nspace, source->rank);
+    }
     /* we always leave space for event hdlr name and a callback object */
     chain->nallocated = ninfo + 2;
     PMIX_INFO_CREATE(chain->info, chain->nallocated);
@@ -231,14 +244,8 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
      * ourselves should someone later register for it */
     cd = PMIX_NEW(pmix_notify_caddy_t);
     cd->status = status;
-    if (NULL == source) {
-        pmix_strncpy(cd->source.nspace, "UNDEF", PMIX_MAX_NSLEN);
-        cd->source.rank = PMIX_RANK_UNDEF;
-    } else {
-        pmix_strncpy(cd->source.nspace, source->nspace, PMIX_MAX_NSLEN);
-        cd->source.rank = source->rank;
-    }
-    cd->range = range;
+    PMIX_LOAD_PROCID(&cd->source, chain->source.nspace, chain->source.rank);
+    cd->range = chain->range;
     if (0 < chain->ninfo) {
         cd->ninfo = chain->ninfo;
         PMIX_INFO_CREATE(cd->info, cd->ninfo);
@@ -264,12 +271,13 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
         memcpy(cd->affected, chain->affected, cd->naffected * sizeof(pmix_proc_t));
     }
     /* cache it */
-    rc = notify_event_cache(cd);
+    rc = pmix_notify_event_cache(cd);
     if (PMIX_SUCCESS != rc) {
         PMIX_ERROR_LOG(rc);
         PMIX_RELEASE(cd);
         goto cleanup;
     }
+    chain->cached = true;
 
     if (PMIX_RANGE_PROC_LOCAL != range && NULL != msg) {
         /* create a callback object as we need to pass it to the
@@ -293,7 +301,7 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
             PMIX_RELEASE(cb);
             goto cleanup;
         }
-    } else {
+    } else if (NULL != cbfunc) {
         cbfunc(PMIX_SUCCESS, cbdata);
     }
 
@@ -313,19 +321,17 @@ static pmix_status_t notify_server_of_event(pmix_status_t status,
 }
 
 
-static void progress_local_event_hdlr(pmix_status_t status,
-                                      pmix_info_t *results, size_t nresults,
-                                      pmix_op_cbfunc_t cbfunc, void *thiscbdata,
-                                      void *notification_cbdata)
+static void cycle_events(int sd, short args, void *cbdata)
 {
-    /* this may be in the host's thread, so we need to threadshift it
-     * before accessing our internal data */
-
-    pmix_event_chain_t *chain = (pmix_event_chain_t*)notification_cbdata;
+    pmix_event_chain_t *chain = (pmix_event_chain_t*)cbdata;
     size_t n, nsave, cnt;
-    pmix_info_t *newinfo;
     pmix_list_item_t *item;
     pmix_event_hdlr_t *nxt;
+    pmix_info_t *newinfo;
+
+    pmix_output_verbose(2, pmix_client_globals.event_output,
+                        "%s progressing local event",
+                        PMIX_NAME_PRINT(&pmix_globals.myid));
 
     /* aggregate the results per RFC0018 - first search the
      * prior chained results to see if any keys have been NULL'd
@@ -341,7 +347,7 @@ static void progress_local_event_hdlr(pmix_status_t status,
      * the array to make space */
 
     /* add in any new results plus space for the returned status */
-    nsave += nresults + 1;
+    nsave += chain->ninterim + 1;
     /* create the new space */
     PMIX_INFO_CREATE(newinfo, nsave);
     /* transfer over the prior data */
@@ -360,11 +366,11 @@ static void progress_local_event_hdlr(pmix_status_t status,
         pmix_strncpy(newinfo[cnt].key, "UNKNOWN", PMIX_MAX_KEYLEN);
     }
     newinfo[cnt].value.type = PMIX_STATUS;
-    newinfo[cnt].value.data.status = status;
+    newinfo[cnt].value.data.status = chain->status;
     ++cnt;
     /* transfer across the new results */
-    for (n=0; n < nresults; n++) {
-        PMIX_INFO_XFER(&newinfo[cnt], &results[n]);
+    for (n=0; n < chain->ninterim; n++) {
+        PMIX_INFO_XFER(&newinfo[cnt], &chain->interim[n]);
         ++cnt;
     }
     /* release the prior results */
@@ -378,14 +384,27 @@ static void progress_local_event_hdlr(pmix_status_t status,
     chain->ninfo = chain->nallocated - 2;
     PMIX_INFO_DESTRUCT(&chain->info[chain->nallocated-2]);
     PMIX_INFO_DESTRUCT(&chain->info[chain->nallocated-1]);
-
+    // call their interim cbfunc
+    if (NULL != chain->opcbfunc) {
+        chain->opcbfunc(PMIX_SUCCESS, chain->cbdata);
+    }
+    
     /* if the caller indicates that the chain is completed,
      * or we completed the "last" event */
-    if (PMIX_EVENT_ACTION_COMPLETE == status || chain->endchain) {
-        goto complete;
+    if (PMIX_EVENT_ACTION_COMPLETE == chain->status || chain->endchain) {
+        if (PMIX_EVENT_ACTION_COMPLETE == chain->status) {
+            chain->status = PMIX_SUCCESS;
+        }
+        /* we still have to call their final callback */
+        if (NULL != chain->final_cbfunc) {
+            chain->final_cbfunc(chain->status, chain->final_cbdata);
+        }
+        /* maintain acctng */
+        PMIX_RELEASE(chain);
+        return;
     }
-    item = NULL;
 
+    item = NULL;
     /* see if we need to continue, starting with the single code events */
     if (1 == chain->evhdlr->ncodes) {
         /* the last handler was for a single code - see if there are
@@ -587,18 +606,31 @@ static void progress_local_event_hdlr(pmix_status_t status,
         }
     }
 
-  complete:
-    /* we still have to call their final callback */
+    /* if we get here, there was nothing more to do, but
+     * we still have to call their final callback */
     if (NULL != chain->final_cbfunc) {
-        chain->final_cbfunc(PMIX_SUCCESS, chain->final_cbdata);
+        chain->final_cbfunc(chain->status, chain->final_cbdata);
         return;
     }
     /* maintain acctng */
     PMIX_RELEASE(chain);
-    /* let the caller know that we are done with their callback */
-    if (NULL != cbfunc) {
-        cbfunc(PMIX_SUCCESS, thiscbdata);
-    }
+}
+
+static void progress_local_event_hdlr(pmix_status_t status,
+                                      pmix_info_t *results, size_t nresults,
+                                      pmix_op_cbfunc_t cbfunc, void *thiscbdata,
+                                      void *notification_cbdata)
+{
+    /* this may be in the host's thread, so we need to threadshift it
+     * before accessing our internal data */
+
+    pmix_event_chain_t *chain = (pmix_event_chain_t*)notification_cbdata;
+    
+    chain->interim = results;
+    chain->ninterim = nresults;
+    chain->opcbfunc = cbfunc;
+    chain->cbdata = thiscbdata;
+    PMIX_THREADSHIFT(chain, cycle_events);
 }
 
 /* given notification of an event, cycle thru our list of
@@ -620,8 +652,8 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
     bool found;
 
     pmix_output_verbose(2, pmix_client_globals.event_output,
-                        "%s:%d invoke_local_event_hdlr for status %s",
-                        pmix_globals.myid.nspace, pmix_globals.myid.rank,
+                        "%s invoke_local_event_hdlr for status %s",
+                        PMIX_NAME_PRINT(&pmix_globals.myid),
                         PMIx_Error_string(chain->status));
 
     /* sanity check */
@@ -629,6 +661,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
         /* should never happen as space must always be
          * reserved for handler name and callback object*/
         rc = PMIX_ERR_BAD_PARAM;
+        pmix_output_verbose(8, pmix_client_globals.event_output,
+                            "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                            __FILE__, __LINE__);
         goto complete;
     }
 
@@ -636,15 +671,25 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
     if (NULL != chain->targets) {
         found = false;
         for (i=0; i < chain->ntargets; i++) {
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s CHECKING TARGET %s",
+                                PMIX_NAME_PRINT(&pmix_globals.myid),
+                                PMIX_NAME_PRINT(&chain->targets[i]));
             if (PMIX_CHECK_PROCID(&chain->targets[i], &pmix_globals.myid)) {
                 found = true;
                 break;
             }
         }
         if (!found) {
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto complete;
         }
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
 
     /* if we registered a "first" handler, and it fits the given range,
      * then invoke it first */
@@ -656,6 +701,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                        chain->affected, chain->naffected)) {
             /* invoke the handler */
             chain->evhdlr = pmix_globals.events.first;
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto invk;
         } else if (NULL != pmix_globals.events.first->codes) {
             /* need to check if this code is included in the array */
@@ -671,6 +719,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
             if (found && pmix_notify_check_range(&pmix_globals.events.first->rng, &chain->source)) {
                 /* invoke the handler */
                 chain->evhdlr = pmix_globals.events.first;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         } else {
@@ -678,11 +729,17 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
             if (pmix_notify_check_range(&pmix_globals.events.first->rng, &chain->source)) {
                 /* invoke the handler */
                 chain->evhdlr = pmix_globals.events.first;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         }
         /* get here if there is no match, so fall thru */
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
 
     /* cycle thru the single-event registrations first */
     PMIX_LIST_FOREACH(evhdlr, &pmix_globals.events.single_events, pmix_event_hdlr_t) {
@@ -692,6 +749,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                            chain->affected, chain->naffected)) {
                 /* invoke the handler */
                 chain->evhdlr = evhdlr;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         }
@@ -707,11 +767,17 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                                chain->affected, chain->naffected)) {
                     /* invoke the handler */
                     chain->evhdlr = evhdlr;
+                    pmix_output_verbose(8, pmix_client_globals.event_output,
+                                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                        __FILE__, __LINE__);
                     goto invk;
                 }
             }
         }
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
 
     /* if they didn't want it to go to a default handler, then ignore them */
     if (!chain->nondefault) {
@@ -722,6 +788,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
                                            chain->affected, chain->naffected)) {
                 /* invoke the handler */
                 chain->evhdlr = evhdlr;
+                pmix_output_verbose(8, pmix_client_globals.event_output,
+                                    "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                    __FILE__, __LINE__);
                 goto invk;
             }
         }
@@ -737,23 +806,37 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
         if (1 == pmix_globals.events.last->ncodes &&
             pmix_globals.events.last->codes[0] == chain->status) {
             chain->evhdlr = pmix_globals.events.last;
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto invk;
         } else if (NULL != pmix_globals.events.last->codes) {
             /* need to check if this code is included in the array */
             for (i=0; i < pmix_globals.events.last->ncodes; i++) {
                 if (pmix_globals.events.last->codes[i] == chain->status) {
                     chain->evhdlr = pmix_globals.events.last;
+                    pmix_output_verbose(8, pmix_client_globals.event_output,
+                                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                        __FILE__, __LINE__);
                     goto invk;
                 }
             }
         } else {
             /* gets run for all codes */
             chain->evhdlr = pmix_globals.events.last;
+            pmix_output_verbose(8, pmix_client_globals.event_output,
+                                "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                                __FILE__, __LINE__);
             goto invk;
         }
     }
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+
+                        __FILE__, __LINE__);
 
     /* if we got here, then nothing was found */
+    rc = PMIX_ERR_NOT_FOUND;
 
   complete:
     /* we still have to call their final callback */
@@ -766,6 +849,9 @@ void pmix_invoke_local_event_hdlr(pmix_event_chain_t *chain)
 
 
   invk:
+    pmix_output_verbose(8, pmix_client_globals.event_output,
+                        "%s %s:%d", PMIX_NAME_PRINT(&pmix_globals.myid),
+                        __FILE__, __LINE__);
     /* start with the chain holding only the given info */
     chain->ninfo = chain->nallocated - 2;
 
@@ -806,6 +892,8 @@ static void local_cbfunc(pmix_status_t status, void *cbdata)
 
 static void _notify_client_event(int sd, short args, void *cbdata)
 {
+    (void)sd;
+    (void)args;
     pmix_notify_caddy_t *cd = (pmix_notify_caddy_t*)cbdata;
     pmix_regevents_info_t *reginfoptr;
     pmix_peer_events_info_t *pr;
@@ -849,7 +937,7 @@ static void _notify_client_event(int sd, short args, void *cbdata)
          * the message until all local procs have received it, or it ages to
          * the point where it gets pushed out by more recent events */
         PMIX_RETAIN(cd);
-        rc = notify_event_cache(cd);
+        rc = pmix_notify_event_cache(cd);
         if (PMIX_SUCCESS != rc) {
             PMIX_ERROR_LOG(rc);
         }
@@ -859,14 +947,30 @@ static void _notify_client_event(int sd, short args, void *cbdata)
      * against our registrations */
     chain = PMIX_NEW(pmix_event_chain_t);
     chain->status = cd->status;
-    pmix_strncpy(chain->source.nspace, cd->source.nspace, PMIX_MAX_NSLEN);
-    chain->source.rank = cd->source.rank;
+    if (holdcd) {
+        chain->cached = true;
+    }
+    PMIX_LOAD_PROCID(&chain->source, cd->source.nspace, cd->source.rank);
     /* we always leave space for a callback object and
      * the evhandler name. */
     chain->nallocated = cd->ninfo + 2;
     PMIX_INFO_CREATE(chain->info, chain->nallocated);
     /* prep the chain for processing */
     pmix_prep_event_chain(chain, cd->info, cd->ninfo, true);
+    /* if we are a tool, we send it to our clients regardless
+     * of the range - we assume that if we are in range, then
+     * so are any clients attached to us
+     *
+     * if we are a server but not a tool, then check the range
+     * before sending it to our client */
+    if (PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
+        /* remove the target designations */
+        if (NULL != chain->targets) {
+            free(chain->targets);
+            chain->targets = NULL;
+            chain->ntargets = 0;
+        }
+    }
 
     /* copy setup to the cd object */
     cd->nondefault = chain->nondefault;
@@ -883,7 +987,7 @@ static void _notify_client_event(int sd, short args, void *cbdata)
             } else {
                 /* look up the nspace for this proc */
                 nptr = NULL;
-                PMIX_LIST_FOREACH(tmp, &pmix_server_globals.nspaces, pmix_namespace_t) {
+                PMIX_LIST_FOREACH(tmp, &pmix_globals.nspaces, pmix_namespace_t) {
                     if (PMIX_CHECK_NSPACE(tmp->nspace, cd->targets[n].nspace)) {
                         nptr = tmp;
                         break;
@@ -930,6 +1034,49 @@ static void _notify_client_event(int sd, short args, void *cbdata)
         return;
     }
 
+    /* check to see if this is a group_complete notification
+     * indicating that a group has asynchronously been formed.
+     * If it is, then we need to track the group */
+    if (PMIX_GROUP_CONSTRUCT_COMPLETE == cd->status) {
+        char *grpid = NULL;
+        pmix_group_t *grp;
+        /* must include the group id */
+        for (n=0; n < cd->ninfo; n++) {
+            if (PMIX_CHECK_KEY(&cd->info[n], PMIX_GROUP_ID)) {
+                grpid = cd->info[n].value.data.string;
+                break;
+            }
+        }
+        if (NULL == grpid) {
+            /* failed to provide the ID */
+            PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
+            /* notify the caller */
+            if (NULL != cd->cbfunc) {
+                cd->cbfunc(PMIX_ERR_BAD_PARAM, cd->cbdata);
+            }
+            PMIX_RELEASE(cd);
+            PMIX_RELEASE(chain);
+            return;
+        }
+        /* must include members */
+        if (NULL == cd->targets || 0 == cd->ntargets) {
+            PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
+            /* notify the caller */
+            if (NULL != cd->cbfunc) {
+                cd->cbfunc(PMIX_ERR_BAD_PARAM, cd->cbdata);
+            }
+            PMIX_RELEASE(cd);
+            PMIX_RELEASE(chain);
+            return;
+        }
+        grp = PMIX_NEW(pmix_group_t);
+        grp->grpid = strdup(grpid);
+        grp->nmbrs = cd->ntargets;
+        PMIX_PROC_CREATE(grp->members, grp->nmbrs);
+        memcpy(grp->members, cd->targets, cd->ntargets * sizeof(pmix_proc_t));
+        pmix_list_append(&pmix_server_globals.groups, &grp->super);
+    }
+
     holdcd = false;
     if (PMIX_RANGE_PROC_LOCAL != cd->range) {
         PMIX_CONSTRUCT(&trk, pmix_list_t);
@@ -964,27 +1111,19 @@ static void _notify_client_event(int sd, short args, void *cbdata)
                                                     pr->affected, pr->naffected)) {
                         continue;
                     }
-                    /* check the range */
-                    if (NULL == cd->targets) {
-                        rngtrk.procs = &cd->source;
-                        rngtrk.nprocs = 1;
-                    } else {
-                        rngtrk.procs = cd->targets;
-                        rngtrk.nprocs = cd->ntargets;
-                    }
-                    rngtrk.range = cd->range;
-                    PMIX_LOAD_PROCID(&proc, pr->peer->info->pname.nspace, pr->peer->info->pname.rank);
-                    if (!pmix_notify_check_range(&rngtrk, &proc)) {
-                        continue;
-                    }
-                    if (NULL != cd->targets) {
-                        /* track the number of targets we have left to notify */
-                        --cd->nleft;
-                        /* if the event was cached and this is the last one,
-                         * then evict this event from the cache */
-                        if (0 == cd->nleft) {
-                            pmix_hotel_checkout(&pmix_globals.notifications, cd->room);
-                            PMIX_RELEASE(cd);
+                    if (!PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
+                        /* check the range, if given */
+                        if (NULL == cd->targets) {
+                            rngtrk.procs = &cd->source;
+                            rngtrk.nprocs = 1;
+                        } else {
+                            rngtrk.procs = cd->targets;
+                            rngtrk.nprocs = cd->ntargets;
+                        }
+                        rngtrk.range = cd->range;
+                        PMIX_LOAD_PROCID(&proc, pr->peer->info->pname.nspace, pr->peer->info->pname.rank);
+                        if (!pmix_notify_check_range(&rngtrk, &proc)) {
+                            continue;
                         }
                     }
                     pmix_output_verbose(2, pmix_server_globals.event_output,
@@ -1044,6 +1183,17 @@ static void _notify_client_event(int sd, short args, void *cbdata)
                     if (PMIX_SUCCESS != rc) {
                         PMIX_RELEASE(bfr);
                     }
+                    if (NULL != cd->targets && 0 < cd->nleft) {
+                        /* track the number of targets we have left to notify */
+                        --cd->nleft;
+                        /* if the event was cached and this is the last one,
+                         * then evict this event from the cache */
+                        if (0 == cd->nleft) {
+                            pmix_hotel_checkout(&pmix_globals.notifications, cd->room);
+                            holdcd = false;
+                            break;
+                        }
+                    }
                 }
             }
         }
@@ -1091,26 +1241,15 @@ pmix_status_t pmix_server_notify_client_of_event(pmix_status_t status,
     size_t n;
 
     pmix_output_verbose(2, pmix_server_globals.event_output,
-                        "pmix_server: notify client of event %s",
-                        PMIx_Error_string(status));
-
-    if (NULL != info) {
-        for (n=0; n < ninfo; n++) {
-            if (PMIX_CHECK_KEY(&info[n], PMIX_EVENT_PROXY) &&
-                PMIX_CHECK_PROCID(info[n].value.data.proc, &pmix_globals.myid)) {
-                return PMIX_OPERATION_SUCCEEDED;
-            }
-        }
-    }
+                        "pmix_server: notify client of event %s range %s",
+                        PMIx_Error_string(status), PMIx_Data_range_string(range));
 
     cd = PMIX_NEW(pmix_notify_caddy_t);
     cd->status = status;
     if (NULL == source) {
-        pmix_strncpy(cd->source.nspace, "UNDEF", PMIX_MAX_NSLEN);
-        cd->source.rank = PMIX_RANK_UNDEF;
+        PMIX_LOAD_PROCID(&cd->source, "UNDEF", PMIX_RANK_UNDEF);
     } else {
-        pmix_strncpy(cd->source.nspace, source->nspace, PMIX_MAX_NSLEN);
-        cd->source.rank = source->rank;
+        PMIX_LOAD_PROCID(&cd->source, source->nspace, source->rank);
     }
     cd->range = range;
     /* have to copy the info to preserve it for future when cached */
@@ -1211,6 +1350,8 @@ bool pmix_notify_check_affected(pmix_proc_t *interested, size_t ninterested,
 
 void pmix_event_timeout_cb(int fd, short flags, void *arg)
 {
+    (void)fd;
+    (void)flags;
     pmix_event_chain_t *ch = (pmix_event_chain_t*)arg;
 
     /* need to acquire the object from its originating thread */
@@ -1221,9 +1362,12 @@ void pmix_event_timeout_cb(int fd, short flags, void *arg)
     /* remove it from the list */
     pmix_list_remove_item(&pmix_globals.cached_events, &ch->super);
 
+    /* protect the chain */
+    PMIX_RETAIN(ch);
+
     /* process this event thru the regular channels */
-    if (PMIX_PROC_IS_SERVER(pmix_globals.mypeer) &&
-        !PMIX_PROC_IS_LAUNCHER(pmix_globals.mypeer)) {
+    if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer) &&
+        !PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer)) {
         pmix_server_notify_client_of_event(ch->status, &ch->source,
                                            ch->range, ch->info, ch->ninfo,
                                            ch->final_cbfunc, ch->final_cbdata);
@@ -1373,6 +1517,7 @@ static void chcon(pmix_event_chain_t *p)
     p->source.rank = PMIX_RANK_UNDEF;
     p->nondefault = false;
     p->endchain = false;
+    p->cached = false;
     p->targets = NULL;
     p->ntargets = 0;
     p->range = PMIX_RANGE_UNDEF;
@@ -1383,7 +1528,11 @@ static void chcon(pmix_event_chain_t *p)
     p->nallocated = 0;
     p->results = NULL;
     p->nresults = 0;
+    p->interim = NULL;
+    p->ninterim = 0;
     p->evhdlr = NULL;
+    p->opcbfunc = NULL;
+    p->cbdata = NULL;
     p->final_cbfunc = NULL;
     p->final_cbdata = NULL;
 }
