@@ -1,12 +1,13 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
- * Copyright (c) 2014-2017 Research Organization for Information Science
- * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2014-2015 Artem Y. Polyakov <artpol84@gmail.com>.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2019      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -16,11 +17,12 @@
 
 /* THIS FILE IS INCLUDED SOLELY TO INSTANTIATE AND INIT/FINALIZE THE GLOBAL CLASSES */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
-#include <src/include/types.h>
-#include <src/include/pmix_stdint.h>
-#include <src/include/pmix_socket_errno.h>
+#include "include/pmix_common.h"
+#include "src/include/types.h"
+#include "src/include/pmix_stdint.h"
+#include "src/include/pmix_socket_errno.h"
 
 #include "src/include/pmix_globals.h"
 
@@ -43,7 +45,7 @@
 #include <dirent.h>
 #endif  /* HAVE_DIRENT_H */
 
-#include <pmix_common.h>
+#include "include/pmix_common.h"
 
 #include "src/mca/bfrops/bfrops_types.h"
 #include "src/class/pmix_hash_table.h"
@@ -66,6 +68,14 @@ PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_namelist_t,
                                 pmix_list_item_t,
                                 NULL, NULL);
 
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_proclist_t,
+                                pmix_list_item_t,
+                                NULL, NULL);
+
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_topo_obj_t,
+                                pmix_object_t,
+                                NULL, NULL);
+
 static void cfcon(pmix_cleanup_file_t *p)
 {
     p->path = NULL;
@@ -100,7 +110,7 @@ static void nscon(pmix_namespace_t *p)
 {
     p->nspace = NULL;
     p->nprocs = 0;
-    p->nlocalprocs = 0;
+    p->nlocalprocs = SIZE_MAX;
     p->all_registered = false;
     p->version_stored = false;
     p->jobbkt = NULL;
@@ -170,7 +180,11 @@ PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_rank_info_t,
 
 static void pcon(pmix_peer_t *p)
 {
-    p->proc_type = PMIX_PROC_UNDEF;
+    p->proc_type.type = PMIX_PROC_UNDEF;
+    p->proc_type.major = PMIX_MAJOR_WILDCARD;
+    p->proc_type.minor = PMIX_MINOR_WILDCARD;
+    p->proc_type.release = PMIX_RELEASE_WILDCARD;
+    p->proc_type.flag = 0;
     p->protocol = PMIX_PROTOCOL_UNDEF;
     p->finalized = false;
     p->info = NULL;
@@ -228,22 +242,25 @@ PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_peer_t,
 
 static void iofreqcon(pmix_iof_req_t *p)
 {
-    p->peer = NULL;
-    memset(&p->pname, 0, sizeof(pmix_name_t));
+    p->requestor = NULL;
+    p->local_id = 0;
+    p->remote_id = 0;
+    p->procs = NULL;
+    p->nprocs = 0;
     p->channels = PMIX_FWD_NO_CHANNELS;
     p->cbfunc = NULL;
 }
 static void iofreqdes(pmix_iof_req_t *p)
 {
-    if (NULL != p->peer) {
-        PMIX_RELEASE(p->peer);
+    if (NULL != p->requestor) {
+        PMIX_RELEASE(p->requestor);
     }
-    if (NULL != p->pname.nspace) {
-        free(p->pname.nspace);
+    if (0 < p->nprocs) {
+        PMIX_PROC_FREE(p->procs, p->nprocs);
     }
 }
 PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_iof_req_t,
-                                pmix_list_item_t,
+                                pmix_object_t,
                                 iofreqcon, iofreqdes);
 
 
@@ -252,6 +269,7 @@ static void scon(pmix_shift_caddy_t *p)
     PMIX_CONSTRUCT_LOCK(&p->lock);
     p->codes = NULL;
     p->ncodes = 0;
+    p->peer = NULL;
     p->pname.nspace = NULL;
     p->pname.rank = PMIX_RANK_UNDEF;
     p->data = NULL;
@@ -275,6 +293,9 @@ static void scon(pmix_shift_caddy_t *p)
 static void scdes(pmix_shift_caddy_t *p)
 {
     PMIX_DESTRUCT_LOCK(&p->lock);
+    if (NULL != p->peer) {
+        PMIX_RELEASE(p->peer);
+    }
     if (NULL != p->pname.nspace) {
         free(p->pname.nspace);
     }
@@ -302,11 +323,13 @@ static void cbcon(pmix_cb_t *p)
     p->nprocs = 0;
     p->info = NULL;
     p->ninfo = 0;
+    p->dist = NULL;
+    p->infocopy = false;
     p->nvals = 0;
     PMIX_CONSTRUCT(&p->kvs, pmix_list_t);
     p->copy = false;
     p->timer_running = false;
-    p->level = PMIX_LEVEL_UNDEF;
+    p->fabric = NULL;
 }
 static void cbdes(pmix_cb_t *p)
 {
@@ -317,6 +340,12 @@ static void cbdes(pmix_cb_t *p)
         free(p->pname.nspace);
     }
     PMIX_DESTRUCT(&p->data);
+    if (p->infocopy) {
+        PMIX_INFO_FREE(p->info, p->ninfo);
+    }
+    if (NULL != p->dist) {
+        PMIX_DEVICE_DIST_FREE(p->dist, p->nvals);
+    }
     PMIX_LIST_DESTRUCT(&p->kvs);
 }
 PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_cb_t,
@@ -327,6 +356,26 @@ PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_info_caddy_t,
                                 pmix_list_item_t,
                                 NULL, NULL);
 
+static void ifcon(pmix_infolist_t *p)
+{
+    PMIX_INFO_CONSTRUCT(&p->info);
+}
+static void ifdes(pmix_infolist_t *p)
+{
+    PMIX_INFO_DESTRUCT(&p->info);
+}
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_infolist_t,
+                                pmix_list_item_t,
+                                ifcon, ifdes);
+
+static void qlcon(pmix_querylist_t *p)
+{
+    PMIX_QUERY_CONSTRUCT(&p->query);
+}
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_querylist_t,
+                                pmix_list_item_t,
+                                qlcon, NULL);
+
 static void qcon(pmix_query_caddy_t *p)
 {
     PMIX_CONSTRUCT_LOCK(&p->lock);
@@ -337,13 +386,16 @@ static void qcon(pmix_query_caddy_t *p)
     p->info = NULL;
     p->ninfo = 0;
     PMIX_BYTE_OBJECT_CONSTRUCT(&p->bo);
+    PMIX_CONSTRUCT(&p->results, pmix_list_t);
+    p->nreplies = 0;
+    p->nrequests = 0;
     p->cbfunc = NULL;
     p->valcbfunc = NULL;
     p->cbdata = NULL;
     p->relcbfunc = NULL;
     p->credcbfunc = NULL;
     p->validcbfunc = NULL;
-    PMIX_CONSTRUCT(&p->results, pmix_list_t);
+    p->stqcbfunc = NULL;
 }
 static void qdes(pmix_query_caddy_t *p)
 {
@@ -352,12 +404,52 @@ static void qdes(pmix_query_caddy_t *p)
     PMIX_PROC_FREE(p->targets, p->ntargets);
     PMIX_INFO_FREE(p->info, p->ninfo);
     PMIX_LIST_DESTRUCT(&p->results);
-    PMIX_QUERY_FREE(p->queries, p->nqueries);
 }
 PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_query_caddy_t,
                                 pmix_object_t,
                                 qcon, qdes);
 
+static void ncon(pmix_notify_caddy_t *p)
+{
+    PMIX_CONSTRUCT_LOCK(&p->lock);
+#if defined(__linux__) && PMIX_HAVE_CLOCK_GETTIME
+    struct timespec tp;
+    (void) clock_gettime(CLOCK_MONOTONIC, &tp);
+    p->ts = tp.tv_sec;
+#else
+    /* Fall back to gettimeofday() if we have nothing else */
+    struct timeval tv;
+    gettimeofday(&tv, NULL);
+    p->ts = tv.tv_sec;
+#endif
+    p->room = -1;
+    memset(p->source.nspace, 0, PMIX_MAX_NSLEN+1);
+    p->source.rank = PMIX_RANK_UNDEF;
+    p->range = PMIX_RANGE_UNDEF;
+    p->targets = NULL;
+    p->ntargets = 0;
+    p->nleft = SIZE_MAX;
+    p->affected = NULL;
+    p->naffected = 0;
+    p->nondefault = false;
+    p->info = NULL;
+    p->ninfo = 0;
+}
+static void ndes(pmix_notify_caddy_t *p)
+{
+    PMIX_DESTRUCT_LOCK(&p->lock);
+    if (NULL != p->info) {
+        PMIX_INFO_FREE(p->info, p->ninfo);
+    }
+    PMIX_PROC_FREE(p->affected, p->naffected);
+    if (NULL != p->targets) {
+        free(p->targets);
+    }
+}
+PMIX_CLASS_INSTANCE(pmix_notify_caddy_t,
+                    pmix_object_t,
+                    ncon, ndes);
+
 void pmix_execute_epilog(pmix_epilog_t *epi)
 {
     pmix_cleanup_file_t *cf, *cfnext;
@@ -374,6 +466,7 @@ void pmix_execute_epilog(pmix_epilog_t *epi)
          * some minimum level of protection */
         tmp = pmix_argv_split(cf->path, ',');
         for (n=0; NULL != tmp[n]; n++) {
+            /* coverity[toctou] */
             rc = stat(tmp[n], &statbuf);
             if (0 != rc) {
                 pmix_output_verbose(10, pmix_globals.debug_output,
@@ -407,6 +500,7 @@ void pmix_execute_epilog(pmix_epilog_t *epi)
          * some minimum level of protection */
         tmp = pmix_argv_split(cd->path, ',');
         for (n=0; NULL != tmp[n]; n++) {
+            /* coverity[toctou] */
             rc = stat(tmp[n], &statbuf);
             if (0 != rc) {
                 pmix_output_verbose(10, pmix_globals.debug_output,
@@ -492,6 +586,7 @@ static void dirpath_destroy(char *path, pmix_cleanup_dir_t *cd, pmix_epilog_t *e
         /* Check to see if it is a directory */
         is_dir = false;
 
+        /* coverity[toctou] */
         rc = stat(filenm, &buf);
         if (0 > rc) {
             /* Handle a race condition. filenm might have been deleted by an
