@@ -10,7 +10,11 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2019      Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
+ * Copyright (c) 2019      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -21,9 +25,8 @@
 #ifndef PMIX_GLOBALS_H
 #define PMIX_GLOBALS_H
 
-#include <src/include/pmix_config.h>
-
-#include <src/include/types.h>
+#include "src/include/pmix_config.h"
+#include "src/include/types.h"
 
 #include <unistd.h>
 #ifdef HAVE_SYS_TYPES_H
@@ -31,8 +34,9 @@
 #endif
 #include PMIX_EVENT_HEADER
 
-#include <pmix.h>
-#include <pmix_common.h>
+#include "include/pmix.h"
+#include "include/pmix_common.h"
+#include "include/pmix_tool.h"
 
 #include "src/class/pmix_hash_table.h"
 #include "src/class/pmix_list.h"
@@ -77,35 +81,62 @@ typedef struct {
 } pmix_namelist_t;
 PMIX_CLASS_DECLARATION(pmix_namelist_t);
 
+/* define a struct for holding entries in the
+ * dictionary of attributes */
+typedef struct {
+    char *name;
+    char *string;
+    pmix_data_type_t type;
+    char **description;
+} pmix_regattr_input_t;
+
+/* define a struct for passing topology objects */
+typedef struct {
+    pmix_object_t super;
+    char *source;
+    void *object;
+} pmix_topo_obj_t;
+PMIX_CLASS_DECLARATION(pmix_topo_obj_t);
+
 /* define a command type for communicating to the
  * pmix server */
 typedef uint8_t pmix_cmd_t;
 
 /* define some commands */
-#define PMIX_REQ_CMD                 0
-#define PMIX_ABORT_CMD               1
-#define PMIX_COMMIT_CMD              2
-#define PMIX_FENCENB_CMD             3
-#define PMIX_GETNB_CMD               4
-#define PMIX_FINALIZE_CMD            5
-#define PMIX_PUBLISHNB_CMD           6
-#define PMIX_LOOKUPNB_CMD            7
-#define PMIX_UNPUBLISHNB_CMD         8
-#define PMIX_SPAWNNB_CMD             9
-#define PMIX_CONNECTNB_CMD          10
-#define PMIX_DISCONNECTNB_CMD       11
-#define PMIX_NOTIFY_CMD             12
-#define PMIX_REGEVENTS_CMD          13
-#define PMIX_DEREGEVENTS_CMD        14
-#define PMIX_QUERY_CMD              15
-#define PMIX_LOG_CMD                16
-#define PMIX_ALLOC_CMD              17
-#define PMIX_JOB_CONTROL_CMD        18
-#define PMIX_MONITOR_CMD            19
-#define PMIX_GET_CREDENTIAL_CMD     20
-#define PMIX_VALIDATE_CRED_CMD      21
-#define PMIX_IOF_PULL_CMD           22
-#define PMIX_IOF_PUSH_CMD           23
+#define PMIX_REQ_CMD                         0
+#define PMIX_ABORT_CMD                       1
+#define PMIX_COMMIT_CMD                      2
+#define PMIX_FENCENB_CMD                     3
+#define PMIX_GETNB_CMD                       4
+#define PMIX_FINALIZE_CMD                    5
+#define PMIX_PUBLISHNB_CMD                   6
+#define PMIX_LOOKUPNB_CMD                    7
+#define PMIX_UNPUBLISHNB_CMD                 8
+#define PMIX_SPAWNNB_CMD                     9
+#define PMIX_CONNECTNB_CMD                  10
+#define PMIX_DISCONNECTNB_CMD               11
+#define PMIX_NOTIFY_CMD                     12
+#define PMIX_REGEVENTS_CMD                  13
+#define PMIX_DEREGEVENTS_CMD                14
+#define PMIX_QUERY_CMD                      15
+#define PMIX_LOG_CMD                        16
+#define PMIX_ALLOC_CMD                      17
+#define PMIX_JOB_CONTROL_CMD                18
+#define PMIX_MONITOR_CMD                    19
+#define PMIX_GET_CREDENTIAL_CMD             20
+#define PMIX_VALIDATE_CRED_CMD              21
+#define PMIX_IOF_PULL_CMD                   22
+#define PMIX_IOF_PUSH_CMD                   23
+#define PMIX_GROUP_CONSTRUCT_CMD            24
+#define PMIX_GROUP_JOIN_CMD                 25
+#define PMIX_GROUP_INVITE_CMD               26
+#define PMIX_GROUP_LEAVE_CMD                27
+#define PMIX_GROUP_DESTRUCT_CMD             28
+#define PMIX_IOF_DEREG_CMD                  29
+#define PMIX_FABRIC_REGISTER_CMD            30
+#define PMIX_FABRIC_UPDATE_CMD              31
+#define PMIX_FABRIC_COMPUTE_DISTANCES_CMD   32
+
 
 /* provide a "pretty-print" function for cmds */
 const char* pmix_command_string(pmix_cmd_t cmd);
@@ -122,16 +153,6 @@ typedef enum {
     PMIX_COLLECT_MAX
 } pmix_collect_t;
 
-/* define a set of flags indicating the level
- * of information being stored/requested */
-typedef enum {
-    PMIX_LEVEL_UNDEF,
-    PMIX_LEVEL_SESSION,
-    PMIX_LEVEL_JOB,
-    PMIX_LEVEL_APP,
-    PMIX_LEVEL_NODE
-} pmix_level_t;
-
 /****    PEER STRUCTURES    ****/
 
 /* clients can only talk to their server, and servers are
@@ -143,7 +164,6 @@ typedef struct pmix_personality_t {
     pmix_bfrop_buffer_type_t type;
     pmix_bfrops_module_t *bfrops;
     pmix_psec_module_t *psec;
-    pmix_ptl_module_t *ptl;
     pmix_gds_base_module_t *gds;
 } pmix_personality_t;
 
@@ -216,7 +236,7 @@ PMIX_CLASS_DECLARATION(pmix_rank_info_t);
 
 
 /* define a very simple caddy for dealing with pmix_info_t
- * objects when transferring portions of arrays */
+ * and pmix_query_t objects when transferring portions of arrays */
 typedef struct {
     pmix_list_item_t super;
     pmix_info_t *info;
@@ -224,6 +244,23 @@ typedef struct {
 } pmix_info_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_info_caddy_t);
 
+typedef struct {
+    pmix_list_item_t super;
+    pmix_info_t info;
+} pmix_infolist_t;
+PMIX_CLASS_DECLARATION(pmix_infolist_t);
+
+typedef struct {
+    pmix_list_item_t super;
+    pmix_query_t query;
+} pmix_querylist_t;
+PMIX_CLASS_DECLARATION(pmix_querylist_t);
+
+typedef struct {
+    pmix_list_item_t super;
+    pmix_proc_t proc;
+} pmix_proclist_t;
+PMIX_CLASS_DECLARATION(pmix_proclist_t);
 
 /* object for tracking peers - each peer can have multiple
  * connections. This can occur if the initial app executes
@@ -256,14 +293,21 @@ PMIX_CLASS_DECLARATION(pmix_peer_t);
 
 /* tracker for IOF requests */
 typedef struct {
-    pmix_list_item_t super;
-    pmix_peer_t *peer;
-    pmix_name_t pname;
+    pmix_object_t super;
+    pmix_peer_t *requestor;
+    size_t local_id;
+    size_t remote_id;
+    pmix_proc_t *procs;
+    size_t nprocs;
     pmix_iof_channel_t channels;
     pmix_iof_cbfunc_t cbfunc;
 } pmix_iof_req_t;
 PMIX_CLASS_DECLARATION(pmix_iof_req_t);
 
+typedef void (*pmix_pstrg_query_cbfunc_t)(pmix_status_t status,
+                                          pmix_list_t *results,
+                                          void *cbdata);
+
 
 /* caddy for query requests */
 typedef struct {
@@ -277,13 +321,16 @@ typedef struct {
     size_t ntargets;
     pmix_info_t *info;
     size_t ninfo;
-    pmix_byte_object_t bo;
     pmix_list_t results;
+    size_t nreplies;
+    size_t nrequests;
+    pmix_byte_object_t bo;
     pmix_info_cbfunc_t cbfunc;
     pmix_value_cbfunc_t valcbfunc;
     pmix_release_cbfunc_t relcbfunc;
     pmix_credential_cbfunc_t credcbfunc;
     pmix_validation_cbfunc_t validcbfunc;
+    pmix_pstrg_query_cbfunc_t stqcbfunc;
     void *cbdata;
 } pmix_query_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_query_caddy_t);
@@ -302,6 +349,7 @@ typedef struct {
     bool hybrid;                    // true if participating procs are from more than one nspace
     pmix_proc_t *pcs;               // copy of the original array of participants
     size_t   npcs;                  // number of procs in the array
+    pmix_list_t nslist;             // unique nspace list of participants
     pmix_lock_t lock;               // flag for waiting for completion
     bool def_complete;              // all local procs have been registered and the trk definition is complete
     pmix_list_t local_cbs;          // list of pmix_server_caddy_t for sending result to the local participants
@@ -329,6 +377,8 @@ typedef struct {
     pmix_server_trkr_t *trk;
     pmix_ptl_hdr_t hdr;
     pmix_peer_t *peer;
+    pmix_info_t *info;
+    size_t ninfo;
 } pmix_server_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_server_caddy_t);
 
@@ -342,6 +392,7 @@ PMIX_CLASS_DECLARATION(pmix_server_caddy_t);
     pmix_status_t *codes;
     size_t ncodes;
     pmix_name_t pname;
+    pmix_peer_t *peer;
     const char *data;
     size_t ndata;
     const char *key;
@@ -384,6 +435,8 @@ typedef struct {
         pmix_lookup_cbfunc_t lookupfn;
         pmix_spawn_cbfunc_t spawnfn;
         pmix_hdlr_reg_cbfunc_t hdlrregfn;
+        pmix_info_cbfunc_t infofn;
+        pmix_device_dist_cbfunc_t distfn;
     } cbfunc;
     size_t errhandler_ref;
     void *cbdata;
@@ -395,11 +448,13 @@ typedef struct {
     size_t nprocs;
     pmix_info_t *info;
     size_t ninfo;
+    pmix_device_distance_t *dist;
+    bool infocopy;
     size_t nvals;
     pmix_list_t kvs;
     bool copy;
     bool timer_running;
-    pmix_level_t level;
+    pmix_fabric_t *fabric;
 } pmix_cb_t;
 PMIX_CLASS_DECLARATION(pmix_cb_t);
 
@@ -411,6 +466,17 @@ PMIX_CLASS_DECLARATION(pmix_cb_t);
     pmix_event_active(&((r)->ev), EV_WRITE, 1);             \
 } while (0)
 
+#define PMIX_THREADSHIFT_DELAY(r, c, t)                     \
+ do {                                                       \
+    struct timeval _tv = {0, 0};                            \
+    pmix_event_evtimer_set(pmix_globals.evbase, &(r)->ev,   \
+                               (c), (r));                   \
+    _tv.tv_sec = (int)(t);                                  \
+    _tv.tv_usec = ((t) - _tv.tv_sec) * 1000000.0;           \
+    PMIX_POST_OBJECT((r));                                  \
+    pmix_event_evtimer_add(&(r)->ev, &_tv);                 \
+} while (0)
+
 
 typedef struct {
     pmix_object_t super;
@@ -449,7 +515,9 @@ typedef struct {
      * handlers can look at it */
     pmix_info_t *info;
     size_t ninfo;
+    /* allow for a buffer to be carried across internal processing */
     pmix_buffer_t *buf;
+    /* the final callback to be executed upon completion of the event */
     pmix_op_cbfunc_t cbfunc;
     void *cbdata;
 } pmix_notify_caddy_t;
@@ -467,17 +535,18 @@ typedef struct {
     uid_t uid;                          // my effective uid
     gid_t gid;                          // my effective gid
     char *hostname;                     // my hostname
+    uint32_t appnum;                    // my appnum
+    pid_t pid;                          // my local pid
     uint32_t nodeid;                    // my nodeid, if given
     int pindex;
     pmix_event_base_t *evbase;
-    bool external_evbase;
     int debug_output;
     pmix_events_t events;               // my event handler registrations.
     bool connected;
     bool commits_pending;
     struct timeval event_window;
     pmix_list_t cached_events;          // events waiting in the window prior to processing
-    pmix_list_t iof_requests;           // list of pmix_iof_req_t IOF requests
+    pmix_pointer_array_t iof_requests;  // array of pmix_iof_req_t IOF requests
     int max_events;                     // size of the notifications hotel
     int event_eviction_time;            // max time to cache notifications
     pmix_hotel_t notifications;         // hotel of pending notifications
@@ -489,19 +558,97 @@ typedef struct {
      * look them up */
     pmix_gds_base_module_t *mygds;
     /* IOF controls */
+    bool pushstdin;
+    pmix_list_t stdin_targets;          // list of pmix_namelist_t
     bool tag_output;
     bool xml_output;
     bool timestamp_output;
     size_t output_limit;
     pmix_list_t nspaces;
+    pmix_topology_t topology;
+    bool external_topology;
+    bool external_progress;
 } pmix_globals_t;
 
 /* provide access to a function to cleanup epilogs */
 PMIX_EXPORT void pmix_execute_epilog(pmix_epilog_t *ep);
 
+PMIX_EXPORT pmix_status_t pmix_notify_event_cache(pmix_notify_caddy_t *cd);
+
 PMIX_EXPORT extern pmix_globals_t pmix_globals;
 PMIX_EXPORT extern pmix_lock_t pmix_global_lock;
 
+static inline bool pmix_check_node_info(const char* key)
+{
+    char *keys[] = {
+    	PMIX_HOSTNAME,
+    	PMIX_HOSTNAME_ALIASES,
+    	PMIX_NODEID,
+    	PMIX_AVAIL_PHYS_MEMORY,
+        PMIX_LOCAL_PEERS,
+        PMIX_LOCAL_PROCS,
+        PMIX_LOCAL_CPUSETS,
+        PMIX_LOCAL_SIZE,
+        PMIX_NODE_SIZE,
+        PMIX_LOCALLDR,
+        NULL
+    };
+    size_t n;
+
+    for (n=0; NULL != keys[n]; n++) {
+        if (0 == strncmp(key, keys[n], PMIX_MAX_KEYLEN)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+static inline bool pmix_check_app_info(const char* key)
+{
+    char *keys[] = {
+        PMIX_APP_SIZE,
+        PMIX_APPLDR,
+        PMIX_APP_ARGV,
+        PMIX_WDIR,
+        PMIX_PSET_NAME,
+        PMIX_APP_MAP_TYPE,
+        PMIX_APP_MAP_REGEX,
+        NULL
+    };
+    size_t n;
+
+    for (n=0; NULL != keys[n]; n++) {
+        if (0 == strncmp(key, keys[n], PMIX_MAX_KEYLEN)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+static inline bool pmix_check_session_info(const char* key)
+{
+    char *keys[] = {
+        PMIX_SESSION_ID,
+        PMIX_CLUSTER_ID,
+        PMIX_UNIV_SIZE,
+        PMIX_TMPDIR,
+        PMIX_TDIR_RMCLEAN,
+        PMIX_HOSTNAME_KEEP_FQDN,
+        PMIX_RM_NAME,
+        PMIX_RM_VERSION,
+        NULL
+    };
+    size_t n;
+
+    for (n=0; NULL != keys[n]; n++) {
+        if (0 == strncmp(key, keys[n], PMIX_MAX_KEYLEN)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+
 END_C_DECLS
 
 #endif /* PMIX_GLOBALS_H */
