@@ -10,12 +10,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2019 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2016-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2016-2020 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -24,7 +24,7 @@
  */
 
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -40,9 +40,10 @@
 #include "src/mca/base/base.h"
 #include "src/mca/base/pmix_mca_base_component_repository.h"
 #include "src/mca/pdl/base/base.h"
-#include "pmix_common.h"
+#include "include/pmix_common.h"
 #include "src/class/pmix_hash_table.h"
 #include "src/util/basename.h"
+#include "src/util/show_help.h"
 
 #if PMIX_HAVE_PDL_SUPPORT
 
@@ -93,6 +94,7 @@ static pmix_hash_table_t pmix_mca_base_component_repository;
 
 static int process_repository_item (const char *filename, void *data)
 {
+    (void)data;
     char name[PMIX_MCA_BASE_MAX_COMPONENT_NAME_LEN + 1];
     char type[PMIX_MCA_BASE_MAX_TYPE_NAME_LEN + 1];
     pmix_mca_base_component_repository_item_t *ri;
@@ -220,8 +222,13 @@ int pmix_mca_base_component_repository_add (const char *path)
             dir = pmix_mca_base_system_default_path;
         }
 
-        if (0 != pmix_pdl_foreachfile(dir, process_repository_item, NULL)) {
-            break;
+        if (0 != pmix_pdl_foreachfile(dir, process_repository_item, NULL) &&
+            !(0 == strcmp(dir, pmix_mca_base_system_default_path) || 0 == strcmp(dir, pmix_mca_base_user_default_path))) {
+            // It is not an error if a directory fails to add (e.g.,
+            // if it doesn't exist).  But we should warn about it as
+            // it is something related to "show_load_errors"
+            pmix_show_help("help-pmix-mca-base.txt",
+                           "failed to add component dir", true, dir);
         }
     } while (NULL != (dir = strtok_r (NULL, sep, &ctx)));
 
@@ -412,20 +419,18 @@ int pmix_mca_base_component_repository_open(pmix_mca_base_framework_t *framework
     char *err_msg = NULL;
     if (PMIX_SUCCESS != pmix_pdl_open(ri->ri_path, true, false, &ri->ri_dlhandle, &err_msg)) {
         if (NULL == err_msg) {
-            err_msg = "pmix_dl_open() error message was NULL!";
-        }
-        /* Because libltdl erroneously says "file not found" for any
-           type of error -- which is especially misleading when the file
-           is actually there but cannot be opened for some other reason
-           (e.g., missing symbol) -- do some simple huersitics and if
-           the file [probably] does exist, print a slightly better error
-           message. */
-        if (0 == strcasecmp("file not found", err_msg) &&
-            (file_exists(ri->ri_path, "lo") ||
-             file_exists(ri->ri_path, "so") ||
-             file_exists(ri->ri_path, "dylib") ||
-             file_exists(ri->ri_path, "dll"))) {
-            err_msg = "perhaps a missing symbol, or compiled for a different version of Open MPI?";
+            err_msg = strdup("pmix_dl_open() error message was NULL!");
+        } else if (file_exists(ri->ri_path, "lo") ||
+                   file_exists(ri->ri_path, "so") ||
+                   file_exists(ri->ri_path, "dylib") ||
+                   file_exists(ri->ri_path, "dll")) {
+            /* Because libltdl erroneously says "file not found" for any
+             * type of error -- which is especially misleading when the file
+             * is actually there but cannot be opened for some other reason
+             * (e.g., missing symbol) -- do some simple huersitics and if
+             * the file [probably] does exist, print a slightly better error
+             * message. */
+            err_msg = strdup("perhaps a missing symbol, or compiled for a different version of OpenPMIx");
         }
         pmix_output_verbose(vl, 0, "pmix_mca_base_component_repository_open: unable to open %s: %s (ignored)",
                             ri->ri_base, err_msg);
@@ -435,11 +440,13 @@ int pmix_mca_base_component_repository_open(pmix_mca_base_framework_t *framework
             f_comp->comp = ri;
             if (0 > asprintf(&(f_comp->error_msg), "%s", err_msg)) {
                 PMIX_RELEASE(f_comp);
+                free(err_msg);
                 return PMIX_ERR_BAD_PARAM;
             }
             pmix_list_append(&framework->framework_failed_components, &f_comp->super);
         }
 
+        free(err_msg);
         return PMIX_ERR_BAD_PARAM;
     }
 
