@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2014-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -19,13 +19,15 @@
  * $HEADER$
  */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
 
 #include "src/util/argv.h"
 #include "src/util/error.h"
 #include "src/util/output.h"
 #include "src/include/pmix_globals.h"
+#include "src/mca/preg/preg.h"
+#include "src/mca/ploc/ploc.h"
 
 #include "src/mca/bfrops/base/base.h"
 
@@ -108,12 +110,14 @@ pmix_status_t pmix_bfrops_base_std_copy(void **dest, void *src,
     case PMIX_BYTE:
     case PMIX_INT8:
     case PMIX_UINT8:
+    case PMIX_LINK_STATE:
         datasize = 1;
         break;
 
     case PMIX_INT16:
     case PMIX_UINT16:
     case PMIX_IOF_CHANNEL:
+    case PMIX_LOCTYPE:
         datasize = 2;
         break;
 
@@ -124,6 +128,7 @@ pmix_status_t pmix_bfrops_base_std_copy(void **dest, void *src,
 
     case PMIX_INT64:
     case PMIX_UINT64:
+    case PMIX_DEVTYPE:
         datasize = 8;
         break;
 
@@ -179,6 +184,10 @@ pmix_status_t pmix_bfrops_base_std_copy(void **dest, void *src,
         datasize = sizeof(pmix_alloc_directive_t);
         break;
 
+    case PMIX_JOB_STATE:
+        datasize = sizeof(pmix_job_state_t);
+        break;
+
     default:
         return PMIX_ERR_UNKNOWN_DATA_TYPE;
     }
@@ -202,6 +211,9 @@ pmix_status_t pmix_bfrops_base_std_copy(void **dest, void *src,
  pmix_status_t pmix_bfrops_base_copy_string(char **dest, char *src,
                                             pmix_data_type_t type)
 {
+    if (PMIX_STRING != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     if (NULL == src) {  /* got zero-length string/NULL pointer - store NULL */
         *dest = NULL;
     } else {
@@ -218,6 +230,9 @@ pmix_status_t pmix_bfrops_base_copy_value(pmix_value_t **dest,
 {
     pmix_value_t *p;
 
+    if (PMIX_VALUE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* create the new object */
     *dest = (pmix_value_t*)malloc(sizeof(pmix_value_t));
     if (NULL == *dest) {
@@ -235,6 +250,9 @@ pmix_status_t pmix_bfrops_base_copy_info(pmix_info_t **dest,
                                          pmix_info_t *src,
                                          pmix_data_type_t type)
 {
+    if (PMIX_VALUE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_info_t*)malloc(sizeof(pmix_info_t));
     pmix_strncpy((*dest)->key, src->key, PMIX_MAX_KEYLEN);
     (*dest)->flags = src->flags;
@@ -245,6 +263,9 @@ pmix_status_t pmix_bfrops_base_copy_buf(pmix_buffer_t **dest,
                                         pmix_buffer_t *src,
                                         pmix_data_type_t type)
 {
+    if (PMIX_BUFFER != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = PMIX_NEW(pmix_buffer_t);
     pmix_bfrops_base_copy_payload(*dest, src);
     return PMIX_SUCCESS;
@@ -256,6 +277,9 @@ pmix_status_t pmix_bfrops_base_copy_app(pmix_app_t **dest,
 {
     size_t j;
 
+    if (PMIX_APP != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_app_t*)malloc(sizeof(pmix_app_t));
     (*dest)->cmd = strdup(src->cmd);
     (*dest)->argv = pmix_argv_copy(src->argv);
@@ -279,6 +303,9 @@ pmix_status_t pmix_bfrops_base_copy_kval(pmix_kval_t **dest,
 {
     pmix_kval_t *p;
 
+    if (PMIX_KVAL != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* create the new object */
     *dest = PMIX_NEW(pmix_kval_t);
     if (NULL == *dest) {
@@ -296,6 +323,9 @@ pmix_status_t pmix_bfrops_base_copy_proc(pmix_proc_t **dest,
                                          pmix_proc_t *src,
                                          pmix_data_type_t type)
 {
+    if (PMIX_PROC != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_proc_t*)malloc(sizeof(pmix_proc_t));
     if (NULL == *dest) {
         return PMIX_ERR_OUT_OF_RESOURCE;
@@ -309,6 +339,9 @@ pmix_status_t pmix_bfrop_base_copy_persist(pmix_persistence_t **dest,
                                            pmix_persistence_t *src,
                                            pmix_data_type_t type)
 {
+    if (PMIX_PERSIST != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_persistence_t*)malloc(sizeof(pmix_persistence_t));
     if (NULL == *dest) {
         return PMIX_ERR_OUT_OF_RESOURCE;
@@ -321,6 +354,10 @@ pmix_status_t pmix_bfrops_base_copy_bo(pmix_byte_object_t **dest,
                                        pmix_byte_object_t *src,
                                        pmix_data_type_t type)
 {
+    if (PMIX_BYTE_OBJECT != type &&
+        PMIX_COMPRESSED_BYTE_OBJECT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_byte_object_t*)malloc(sizeof(pmix_byte_object_t));
     if (NULL == *dest) {
         return PMIX_ERR_OUT_OF_RESOURCE;
@@ -335,6 +372,9 @@ pmix_status_t pmix_bfrops_base_copy_pdata(pmix_pdata_t **dest,
                                           pmix_pdata_t *src,
                                           pmix_data_type_t type)
 {
+    if (PMIX_PDATA != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_pdata_t*)malloc(sizeof(pmix_pdata_t));
     pmix_strncpy((*dest)->proc.nspace, src->proc.nspace, PMIX_MAX_NSLEN);
     (*dest)->proc.rank = src->proc.rank;
@@ -348,6 +388,9 @@ pmix_status_t pmix_bfrops_base_copy_pinfo(pmix_proc_info_t **dest,
 {
     pmix_proc_info_t *p;
 
+    if (PMIX_INFO != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     PMIX_PROC_INFO_CREATE(p, 1);
     if (NULL == p) {
         return PMIX_ERR_NOMEM;
@@ -366,6 +409,22 @@ pmix_status_t pmix_bfrops_base_copy_pinfo(pmix_proc_info_t **dest,
     return PMIX_SUCCESS;
 }
 
+static pmix_status_t fill_coord(pmix_coord_t *dst,
+                                pmix_coord_t *src)
+{
+    dst->view = src->view;
+    dst->dims = src->dims;
+    if (0 < dst->dims) {
+        dst->coord = (uint32_t*)malloc(dst->dims * sizeof(uint32_t));
+        if (NULL == dst->coord) {
+            return PMIX_ERR_NOMEM;
+        }
+        memcpy(dst->coord, src->coord, dst->dims * sizeof(uint32_t));
+    }
+    return PMIX_SUCCESS;
+}
+
+
 /* the pmix_data_array_t is a little different in that it
  * is an array of values, and so we cannot just copy one
  * value at a time. So handle all value types here */
@@ -387,6 +446,15 @@ pmix_status_t pmix_bfrops_base_copy_darray(pmix_data_array_t **dest,
     pmix_proc_info_t *pi, *si;
     pmix_query_t *pq, *sq;
     pmix_envar_t *pe, *se;
+    pmix_regattr_t *pr, *sr;
+    pmix_coord_t *pc, *sc;
+    pmix_cpuset_t *pcpuset, *scpuset;
+    pmix_geometry_t *pgeoset, *sgeoset;
+    pmix_device_distance_t *pdevdist, *sdevdist;
+
+    if (PMIX_DATA_ARRAY != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
 
     p = (pmix_data_array_t*)calloc(1, sizeof(pmix_data_array_t));
     if (NULL == p) {
@@ -800,6 +868,112 @@ pmix_status_t pmix_bfrops_base_copy_darray(pmix_data_array_t **dest,
                 pe[n].separator = se[n].separator;
             }
             break;
+        case PMIX_COORD:
+            p->array = malloc(src->size * sizeof(pmix_coord_t));
+            if (NULL == p->array) {
+                free(p);
+                return PMIX_ERR_NOMEM;
+            }
+            pc = (pmix_coord_t*)p->array;
+            sc = (pmix_coord_t*)src->array;
+            for (n=0; n < src->size; n++) {
+                rc = fill_coord(&pc[n], &sc[n]);
+                if (PMIX_SUCCESS != rc) {
+                    PMIX_COORD_FREE(pc, src->size);
+                    free(p);
+                    return rc;
+                }
+            }
+            break;
+        case PMIX_REGATTR:
+            PMIX_REGATTR_CREATE(p->array, src->size);
+            if (NULL == p->array) {
+                free(p);
+                return PMIX_ERR_NOMEM;
+            }
+            pr = (pmix_regattr_t*)p->array;
+            sr = (pmix_regattr_t*)src->array;
+            for (n=0; n < src->size; n++) {
+                if (NULL != sr[n].name) {
+                    pr[n].name = strdup(sr[n].name);
+                }
+                PMIX_LOAD_KEY(pr[n].string, sr[n].string);
+                pr[n].type = sr[n].type;
+                pr[n].description = pmix_argv_copy(sr[n].description);
+            }
+            break;
+        case PMIX_PROC_CPUSET:
+            PMIX_CPUSET_CREATE(p->array, src->size);
+            if (NULL == p->array) {
+                free(p);
+                return PMIX_ERR_NOMEM;
+            }
+            pcpuset = (pmix_cpuset_t*)p->array;
+            scpuset = (pmix_cpuset_t*)src->array;
+            for (n=0; n < src->size; n++) {
+                rc = pmix_ploc.copy_cpuset(&pcpuset[n], &scpuset[n]);
+                if (PMIX_SUCCESS != rc) {
+                    pmix_ploc.release_cpuset(pcpuset, src->size);
+                    free(p->array);
+                    free(p);
+                    return rc;
+                }
+            }
+            break;
+        case PMIX_GEOMETRY:
+            PMIX_GEOMETRY_CREATE(p->array, src->size);
+            if (NULL == p->array) {
+                free(p);
+                return PMIX_ERR_NOMEM;
+            }
+            pgeoset = (pmix_geometry_t*)p->array;
+            sgeoset = (pmix_geometry_t*)src->array;
+            for (n=0; n < src->size; n++) {
+                pgeoset[n].fabric = sgeoset[n].fabric;
+                if (NULL != sgeoset[n].uuid) {
+                    pgeoset[n].uuid = strdup(sgeoset[n].uuid);
+                }
+                if (NULL != sgeoset[n].osname) {
+                    pgeoset[n].osname = strdup(sgeoset[n].osname);
+                }
+                if (NULL != sgeoset[n].coordinates) {
+                    pgeoset[n].ncoords = sgeoset[n].ncoords;
+                    pgeoset[n].coordinates = (pmix_coord_t*)malloc(pgeoset[n].ncoords * sizeof(pmix_coord_t));
+                    if (NULL == pgeoset[n].coordinates) {
+                        free(p);
+                        return PMIX_ERR_NOMEM;
+                    }
+                    for (m=0; m < pgeoset[n].ncoords; m++) {
+                        rc = fill_coord(&pgeoset[n].coordinates[m], &sgeoset[n].coordinates[m]);
+                        if (PMIX_SUCCESS != rc) {
+                            PMIX_GEOMETRY_FREE(pgeoset, src->size);
+                            free(p);
+                            return rc;
+                        }
+                    }
+                }
+            }
+            break;
+        case PMIX_DEVICE_DIST:
+            PMIX_DEVICE_DIST_CREATE(p->array, src->size);
+            if (NULL == p->array) {
+                free(p);
+                return PMIX_ERR_NOMEM;
+            }
+            pdevdist = (pmix_device_distance_t*)p->array;
+            sdevdist = (pmix_device_distance_t*)src->array;
+            for (n=0; n < src->size; n++) {
+                if (NULL != sdevdist[n].uuid) {
+                    pdevdist[n].uuid = strdup(sdevdist[n].uuid);
+                }
+                if (NULL != sdevdist[n].osname) {
+                    pdevdist[n].osname = strdup(sdevdist[n].osname);
+                }
+                pdevdist[n].type = sdevdist[n].type;
+                pdevdist[n].mindist = sdevdist[n].mindist;
+                pdevdist[n].maxdist = sdevdist[n].maxdist;
+            }
+            break;
         default:
             free(p);
             return PMIX_ERR_UNKNOWN_DATA_TYPE;
@@ -815,6 +989,9 @@ pmix_status_t pmix_bfrops_base_copy_query(pmix_query_t **dest,
 {
     pmix_status_t rc;
 
+    if (PMIX_QUERY != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     *dest = (pmix_query_t*)malloc(sizeof(pmix_query_t));
     if (NULL != src->keys) {
         (*dest)->keys = pmix_argv_copy(src->keys);
@@ -833,6 +1010,9 @@ pmix_status_t pmix_bfrops_base_copy_envar(pmix_envar_t **dest,
                                           pmix_envar_t *src,
                                           pmix_data_type_t type)
 {
+    if (PMIX_ENVAR != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     PMIX_ENVAR_CREATE(*dest, 1);
     if (NULL == (*dest)) {
         return PMIX_ERR_NOMEM;
@@ -846,3 +1026,203 @@ pmix_status_t pmix_bfrops_base_copy_envar(pmix_envar_t **dest,
     (*dest)->separator = src->separator;
     return PMIX_SUCCESS;
 }
+
+pmix_status_t pmix_bfrops_base_copy_coord(pmix_coord_t **dest,
+                                          pmix_coord_t *src,
+                                          pmix_data_type_t type)
+{
+    pmix_coord_t *d;
+    pmix_status_t rc;
+
+    if (PMIX_COORD != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    d = (pmix_coord_t*)malloc(sizeof(pmix_coord_t));
+    if (NULL == d) {
+        return PMIX_ERR_NOMEM;
+    }
+    PMIX_COORD_CONSTRUCT(d);
+    rc = fill_coord(d, src);
+    if (PMIX_SUCCESS != rc) {
+        PMIX_COORD_DESTRUCT(d);
+        free(d);
+    } else {
+        *dest = d;
+    }
+    return rc;
+}
+
+pmix_status_t pmix_bfrops_base_copy_regattr(pmix_regattr_t **dest,
+                                            pmix_regattr_t *src,
+                                            pmix_data_type_t type)
+{
+    if (PMIX_REGATTR != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    PMIX_REGATTR_CREATE(*dest, 1);
+    if (NULL == (*dest)) {
+        return PMIX_ERR_NOMEM;
+    }
+    if (NULL != src->name) {
+        (*dest)->name = strdup(src->name);
+    }
+    PMIX_LOAD_KEY((*dest)->string, src->string);
+    (*dest)->type = src->type;
+    (*dest)->description = pmix_argv_copy(src->description);
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_copy_regex(char **dest,
+                                          char *src,
+                                          pmix_data_type_t type)
+{
+    size_t len;
+
+    if (PMIX_REGEX != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    return pmix_preg.copy(dest, &len, src);
+}
+
+pmix_status_t pmix_bfrops_base_copy_cpuset(pmix_cpuset_t **dest,
+                                           pmix_cpuset_t *src,
+                                           pmix_data_type_t type)
+{
+    pmix_cpuset_t *dst;
+    pmix_status_t rc;
+
+    if (PMIX_PROC_CPUSET != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    PMIX_CPUSET_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+
+    rc = pmix_ploc.copy_cpuset(dst, src);
+    if (PMIX_SUCCESS == rc) {
+        *dest = dst;
+    } else {
+        free(dst);
+    }
+    return rc;
+}
+
+pmix_status_t pmix_bfrops_base_copy_geometry(pmix_geometry_t **dest,
+                                             pmix_geometry_t *src,
+                                             pmix_data_type_t type)
+{
+    pmix_geometry_t *dst;
+    pmix_status_t rc;
+    size_t n;
+
+    if (PMIX_GEOMETRY != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    PMIX_GEOMETRY_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+
+    dst->fabric = src->fabric;
+    if (NULL != src->uuid) {
+        dst->uuid = strdup(src->uuid);
+    }
+    if (NULL != src->osname) {
+        dst->osname = strdup(src->osname);
+    }
+    if (NULL != src->coordinates) {
+        dst->ncoords = src->ncoords;
+        dst->coordinates = (pmix_coord_t*)calloc(dst->ncoords, sizeof(pmix_coord_t));
+        for (n=0; n < dst->ncoords; n++) {
+            rc = fill_coord(&dst->coordinates[n], &src->coordinates[n]);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_GEOMETRY_FREE(dst, 1);
+                return rc;
+            }
+        }
+    }
+
+    *dest = dst;
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_copy_devdist(pmix_device_distance_t **dest,
+                                            pmix_device_distance_t *src,
+                                            pmix_data_type_t type)
+{
+    pmix_device_distance_t *dst;
+
+    if (PMIX_DEVICE_DIST != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    PMIX_DEVICE_DIST_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+
+    if (NULL != src->uuid) {
+        dst->uuid = strdup(src->uuid);
+    }
+    if (NULL != src->osname) {
+        dst->osname = strdup(src->osname);
+    }
+    dst->type = src->type;
+    dst->mindist = src->mindist;
+    dst->maxdist = src->maxdist;
+
+    *dest = dst;
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_copy_endpoint(pmix_endpoint_t **dest,
+                                             pmix_endpoint_t *src,
+                                             pmix_data_type_t type)
+{
+    pmix_endpoint_t *dst;
+
+    if (PMIX_ENDPOINT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    PMIX_ENDPOINT_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+
+    if (NULL != src->uuid) {
+        dst->uuid = strdup(src->uuid);
+    }
+    if (NULL != src->endpt.bytes) {
+        dst->endpt.bytes = (char*)malloc(src->endpt.size);
+        memcpy(dst->endpt.bytes, src->endpt.bytes, src->endpt.size);
+        dst->endpt.size = src->endpt.size;
+    }
+
+    *dest = dst;
+    return PMIX_SUCCESS;
+}
+
+pmix_status_t pmix_bfrops_base_copy_topology(pmix_topology_t **dest,
+                                             pmix_topology_t *src,
+                                             pmix_data_type_t type)
+{
+    pmix_topology_t *dst;
+    pmix_status_t rc;
+
+    if (PMIX_TOPO != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    PMIX_TOPOLOGY_CREATE(dst, 1);
+    if (NULL == dst) {
+        return PMIX_ERR_NOMEM;
+    }
+
+    rc = pmix_ploc.copy_topology(dst, src);
+    if (PMIX_SUCCESS == rc) {
+        *dest = dst;
+    } else {
+        free(dst);
+    }
+    return rc;
+}
