@@ -9,7 +9,9 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2015-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2019      Mellanox Technologies, Inc.
+ *                         All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -17,7 +19,7 @@
  * $HEADER$
  */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
 
 #include <stdio.h>
@@ -28,6 +30,7 @@
 #include "src/util/argv.h"
 #include "src/util/error.h"
 #include "src/include/pmix_globals.h"
+#include "src/mca/preg/preg.h"
 
 #include "src/mca/bfrops/base/base.h"
 
@@ -59,6 +62,13 @@ void pmix_bfrops_base_value_load(pmix_value_t *v, const void *data,
     pmix_envar_t *envar;
     pmix_data_array_t *darray;
     pmix_status_t rc;
+    pmix_coord_t *coord;
+    pmix_regattr_t *regattr;
+    pmix_topology_t *topo;
+    pmix_cpuset_t *cpuset;
+    pmix_geometry_t *geometry;
+    pmix_endpoint_t *endpoint;
+    pmix_device_distance_t *devdist;
 
     v->type = type;
     if (NULL == data) {
@@ -143,6 +153,8 @@ void pmix_bfrops_base_value_load(pmix_value_t *v, const void *data,
             memcpy(v->data.proc, data, sizeof(pmix_proc_t));
             break;
         case PMIX_BYTE_OBJECT:
+        case PMIX_COMPRESSED_STRING:
+        case PMIX_COMPRESSED_BYTE_OBJECT:
             bo = (pmix_byte_object_t*)data;
             v->data.bo.bytes = (char*)malloc(bo->size);
             if (NULL == v->data.bo.bytes) {
@@ -181,9 +193,19 @@ void pmix_bfrops_base_value_load(pmix_value_t *v, const void *data,
             memcpy(&(v->data.pinfo->pid), &pi->pid, sizeof(pid_t));
             memcpy(&(v->data.pinfo->exit_code), &pi->exit_code, sizeof(int));
             break;
+        case PMIX_DATA_ARRAY:
+            darray = (pmix_data_array_t*)data;
+            rc = pmix_bfrops_base_copy_darray(&v->data.darray, darray, PMIX_DATA_ARRAY);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+            }
+            break;
         case PMIX_POINTER:
             v->data.ptr = (void*)data;
             break;
+        case PMIX_ALLOC_DIRECTIVE:
+            memcpy(&(v->data.adir), data, sizeof(pmix_alloc_directive_t));
+            break;
         case PMIX_ENVAR:
             envar = (pmix_envar_t*)data;
             if (NULL != envar->envar) {
@@ -194,14 +216,78 @@ void pmix_bfrops_base_value_load(pmix_value_t *v, const void *data,
             }
             v->data.envar.separator = envar->separator;
             break;
-        case PMIX_DATA_ARRAY:
-            darray = (pmix_data_array_t*)data;
-            rc = pmix_bfrops_base_copy_darray(&v->data.darray, darray, PMIX_DATA_ARRAY);
+        case PMIX_COORD:
+            coord = (pmix_coord_t*)data;
+            rc = pmix_bfrops_base_copy_coord(&v->data.coord, coord, PMIX_COORD);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+            }
+            break;
+        case PMIX_LINK_STATE:
+            memcpy(&(v->data.linkstate), data, sizeof(pmix_link_state_t));
+            break;
+        case PMIX_JOB_STATE:
+            memcpy(&(v->data.jstate), data, sizeof(pmix_job_state_t));
+            break;
+        case PMIX_TOPO:
+            topo = (pmix_topology_t*)data;
+            rc = pmix_bfrops_base_copy_topology(&v->data.topo, topo, PMIX_TOPO);
+            if (PMIX_ERR_INIT == rc || PMIX_ERR_NOT_SUPPORTED == rc) {
+                /* we are not initialized yet, so just copy the pointer */
+                v->data.topo = topo;
+                rc = PMIX_SUCCESS;
+            }
+            break;
+        case PMIX_PROC_CPUSET:
+            cpuset = (pmix_cpuset_t*)data;
+            rc = pmix_bfrops_base_copy_cpuset(&v->data.cpuset, cpuset, PMIX_PROC_CPUSET);
+            if (PMIX_ERR_INIT == rc || PMIX_ERR_NOT_SUPPORTED == rc) {
+                /* we are not initialized yet, so just copy the pointer */
+                v->data.cpuset = cpuset;
+                rc = PMIX_SUCCESS;
+            }
+            break;
+        case PMIX_LOCTYPE:
+            memcpy(&(v->data.locality), data, sizeof(pmix_locality_t));
+            break;
+        case PMIX_GEOMETRY:
+            geometry = (pmix_geometry_t*)data;
+            rc = pmix_bfrops_base_copy_geometry(&v->data.geometry, geometry, PMIX_GEOMETRY);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+            }
+            break;
+        case PMIX_DEVTYPE:
+            memcpy(&(v->data.devtype), data, sizeof(pmix_device_type_t));
+            break;
+        case PMIX_DEVICE_DIST:
+            devdist = (pmix_device_distance_t*)data;
+            rc = pmix_bfrops_base_copy_devdist(&v->data.devdist, devdist, PMIX_DEVICE_DIST);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+            }
+            break;
+        case PMIX_ENDPOINT:
+            endpoint = (pmix_endpoint_t*)data;
+            rc = pmix_bfrops_base_copy_endpoint(&v->data.endpoint, endpoint, PMIX_ENDPOINT);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+            }
+            break;
+        case PMIX_REGATTR:
+            regattr = (pmix_regattr_t*)data;
+            rc = pmix_bfrops_base_copy_regattr((pmix_regattr_t**)&v->data.ptr, regattr, PMIX_REGATTR);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+            }
+            break;
+        case PMIX_REGEX:
+            /* load it into the byte object */
+            rc = pmix_preg.copy(&v->data.bo.bytes, &v->data.bo.size, (char*)data);
             if (PMIX_SUCCESS != rc) {
                 PMIX_ERROR_LOG(rc);
             }
             break;
-
         default:
             /* silence warnings */
             break;
@@ -217,6 +303,7 @@ pmix_status_t pmix_bfrops_base_value_unload(pmix_value_t *kv,
     pmix_status_t rc;
     pmix_envar_t *envar;
     pmix_data_array_t **darray;
+    pmix_regattr_t *regattr, *r;
 
     rc = PMIX_SUCCESS;
     if (NULL == data ||
@@ -305,7 +392,23 @@ pmix_status_t pmix_bfrops_base_value_unload(pmix_value_t *kv,
             memcpy(*data, &(kv->data.time), sizeof(time_t));
             *sz = sizeof(time_t);
             break;
+        case PMIX_STATUS:
+            memcpy(*data, &(kv->data.status), sizeof(pmix_status_t));
+            *sz = sizeof(pmix_status_t);
+            break;
+        case PMIX_PROC_RANK:
+            memcpy(*data, &(kv->data.rank), sizeof(pmix_rank_t));
+            *sz = sizeof(pmix_rank_t);
+            break;
+        case PMIX_PROC:
+            rc = pmix_bfrops_base_copy_proc((pmix_proc_t**)data, kv->data.proc, PMIX_PROC);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_proc_t);
+            }
+            break;
         case PMIX_BYTE_OBJECT:
+        case PMIX_COMPRESSED_STRING:
+        case PMIX_COMPRESSED_BYTE_OBJECT:
             if (NULL != kv->data.bo.bytes && 0 < kv->data.bo.size) {
                 *data = kv->data.bo.bytes;
                 *sz = kv->data.bo.size;
@@ -330,14 +433,26 @@ pmix_status_t pmix_bfrops_base_value_unload(pmix_value_t *kv,
             memcpy(*data, &(kv->data.state), sizeof(pmix_proc_state_t));
             *sz = sizeof(pmix_proc_state_t);
             break;
-        case PMIX_POINTER:
-            *data = (void*)kv->data.ptr;
-            *sz = sizeof(void*);
+        case PMIX_PROC_INFO:
+            rc = pmix_bfrops_base_copy_pinfo((pmix_proc_info_t**)data, kv->data.pinfo, PMIX_PROC_INFO);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_proc_info_t);
+            }
             break;
         case PMIX_DATA_ARRAY:
             darray = (pmix_data_array_t**)data;
             rc = pmix_bfrops_base_copy_darray(darray, kv->data.darray, PMIX_DATA_ARRAY);
-            *sz = sizeof(pmix_data_array_t);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_data_array_t);
+            }
+            break;
+        case PMIX_POINTER:
+            *data = (void*)kv->data.ptr;
+            *sz = sizeof(void*);
+            break;
+        case PMIX_ALLOC_DIRECTIVE:
+            memcpy(*data, &(kv->data.adir), sizeof(pmix_alloc_directive_t));
+            *sz = sizeof(pmix_alloc_directive_t);
             break;
         case PMIX_ENVAR:
             PMIX_ENVAR_CREATE(envar, 1);
@@ -354,6 +469,92 @@ pmix_status_t pmix_bfrops_base_value_unload(pmix_value_t *kv,
             *data = envar;
             *sz = sizeof(pmix_envar_t);
             break;
+        case PMIX_COORD:
+            rc = pmix_bfrops_base_copy_coord((pmix_coord_t**)data, kv->data.coord, PMIX_COORD);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_coord_t);
+            }
+            break;
+        case PMIX_LINK_STATE:
+            memcpy(*data, &(kv->data.linkstate), sizeof(pmix_link_state_t));
+            *sz = sizeof(pmix_link_state_t);
+            break;
+        case PMIX_JOB_STATE:
+            memcpy(*data, &(kv->data.jstate), sizeof(pmix_job_state_t));
+            *sz = sizeof(pmix_job_state_t);
+            break;
+        case PMIX_TOPO:
+            rc = pmix_bfrops_base_copy_topology((pmix_topology_t**)data, kv->data.topo, PMIX_TOPO);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_topology_t);
+            } else if (PMIX_ERR_INIT == rc || PMIX_ERR_NOT_SUPPORTED == rc) {
+                *data = malloc(sizeof(pmix_topology_t));
+                memcpy(*data, kv->data.topo, sizeof(pmix_topology_t));
+                *sz = sizeof(pmix_topology_t);
+                rc = PMIX_SUCCESS;
+            }
+            break;
+        case PMIX_PROC_CPUSET:
+            rc = pmix_bfrops_base_copy_cpuset((pmix_cpuset_t**)data, kv->data.cpuset, PMIX_PROC_CPUSET);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_cpuset_t);
+            } else if (PMIX_ERR_INIT == rc || PMIX_ERR_NOT_SUPPORTED == rc) {
+                *data = malloc(sizeof(pmix_cpuset_t));
+                memcpy(*data, kv->data.cpuset, sizeof(pmix_cpuset_t));
+                *sz = sizeof(pmix_cpuset_t);
+                rc = PMIX_SUCCESS;
+            }
+            break;
+        case PMIX_LOCTYPE:
+            memcpy(*data, &(kv->data.locality), sizeof(pmix_locality_t));
+            *sz = sizeof(pmix_locality_t);
+            break;
+        case PMIX_GEOMETRY:
+            rc = pmix_bfrops_base_copy_geometry((pmix_geometry_t**)data, kv->data.geometry, PMIX_GEOMETRY);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_geometry_t);
+            }
+            break;
+        case PMIX_DEVTYPE:
+            memcpy(*data, &(kv->data.devtype), sizeof(pmix_device_type_t));
+            *sz = sizeof(pmix_device_type_t);
+            break;
+        case PMIX_DEVICE_DIST:
+            rc = pmix_bfrops_base_copy_devdist((pmix_device_distance_t**)data, kv->data.devdist, PMIX_DEVICE_DIST);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_device_distance_t);
+            }
+            break;
+        case PMIX_ENDPOINT:
+            rc = pmix_bfrops_base_copy_endpoint((pmix_endpoint_t**)data, kv->data.endpoint, PMIX_ENDPOINT);
+            if (PMIX_SUCCESS == rc) {
+                *sz = sizeof(pmix_endpoint_t);
+            }
+            break;
+        case PMIX_REGATTR:
+            PMIX_REGATTR_CREATE(regattr, 1);
+            if (NULL == regattr) {
+                return PMIX_ERR_NOMEM;
+            }
+            r = (pmix_regattr_t*)kv->data.ptr;
+            if (NULL != r->name) {
+                regattr->name = strdup(r->name);
+            }
+            PMIX_LOAD_KEY(regattr->string, r->string);
+            regattr->type = r->type;
+            regattr->description = pmix_argv_copy(r->description);
+            *data = regattr;
+            *sz = sizeof(pmix_regattr_t);
+            break;
+        case PMIX_REGEX:
+            if (NULL != kv->data.bo.bytes && 0 < kv->data.bo.size) {
+                *data = kv->data.bo.bytes;
+                *sz = kv->data.bo.size;
+            } else {
+                *data = NULL;
+                *sz = 0;
+            }
+            break;
         default:
             /* silence warnings */
             rc = PMIX_ERROR;
@@ -501,6 +702,27 @@ pmix_value_cmp_t pmix_bfrops_base_value_cmp(pmix_value_t *p,
             }
             rc = PMIX_EQUAL;
             break;
+        case PMIX_COORD:
+            ret = memcmp(p->data.coord, p1->data.coord, sizeof(pmix_coord_t));
+            if (0 > ret) {
+                return PMIX_VALUE2_GREATER;
+            } else if (0 < ret) {
+                return PMIX_VALUE1_GREATER;
+            } else {
+                return PMIX_EQUAL;
+            }
+            break;
+        case PMIX_REGATTR:
+            ret = memcmp(p->data.ptr, p1->data.ptr, sizeof(pmix_regattr_t));
+            if (0 > ret) {
+                return PMIX_VALUE2_GREATER;
+            } else if (0 < ret) {
+                return PMIX_VALUE1_GREATER;
+            } else {
+                return PMIX_EQUAL;
+            }
+            break;
+
         default:
             pmix_output(0, "COMPARE-PMIX-VALUE: UNSUPPORTED TYPE %d", (int)p->type);
     }
@@ -511,6 +733,8 @@ pmix_value_cmp_t pmix_bfrops_base_value_cmp(pmix_value_t *p,
 pmix_status_t pmix_bfrops_base_value_xfer(pmix_value_t *p,
                                           const pmix_value_t *src)
 {
+    pmix_status_t rc;
+
     /* copy the right field */
     p->type = src->type;
     switch (src->type) {
@@ -588,6 +812,9 @@ pmix_status_t pmix_bfrops_base_value_xfer(pmix_value_t *p,
     case PMIX_STATUS:
         memcpy(&p->data.status, &src->data.status, sizeof(pmix_status_t));
         break;
+    case PMIX_PROC_RANK:
+        memcpy(&p->data.rank, &src->data.rank, sizeof(pmix_rank_t));
+        break;
     case PMIX_PROC:
         PMIX_PROC_CREATE(p->data.proc, 1);
         if (NULL == p->data.proc) {
@@ -595,11 +822,10 @@ pmix_status_t pmix_bfrops_base_value_xfer(pmix_value_t *p,
         }
         memcpy(p->data.proc, src->data.proc, sizeof(pmix_proc_t));
         break;
-    case PMIX_PROC_RANK:
-        memcpy(&p->data.rank, &src->data.rank, sizeof(pmix_rank_t));
-        break;
     case PMIX_BYTE_OBJECT:
     case PMIX_COMPRESSED_STRING:
+    case PMIX_REGEX:
+    case PMIX_COMPRESSED_BYTE_OBJECT:
         memset(&p->data.bo, 0, sizeof(pmix_byte_object_t));
         if (NULL != src->data.bo.bytes && 0 < src->data.bo.size) {
             p->data.bo.bytes = malloc(src->data.bo.size);
@@ -629,6 +855,9 @@ pmix_status_t pmix_bfrops_base_value_xfer(pmix_value_t *p,
     case PMIX_POINTER:
         p->data.ptr = src->data.ptr;
         break;
+    case PMIX_ALLOC_DIRECTIVE:
+        memcpy(&p->data.adir, &src->data.adir, sizeof(pmix_alloc_directive_t));
+        break;
     case PMIX_ENVAR:
         PMIX_ENVAR_CONSTRUCT(&p->data.envar);
         if (NULL != src->data.envar.envar) {
@@ -639,7 +868,44 @@ pmix_status_t pmix_bfrops_base_value_xfer(pmix_value_t *p,
         }
         p->data.envar.separator = src->data.envar.separator;
         break;
-
+    case PMIX_COORD:
+        return pmix_bfrops_base_copy_coord(&p->data.coord, src->data.coord, PMIX_COORD);
+    case PMIX_LINK_STATE:
+        memcpy(&p->data.linkstate, &src->data.linkstate, sizeof(pmix_link_state_t));
+        break;
+    case PMIX_JOB_STATE:
+        memcpy(&p->data.jstate, &src->data.jstate, sizeof(pmix_job_state_t));
+        break;
+    case PMIX_TOPO:
+        rc = pmix_bfrops_base_copy_topology(&p->data.topo, src->data.topo, PMIX_TOPO);
+        if (PMIX_ERR_INIT == rc || PMIX_ERR_NOT_SUPPORTED == rc) {
+            /* we are being asked to do this before init, so
+             * just copy the pointer across */
+            p->data.topo = src->data.topo;
+        }
+        break;
+    case PMIX_PROC_CPUSET:
+        rc = pmix_bfrops_base_copy_cpuset(&p->data.cpuset, src->data.cpuset, PMIX_PROC_CPUSET);
+        if (PMIX_ERR_INIT == rc || PMIX_ERR_NOT_SUPPORTED == rc) {
+            /* we are being asked to do this before init, so
+             * just copy the pointer across */
+            p->data.cpuset = src->data.cpuset;
+        }
+        break;
+    case PMIX_LOCTYPE:
+        memcpy(&p->data.locality, &src->data.locality, sizeof(pmix_locality_t));
+        break;
+    case PMIX_GEOMETRY:
+        return pmix_bfrops_base_copy_geometry(&p->data.geometry, src->data.geometry, PMIX_GEOMETRY);
+    case PMIX_DEVTYPE:
+        memcpy(&p->data.devtype, &src->data.devtype, sizeof(pmix_device_type_t));
+        break;
+    case PMIX_DEVICE_DIST:
+        return pmix_bfrops_base_copy_devdist(&p->data.devdist, src->data.devdist, PMIX_DEVICE_DIST);
+    case PMIX_ENDPOINT:
+        return pmix_bfrops_base_copy_endpoint(&p->data.endpoint, src->data.endpoint, PMIX_ENDPOINT);
+    case PMIX_REGATTR:
+        return pmix_bfrops_base_copy_regattr((pmix_regattr_t**)&p->data.ptr, src->data.ptr, PMIX_REGATTR);
     default:
         pmix_output(0, "PMIX-XFER-VALUE: UNSUPPORTED TYPE %d", (int)src->type);
         return PMIX_ERROR;
@@ -727,39 +993,23 @@ bool pmix_bfrop_too_small(pmix_buffer_t *buffer, size_t bytes_reqd)
     return false;
 }
 
-pmix_status_t pmix_bfrop_store_data_type(pmix_buffer_t *buffer, pmix_data_type_t type)
+pmix_status_t pmix_bfrop_store_data_type(pmix_pointer_array_t *regtypes,
+                                         pmix_buffer_t *buffer, pmix_data_type_t type)
 {
-    uint16_t tmp;
-    char *dst;
+    pmix_status_t ret;
 
-    /* check to see if buffer needs extending */
-     if (NULL == (dst = pmix_bfrop_buffer_extend(buffer, sizeof(tmp)))) {
-        return PMIX_ERR_OUT_OF_RESOURCE;
-    }
-    tmp = pmix_htons(type);
-    memcpy(dst, &tmp, sizeof(tmp));
-    buffer->pack_ptr += sizeof(tmp);
-    buffer->bytes_used += sizeof(tmp);
-
-    return PMIX_SUCCESS;
+    PMIX_BFROPS_PACK_TYPE(ret, buffer, &type, 1, PMIX_UINT16, regtypes);
+    return ret;
 }
 
-pmix_status_t pmix_bfrop_get_data_type(pmix_buffer_t *buffer, pmix_data_type_t *type)
+pmix_status_t pmix_bfrop_get_data_type(pmix_pointer_array_t *regtypes,
+                                       pmix_buffer_t *buffer, pmix_data_type_t *type)
 {
-    uint16_t tmp;
+    pmix_status_t ret;
+    int32_t m = 1;
 
-    /* check to see if there's enough data in buffer */
-    if (pmix_bfrop_too_small(buffer, sizeof(tmp))) {
-        return PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER;
-    }
-
-    /* unpack the data */
-    memcpy(&tmp, buffer->unpack_ptr, sizeof(tmp));
-    tmp = pmix_ntohs(tmp);
-    memcpy(type, &tmp, sizeof(tmp));
-    buffer->unpack_ptr += sizeof(tmp);
-
-    return PMIX_SUCCESS;
+    PMIX_BFROPS_UNPACK_TYPE(ret, buffer, type, &m, PMIX_UINT16, regtypes);
+    return ret;
 }
 
 const char* pmix_bfrops_base_data_type_string(pmix_pointer_array_t *regtypes,
@@ -774,3 +1024,82 @@ const char* pmix_bfrops_base_data_type_string(pmix_pointer_array_t *regtypes,
 
     return info->odti_name;
 }
+
+PMIX_EXPORT void* pmix_info_list_start(void)
+{
+    pmix_list_t *p;
+
+    p = PMIX_NEW(pmix_list_t);
+    return p;
+}
+
+PMIX_EXPORT pmix_status_t pmix_info_list_add(void *ptr,
+                                             pmix_key_t key,
+                                             void *value,
+                                             pmix_data_type_t type)
+{
+    pmix_list_t *p = (pmix_list_t*)ptr;
+    pmix_infolist_t *iptr;
+
+    iptr = PMIX_NEW(pmix_infolist_t);
+    if (NULL == iptr) {
+        return PMIX_ERR_NOMEM;
+    }
+    PMIX_INFO_LOAD(&iptr->info, key, value, type);
+    pmix_list_append(p, &iptr->super);
+    return PMIX_SUCCESS;
+}
+
+PMIX_EXPORT pmix_status_t pmix_info_list_xfer(void *ptr,
+                                              const pmix_info_t *info)
+{
+    pmix_list_t *p = (pmix_list_t*)ptr;
+    pmix_infolist_t *iptr;
+
+    iptr = PMIX_NEW(pmix_infolist_t);
+    if (NULL == iptr) {
+        return PMIX_ERR_NOMEM;
+    }
+    PMIX_INFO_XFER(&iptr->info, info);
+    pmix_list_append(p, &iptr->super);
+    return PMIX_SUCCESS;
+}
+
+PMIX_EXPORT pmix_status_t pmix_info_list_convert(void *ptr, pmix_data_array_t *par)
+{
+    pmix_list_t *p = (pmix_list_t*)ptr;
+    size_t n;
+    pmix_infolist_t *iptr;
+    pmix_info_t *array;
+
+    if (NULL == par || NULL == ptr) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    n = pmix_list_get_size(p);
+    if (0 == n) {
+        return PMIX_ERR_EMPTY;
+    }
+    PMIX_INFO_CREATE(par->array, n);
+    if (NULL == par->array) {
+        return PMIX_ERR_NOMEM;
+    }
+    par->type = PMIX_INFO;
+    par->size = n;
+    array = (pmix_info_t*)par->array;
+
+    /* transfer the elements across */
+    n=0;
+    PMIX_LIST_FOREACH(iptr, p, pmix_infolist_t) {
+        PMIX_INFO_XFER(&array[n], &iptr->info);
+        ++n;
+    }
+
+    return PMIX_SUCCESS;
+}
+
+PMIX_EXPORT void pmix_info_list_release(void *ptr)
+{
+    pmix_list_t *p = (pmix_list_t*)ptr;
+    PMIX_LIST_RELEASE(p);
+}
