@@ -10,7 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, Inc.  All rights reserved.
- * Copyright (c) 2014-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  *
@@ -21,9 +21,9 @@
  * $HEADER$
  */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
-#include <src/include/pmix_stdint.h>
+#include "src/include/pmix_stdint.h"
 
 #include <stdio.h>
 #ifdef HAVE_TIME_H
@@ -31,7 +31,7 @@
 #endif
 
 #include "src/util/error.h"
-
+#include "src/mca/ploc/ploc.h"
 #include "src/include/pmix_globals.h"
 #include "src/mca/bfrops/base/base.h"
 
@@ -64,6 +64,9 @@ int pmix_bfrops_base_print_bool(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_BOOL != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -105,6 +108,9 @@ int pmix_bfrops_base_print_byte(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_BYTE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -145,6 +151,9 @@ int pmix_bfrops_base_print_string(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_STRING != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -185,6 +194,9 @@ int pmix_bfrops_base_print_size(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_SIZE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -225,6 +237,9 @@ int pmix_bfrops_base_print_pid(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_PID != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -264,6 +279,9 @@ int pmix_bfrops_base_print_int(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_INT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -304,6 +322,9 @@ int pmix_bfrops_base_print_uint(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_UINT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -344,6 +365,9 @@ int pmix_bfrops_base_print_uint8(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_UINT8 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -384,6 +408,9 @@ int pmix_bfrops_base_print_uint16(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_UINT16 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -424,6 +451,9 @@ int pmix_bfrops_base_print_uint32(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_UINT32 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -464,6 +494,9 @@ int pmix_bfrops_base_print_int8(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_INT8 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -504,6 +537,9 @@ int pmix_bfrops_base_print_int16(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_INT16 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -544,6 +580,9 @@ int pmix_bfrops_base_print_int32(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_INT32 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -584,6 +623,9 @@ int pmix_bfrops_base_print_uint64(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_UINT64 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -625,6 +667,9 @@ int pmix_bfrops_base_print_int64(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_INT64 != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -665,6 +710,9 @@ int pmix_bfrops_base_print_float(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_FLOAT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -705,6 +753,9 @@ int pmix_bfrops_base_print_double(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_DOUBLE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -746,6 +797,9 @@ int pmix_bfrops_base_print_time(char **output, char *prefix,
     char *t;
     int ret;
 
+    if (PMIX_TIME != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -789,6 +843,9 @@ int pmix_bfrops_base_print_timeval(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_TIMEVAL != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -830,6 +887,9 @@ int pmix_bfrops_base_print_status(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_STATUS != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -875,7 +935,12 @@ int pmix_bfrops_base_print_status(char **output, char *prefix,
  {
     char *prefx;
     int rc;
+    pmix_regattr_t *r;
+    char *tp;
 
+    if (PMIX_VALUE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1014,19 +1079,61 @@ int pmix_bfrops_base_print_status(char **output, char *prefix,
             rc = asprintf(output, "%sPMIX_VALUE: Data type: DATA_ARRAY\tARRAY SIZE: %ld",
                           prefx, (long)src->data.darray->size);
             break;
+        case PMIX_REGATTR:
+            r = (pmix_regattr_t*)src->data.ptr;
+            rc = asprintf(output, "%sPMIX_VALUE: Data type: PMIX_REGATTR\tName: %s\tString: %s",
+                          prefx, (NULL == r->name) ? "NULL" : r->name,
+                          (0 == strlen(r->string)) ? "NULL" : r->string);
+            break;
+        case PMIX_ALLOC_DIRECTIVE:
+            rc = pmix_bfrops_base_print_alloc_directive(output, prefx, &src->data.adir, PMIX_ALLOC_DIRECTIVE);
+            break;
         case PMIX_ENVAR:
             rc = asprintf(output, "%sPMIX_VALUE: Data type: PMIX_ENVAR\tName: %s\tValue: %s\tSeparator: %c",
                           prefx, (NULL == src->data.envar.envar) ? "NULL" : src->data.envar.envar,
                           (NULL == src->data.envar.value) ? "NULL" : src->data.envar.value,
                           src->data.envar.separator);
             break;
-
-        /**** DEPRECATED ****/
-        case PMIX_INFO_ARRAY:
-            rc = asprintf(output, "%sPMIX_VALUE: Data type: INFO_ARRAY\tARRAY SIZE: %ld",
-                          prefx, (long)src->data.array->size);
+        case PMIX_COORD:
+            if (PMIX_COORD_VIEW_UNDEF == src->data.coord->view) {
+                tp = "UNDEF";
+            } else if (PMIX_COORD_LOGICAL_VIEW == src->data.coord->view) {
+                tp = "LOGICAL";
+            } else if (PMIX_COORD_PHYSICAL_VIEW == src->data.coord->view) {
+                tp = "PHYSICAL";
+            } else {
+                tp = "UNRECOGNIZED";
+            }
+            rc = asprintf(output, "%sPMIX_VALUE: Data type: PMIX_COORD\tView: %s\tDims: %lu",
+                          prefx, tp, (unsigned long)src->data.coord->dims);
+            break;
+        case PMIX_LINK_STATE:
+            rc = pmix_bfrops_base_print_linkstate(output, prefx, &src->data.linkstate, PMIX_LINK_STATE);
+            break;
+        case PMIX_JOB_STATE:
+            rc = pmix_bfrops_base_print_jobstate(output, prefx, &src->data.jstate, PMIX_JOB_STATE);
+            break;
+        case PMIX_TOPO:
+            rc = pmix_bfrops_base_print_topology(output, prefx, src->data.topo, PMIX_TOPO);
+            break;
+        case PMIX_PROC_CPUSET:
+            rc = pmix_bfrops_base_print_cpuset(output, prefx, src->data.cpuset, PMIX_PROC_CPUSET);
+            break;
+        case PMIX_LOCTYPE:
+            rc = pmix_bfrops_base_print_locality(output, prefx, &src->data.locality, PMIX_LOCTYPE);
+            break;
+        case PMIX_GEOMETRY:
+            rc = pmix_bfrops_base_print_geometry(output, prefx, src->data.geometry, PMIX_GEOMETRY);
+            break;
+        case PMIX_DEVTYPE:
+            rc = pmix_bfrops_base_print_devtype(output, prefx, &src->data.devtype, PMIX_DEVTYPE);
+            break;
+        case PMIX_DEVICE_DIST:
+            rc = pmix_bfrops_base_print_devdist(output, prefx, src->data.devdist, PMIX_DEVICE_DIST);
+            break;
+        case PMIX_ENDPOINT:
+            rc = pmix_bfrops_base_print_endpoint(output, prefx, src->data.endpoint, PMIX_ENDPOINT);
             break;
-        /********************/
         default:
             rc = asprintf(output, "%sPMIX_VALUE: Data type: UNKNOWN\tValue: UNPRINTABLE", prefx);
             break;
@@ -1047,6 +1154,9 @@ int pmix_bfrops_base_print_info(char **output, char *prefix,
     char *tmp=NULL, *tmp2=NULL;
     int ret;
 
+    if (PMIX_INFO != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     pmix_bfrops_base_print_value(&tmp, NULL, &src->value, PMIX_VALUE);
     pmix_bfrops_base_print_info_directives(&tmp2, NULL, &src->flags, PMIX_INFO_DIRECTIVES);
     ret = asprintf(output, "%sKEY: %s\n%s\t%s\n%s\t%s", prefix, src->key,
@@ -1066,6 +1176,9 @@ int pmix_bfrops_base_print_pdata(char **output, char *prefix,
     char *tmp1, *tmp2;
     int ret;
 
+    if (PMIX_PDATA != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     pmix_bfrops_base_print_proc(&tmp1, NULL, &src->proc, PMIX_PROC);
     pmix_bfrops_base_print_value(&tmp2, NULL, &src->value, PMIX_VALUE);
     ret = asprintf(output, "%s  %s  KEY: %s %s", prefix, tmp1, src->key,
@@ -1086,12 +1199,20 @@ int pmix_bfrops_base_print_pdata(char **output, char *prefix,
 int pmix_bfrops_base_print_buf(char **output, char *prefix,
                                pmix_buffer_t *src, pmix_data_type_t type)
 {
+    if (NULL == output || NULL == prefix ||
+        NULL == src || PMIX_BUFFER != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     return PMIX_SUCCESS;
 }
 
 int pmix_bfrops_base_print_app(char **output, char *prefix,
                                pmix_app_t *src, pmix_data_type_t type)
 {
+    if (NULL == output || NULL == prefix ||
+        NULL == src || PMIX_APP != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     return PMIX_SUCCESS;
 }
 
@@ -1101,6 +1222,9 @@ int pmix_bfrops_base_print_proc(char **output, char *prefix,
     char *prefx;
     int rc;
 
+    if (PMIX_PROC != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1140,12 +1264,10 @@ int pmix_bfrops_base_print_proc(char **output, char *prefix,
 int pmix_bfrops_base_print_kval(char **output, char *prefix,
                                 pmix_kval_t *src, pmix_data_type_t type)
 {
-    return PMIX_SUCCESS;
-}
-
-pmix_status_t pmix_bfrops_base_print_modex(char **output, char *prefix,
-                                           pmix_modex_data_t *src, pmix_data_type_t type)
-{
+    if (NULL == output || NULL == prefix ||
+        NULL == src || PMIX_KVAL != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     return PMIX_SUCCESS;
 }
 
@@ -1154,6 +1276,9 @@ int pmix_bfrops_base_print_persist(char **output, char *prefix,
 {
     char *prefx;
 
+    if (PMIX_PERSIST != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1190,6 +1315,9 @@ pmix_status_t pmix_bfrops_base_print_scope(char **output, char *prefix,
 {
     char *prefx;
 
+    if (PMIX_SCOPE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1216,6 +1344,9 @@ pmix_status_t pmix_bfrops_base_print_range(char **output, char *prefix,
 {
     char *prefx;
 
+    if (PMIX_DATA_RANGE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1241,6 +1372,9 @@ pmix_status_t pmix_bfrops_base_print_cmd(char **output, char *prefix,
 {
     char *prefx;
 
+    if (PMIX_COMMAND != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1267,6 +1401,9 @@ pmix_status_t pmix_bfrops_base_print_info_directives(char **output, char *prefix
 {
     char *prefx;
 
+    if (PMIX_INFO_DIRECTIVES != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1294,6 +1431,9 @@ pmix_status_t pmix_bfrops_base_print_datatype(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_DATA_TYPE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1334,6 +1474,10 @@ int pmix_bfrops_base_print_bo(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_BYTE_OBJECT != type &&
+        PMIX_COMPRESSED_BYTE_OBJECT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1345,7 +1489,8 @@ int pmix_bfrops_base_print_bo(char **output, char *prefix,
 
     /* if src is NULL, just print data type and return */
     if (NULL == src) {
-        ret = asprintf(output, "%sData type: PMIX_BYTE_OBJECT\tValue: NULL pointer", prefx);
+        ret = asprintf(output, "%sData type: %s\tValue: NULL pointer", prefx,
+                       (PMIX_COMPRESSED_BYTE_OBJECT == type) ? "PMIX_COMPRESSED_BYTE_OBJECT" : "PMIX_BYTE_OBJECT");
         if (prefx != prefix) {
             free(prefx);
         }
@@ -1356,7 +1501,9 @@ int pmix_bfrops_base_print_bo(char **output, char *prefix,
         }
     }
 
-    ret = asprintf(output, "%sData type: PMIX_BYTE_OBJECT\tSize: %ld", prefx, (long)src->size);
+    ret = asprintf(output, "%sData type: %s\tSize: %ld", prefx,
+                   (PMIX_COMPRESSED_BYTE_OBJECT == type) ? "PMIX_COMPRESSED_BYTE_OBJECT" : "PMIX_BYTE_OBJECT",
+                   (long)src->size);
     if (prefx != prefix) {
         free(prefx);
     }
@@ -1374,6 +1521,9 @@ int pmix_bfrops_base_print_ptr(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_POINTER != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1402,6 +1552,9 @@ pmix_status_t pmix_bfrops_base_print_pstate(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_PROC_STATE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1432,6 +1585,9 @@ pmix_status_t pmix_bfrops_base_print_pinfo(char **output, char *prefix,
     pmix_status_t rc = PMIX_SUCCESS;
     char *p2, *tmp;
 
+    if (PMIX_PROC_INFO != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1474,6 +1630,9 @@ pmix_status_t pmix_bfrops_base_print_darray(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_DATA_ARRAY != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1505,6 +1664,9 @@ pmix_status_t pmix_bfrops_base_print_query(char **output, char *prefix,
     char *tmp, *t2, *t3;
     size_t n;
 
+    if (PMIX_QUERY != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1576,6 +1738,9 @@ pmix_status_t pmix_bfrops_base_print_rank(char **output, char *prefix,
     char *prefx;
     int rc;
 
+    if (PMIX_PROC_RANK != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1621,6 +1786,9 @@ pmix_status_t pmix_bfrops_base_print_alloc_directive(char **output, char *prefix
     char *prefx;
     int ret;
 
+    if (PMIX_ALLOC_DIRECTIVE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1650,6 +1818,9 @@ pmix_status_t pmix_bfrops_base_print_iof_channel(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_IOF_CHANNEL != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1679,6 +1850,9 @@ pmix_status_t pmix_bfrops_base_print_envar(char **output, char *prefix,
     char *prefx;
     int ret;
 
+    if (PMIX_ENVAR != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
     /* deal with NULL prefix */
     if (NULL == prefix) {
         if (0 > asprintf(&prefx, " ")) {
@@ -1703,36 +1877,487 @@ pmix_status_t pmix_bfrops_base_print_envar(char **output, char *prefix,
     }
 }
 
-
-/**** DEPRECATED ****/
-pmix_status_t pmix_bfrops_base_print_array(char **output, char *prefix,
-                                           pmix_info_array_t *src, pmix_data_type_t type)
+pmix_status_t pmix_bfrops_base_print_coord(char **output, char *prefix,
+                                           pmix_coord_t *src,
+                                           pmix_data_type_t type)
 {
-    size_t j;
-    char *tmp, *tmp2, *tmp3, *pfx;
-    pmix_info_t *s1;
+    char *prefx;
+    int ret;
+    char *tp;
 
-    if (0 > asprintf(&tmp, "%sARRAY SIZE: %ld", prefix, (long)src->size)) {
-        return PMIX_ERR_NOMEM;
+    if (PMIX_COORD != type) {
+        return PMIX_ERR_BAD_PARAM;
     }
-    if (0 > asprintf(&pfx, "\n%s\t",  (NULL == prefix) ? "" : prefix)) {
-        free(tmp);
-        return PMIX_ERR_NOMEM;
-    }
-    s1 = (pmix_info_t*)src->array;
-
-    for (j=0; j < src->size; j++) {
-        pmix_bfrops_base_print_info(&tmp2, pfx, &s1[j], PMIX_INFO);
-        if (0 > asprintf(&tmp3, "%s%s", tmp, tmp2)) {
-            free(tmp);
-            free(tmp2);
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
             return PMIX_ERR_NOMEM;
         }
-        free(tmp);
-        free(tmp2);
-        tmp = tmp3;
+    } else {
+        prefx = prefix;
     }
-    *output = tmp;
+
+    if (PMIX_COORD_VIEW_UNDEF == src->view) {
+        tp = "UNDEF";
+    } else if (PMIX_COORD_LOGICAL_VIEW == src->view) {
+        tp = "LOGICAL";
+    } else if (PMIX_COORD_PHYSICAL_VIEW == src->view) {
+        tp = "PHYSICAL";
+    } else {
+        tp = "UNRECOGNIZED";
+    }
+    ret = asprintf(output, "%sData type: PMIX_COORD\tView: %s\tDims: %lu",
+                   prefx, tp, (unsigned long)src->dims);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_regattr(char **output, char *prefix,
+                                             pmix_regattr_t *src,
+                                             pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_REGATTR != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_REGATTR\tName: %s\tString: %s",
+                   prefx, (NULL == src->name) ? "NULL" : src->name,
+                   (0 == strlen(src->string)) ? "NULL" : src->string);
+
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_regex(char **output, char *prefix,
+                                           char *src,
+                                           pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_REGEX != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_REGEX\tName: %s", prefx, src);
+
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_jobstate(char **output, char *prefix,
+                                              pmix_job_state_t *src,
+                                              pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_JOB_STATE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_JOB_STATE\tValue: %s",
+                   prefx, PMIx_Job_state_string(*src));
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_linkstate(char **output, char *prefix,
+                                               pmix_link_state_t *src,
+                                               pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_LINK_STATE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_LINK_STATE\tValue: %s",
+                   prefx, PMIx_Link_state_string(*src));
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_cpuset(char **output, char *prefix,
+                                            pmix_cpuset_t *src,
+                                            pmix_data_type_t type)
+{
+    char *prefx, *string;
+    int ret;
+
+    if (PMIX_PROC_CPUSET != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    string = pmix_ploc.print_cpuset(src);
+    if (NULL == string) {
+        return PMIX_ERR_NOT_SUPPORTED;
+    }
+
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            free(string);
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_CPUSET\tValue: %s",
+                   prefx, string);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+    free(string);
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_geometry(char **output, char *prefix,
+                                              pmix_geometry_t *src,
+                                              pmix_data_type_t type)
+{
+    char *prefx, *tmp, *pfx2, **result = NULL;
+    int ret;
+    size_t n;
+
+    if (PMIX_GEOMETRY != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+    if (0 > asprintf(&pfx2, "%s\t", prefx)) {
+        if (prefx != prefix) {
+            free(prefx);
+        }
+        return PMIX_ERR_NOMEM;
+    }
+
+    ret = asprintf(&tmp, "%sData type: PMIX_GEOMETRY\tValue: Fabric: %"PRIsize_t" UUID: %s OSName: %s",
+                   prefx, src->fabric, (NULL == src->uuid) ? "NULL" : src->uuid,
+                   (NULL == src->osname) ? "NULL" : src->osname);
+    if (0 > ret) {
+        if (prefx != prefix) {
+            free(prefx);
+        }
+        free(pfx2);
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    }
+    pmix_argv_append_nosize(&result, tmp);
+    free(tmp);
+
+    for (n=0; n < src->ncoords; n++) {
+        ret = pmix_bfrops_base_print_coord(&tmp, pfx2, &src->coordinates[n], PMIX_COORD);
+        if (PMIX_SUCCESS != ret) {
+            if (prefx != prefix) {
+                free(prefx);
+            }
+            free(pfx2);
+            if (NULL != result) {
+                pmix_argv_free(result);
+            }
+            return ret;
+        }
+        pmix_argv_append_nosize(&result, tmp);
+        free(tmp);
+    }
+
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    *output = pmix_argv_join(result, '\n');
+    pmix_argv_free(result);
+
     return PMIX_SUCCESS;
 }
-/********************/
+
+pmix_status_t pmix_bfrops_base_print_devdist(char **output, char *prefix,
+                                             pmix_device_distance_t *src,
+                                             pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_DEVICE_DIST != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_DEVICE_DIST\tValue: UUID: %s OSName: %s Type: %s Min: %u Max: %u",
+                   prefx, (NULL == src->uuid) ? "NULL" : src->uuid,
+                   (NULL == src->osname) ? "NULL" : src->osname,
+                   PMIx_Device_type_string(src->type),
+                   (unsigned)src->mindist, (unsigned)src->maxdist);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_endpoint(char **output, char *prefix,
+                                              pmix_endpoint_t *src,
+                                              pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_ENDPOINT != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_ENDPOINT\tValue: %s #bytes: %"PRIsize_t,
+                   prefx, (NULL == src->uuid) ? "NULL" : src->uuid, src->endpt.size);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_topology(char **output, char *prefix,
+                                              pmix_topology_t *src,
+                                              pmix_data_type_t type)
+{
+    char *prefx, *string;
+    int ret;
+
+    if (PMIX_TOPO != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    string = pmix_ploc.print_topology(src);
+    if (NULL == string) {
+        return PMIX_ERR_NOT_SUPPORTED;
+    }
+
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            free(string);
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_TOPO\tValue: %s",
+                   prefx, string);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+    free(string);
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_devtype(char **output, char *prefix,
+                                             pmix_device_type_t *src,
+                                             pmix_data_type_t type)
+{
+    char *prefx;
+    int ret;
+
+    if (PMIX_DEVTYPE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_DEVICE_TYPE\tValue: 0x%"PRIx64,
+                   prefx, (uint64_t)src);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
+
+pmix_status_t pmix_bfrops_base_print_locality(char **output, char *prefix,
+                                              pmix_locality_t *src,
+                                              pmix_data_type_t type)
+{
+    char *prefx, **tmp = NULL, *str;
+    int ret;
+
+    if (PMIX_LOCTYPE != type) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    /* deal with NULL prefix */
+    if (NULL == prefix) {
+        if (0 > asprintf(&prefx, " ")) {
+            return PMIX_ERR_NOMEM;
+        }
+    } else {
+        prefx = prefix;
+    }
+
+    if (PMIX_LOCALITY_UNKNOWN == *src) {
+        str = strdup("UNKNOWN");
+    } else if (PMIX_LOCALITY_NONLOCAL == *src) {
+        str = strdup("NONLOCAL");
+    } else {
+        if (PMIX_LOCALITY_SHARE_HWTHREAD & *src) {
+            pmix_argv_append_nosize(&tmp, "HWTHREAD");
+        }
+        if (PMIX_LOCALITY_SHARE_CORE & *src) {
+            pmix_argv_append_nosize(&tmp, "CORE");
+        }
+        if (PMIX_LOCALITY_SHARE_L1CACHE & *src) {
+            pmix_argv_append_nosize(&tmp, "L1");
+        }
+        if (PMIX_LOCALITY_SHARE_L2CACHE & *src) {
+            pmix_argv_append_nosize(&tmp, "L2");
+        }
+        if (PMIX_LOCALITY_SHARE_L3CACHE & *src) {
+            pmix_argv_append_nosize(&tmp, "L3");
+        }
+        if (PMIX_LOCALITY_SHARE_PACKAGE & *src) {
+            pmix_argv_append_nosize(&tmp, "CORE");
+        }
+        if (PMIX_LOCALITY_SHARE_NUMA & *src) {
+            pmix_argv_append_nosize(&tmp, "NUMA");
+        }
+        if (PMIX_LOCALITY_SHARE_NODE & *src) {
+            pmix_argv_append_nosize(&tmp, "NODE");
+        }
+        str = pmix_argv_join(tmp, ':');
+        pmix_argv_free(tmp);
+    }
+
+    ret = asprintf(output, "%sData type: PMIX_LOCALITY\tValue: %s",
+                   prefx, str);
+    if (prefx != prefix) {
+        free(prefx);
+    }
+    free(str);
+
+    if (0 > ret) {
+        return PMIX_ERR_OUT_OF_RESOURCE;
+    } else {
+        return PMIX_SUCCESS;
+    }
+}
