@@ -3,7 +3,7 @@
  * Copyright (c) 2010-2013 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2016-2018 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2016-2019 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -73,7 +73,7 @@ BEGIN_C_DECLS
 
 typedef struct pmix_pif_t {
     pmix_list_item_t     super;
-    char                if_name[IF_NAMESIZE+1];
+    char                if_name[PMIX_IF_NAMESIZE+1];
     int                 if_index;
     uint16_t            if_kernel_index;
     uint16_t            af_family;
