@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
- * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2018      Research Organization for Information Science
@@ -13,9 +13,9 @@
  * $HEADER$
  */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
-#include <pmix_common.h>
+#include "include/pmix_common.h"
 #include "src/include/pmix_globals.h"
 
 #include "src/class/pmix_list.h"
@@ -28,7 +28,7 @@
 #include "src/mca/pnet/base/base.h"
 
 
-static pmix_status_t process_maps(char *nspace, char *nregex, char *pregex);
+static pmix_status_t process_maps(char *nspace, char **nodes, char **procs);
 
 /* NOTE: a tool (e.g., prun) may call this function to
  * harvest local envars for inclusion in a call to
@@ -39,11 +39,10 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
                                       pmix_list_t *ilist)
 {
     pmix_pnet_base_active_module_t *active;
-    pmix_status_t rc = PMIX_SUCCESS;
+    pmix_status_t rc;
     pmix_namespace_t *nptr, *ns;
     size_t n;
-    char *nregex, *pregex;
-    char *params[2] = {"PMIX_MCA_", NULL};
+    char **nodes, **procs;
 
     if (!pmix_pnet_globals.initialized) {
         return PMIX_ERR_INIT;
@@ -56,7 +55,14 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
     if (NULL == nspace || NULL == ilist) {
         return PMIX_ERR_BAD_PARAM;
     }
-    if (PMIX_PROC_IS_GATEWAY(pmix_globals.mypeer)) {
+
+    if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+        PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+        if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+            PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+            return PMIX_SUCCESS;
+        }
+
         nptr = NULL;
         /* find this nspace - note that it may not have
          * been registered yet */
@@ -70,6 +76,7 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
             /* add it */
             nptr = PMIX_NEW(pmix_namespace_t);
             if (NULL == nptr) {
+                PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
                 return PMIX_ERR_NOMEM;
             }
             nptr->nspace = strdup(nspace);
@@ -78,16 +85,24 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
 
         if (NULL != info) {
             /* check for description of the node and proc maps */
-            nregex = NULL;
-            pregex = NULL;
+            nodes = NULL;
+            procs = NULL;
             for (n=0; n < ninfo; n++) {
-                if (0 == strncmp(info[n].key, PMIX_NODE_MAP, PMIX_MAX_KEYLEN)) {
-                    nregex = info[n].value.data.string;
-                } else if (0 == strncmp(info[n].key, PMIX_PROC_MAP, PMIX_MAX_KEYLEN)) {
-                    pregex = info[n].value.data.string;
+                if (PMIX_CHECK_KEY(&info[n], PMIX_NODE_MAP)) {
+                    rc = pmix_preg.parse_nodes(info[n].value.data.bo.bytes, &nodes);
+                    if (PMIX_SUCCESS != rc) {
+                        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+                        return rc;
+                    }
+                } else if (PMIX_CHECK_KEY(&info[n], PMIX_PROC_MAP)) {
+                    rc = pmix_preg.parse_procs(info[n].value.data.bo.bytes, &procs);
+                    if (PMIX_SUCCESS != rc) {
+                        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+                        return rc;
+                    }
                 }
             }
-            if (NULL != nregex && NULL != pregex) {
+            if (NULL != nodes && NULL != procs) {
                 /* assemble the pnet node and proc descriptions
                  * NOTE: this will eventually be folded into the
                  * new shared memory system, but we do it here
@@ -95,32 +110,32 @@ pmix_status_t pmix_pnet_base_allocate(char *nspace,
                  * the host will not have registered the clients
                  * and nspace prior to calling allocate
                  */
-                rc = process_maps(nspace, nregex, pregex);
+                rc = process_maps(nspace, nodes, procs);
+                pmix_argv_free(nodes);
+                pmix_argv_free(procs);
                 if (PMIX_SUCCESS != rc) {
+                    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
                     return rc;
                 }
             }
-            /* process the allocation request */
-            for (n=0; n < ninfo; n++) {
-                PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
-                    if (NULL != active->module->allocate) {
-                        if (PMIX_SUCCESS == (rc = active->module->allocate(nptr, &info[n], ilist))) {
-                            break;
-                        }
-                        if (PMIX_ERR_TAKE_NEXT_OPTION != rc) {
-                            /* true error */
-                            return rc;
-                        }
-                    }
+        }
+        /* process the allocation request */
+        PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
+            if (NULL != active->module->allocate) {
+                if (PMIX_SUCCESS == (rc = active->module->allocate(nptr, info, ninfo, ilist))) {
+                    continue;
+                }
+                if (PMIX_ERR_TAKE_NEXT_OPTION != rc) {
+                    /* true error */
+                    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+                    return rc;
                 }
             }
         }
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
     }
 
-    /* add any local PMIx MCA params */
-    rc = pmix_pnet_base_harvest_envars(params, NULL, ilist);
-
-    return rc;
+    return PMIX_SUCCESS;
 }
 
 /* can only be called by a server */
@@ -144,6 +159,12 @@ pmix_status_t pmix_pnet_base_setup_local_network(char *nspace,
         return PMIX_ERR_BAD_PARAM;
     }
 
+    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+        return PMIX_SUCCESS;
+    }
+
     /* find this proc's nspace object */
     nptr = NULL;
     PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
@@ -156,6 +177,7 @@ pmix_status_t pmix_pnet_base_setup_local_network(char *nspace,
         /* add it */
         nptr = PMIX_NEW(pmix_namespace_t);
         if (NULL == nptr) {
+            PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
             return PMIX_ERR_NOMEM;
         }
         nptr->nspace = strdup(nspace);
@@ -165,11 +187,13 @@ pmix_status_t pmix_pnet_base_setup_local_network(char *nspace,
     PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
         if (NULL != active->module->setup_local_network) {
             if (PMIX_SUCCESS != (rc = active->module->setup_local_network(nptr, info, ninfo))) {
+                PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
                 return rc;
             }
         }
     }
 
+    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
     return PMIX_SUCCESS;
 }
 
@@ -184,11 +208,20 @@ pmix_status_t pmix_pnet_base_setup_fork(const pmix_proc_t *proc, char ***env)
         return PMIX_ERR_INIT;
     }
 
+    pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                        "pnet: setup_fork called");
+
     /* protect against bozo inputs */
     if (NULL == proc || NULL == env) {
         return PMIX_ERR_BAD_PARAM;
     }
 
+    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+        return PMIX_SUCCESS;
+    }
+
     /* find this proc's nspace object */
     nptr = NULL;
     PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
@@ -201,6 +234,7 @@ pmix_status_t pmix_pnet_base_setup_fork(const pmix_proc_t *proc, char ***env)
         /* add it */
         nptr = PMIX_NEW(pmix_namespace_t);
         if (NULL == nptr) {
+            PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
             return PMIX_ERR_NOMEM;
         }
         nptr->nspace = strdup(proc->nspace);
@@ -211,11 +245,13 @@ pmix_status_t pmix_pnet_base_setup_fork(const pmix_proc_t *proc, char ***env)
         if (NULL != active->module->setup_fork) {
             rc = active->module->setup_fork(nptr, proc, env);
             if (PMIX_SUCCESS != rc && PMIX_ERR_NOT_AVAILABLE != rc) {
+                PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
                 return rc;
             }
         }
     }
 
+    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
     return PMIX_SUCCESS;
 }
 
@@ -227,18 +263,28 @@ void pmix_pnet_base_child_finalized(pmix_proc_t *peer)
         return;
     }
 
+    pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                        "pnet: child_finalized called");
+
     /* protect against bozo inputs */
     if (NULL == peer) {
         PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
         return;
     }
 
+    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+        return;
+    }
+
     PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
         if (NULL != active->module->child_finalized) {
             active->module->child_finalized(peer);
         }
     }
 
+    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
     return;
 }
 
@@ -250,17 +296,27 @@ void pmix_pnet_base_local_app_finalized(pmix_namespace_t *nptr)
         return;
     }
 
+    pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                        "pnet: local_app_finalized called");
+
     /* protect against bozo inputs */
     if (NULL == nptr) {
         return;
     }
 
+    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+        return;
+    }
+
     PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
         if (NULL != active->module->local_app_finalized) {
             active->module->local_app_finalized(nptr);
         }
     }
 
+    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
     return;
 }
 
@@ -275,11 +331,20 @@ void pmix_pnet_base_deregister_nspace(char *nspace)
         return;
     }
 
+    pmix_output_verbose(2, pmix_pnet_base_framework.framework_output,
+                        "pnet: deregister_nspace called");
+
     /* protect against bozo inputs */
     if (NULL == nspace) {
         return;
     }
 
+    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+        return;
+    }
+
     /* find this nspace object */
     nptr = NULL;
     PMIX_LIST_FOREACH(ns, &pmix_globals.nspaces, pmix_namespace_t) {
@@ -290,6 +355,7 @@ void pmix_pnet_base_deregister_nspace(char *nspace)
     }
     if (NULL == nptr) {
         /* nothing we can do */
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
         return;
     }
 
@@ -317,6 +383,7 @@ void pmix_pnet_base_deregister_nspace(char *nspace)
             }
         }
     }
+    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
 }
 
 static void cicbfunc(pmix_status_t status,
@@ -376,6 +443,11 @@ void pmix_pnet_base_collect_inventory(pmix_info_t directives[], size_t ndirs,
         }
         return;
     }
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        cbfunc(PMIX_SUCCESS, NULL, cbdata);
+        return;
+    }
+
     /* create the rollup object */
     myrollup = PMIX_NEW(pmix_inventory_rollup_t);
     if (NULL == myrollup) {
@@ -479,6 +551,11 @@ void pmix_pnet_base_deliver_inventory(pmix_info_t info[], size_t ninfo,
         }
         return;
     }
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        cbfunc(PMIX_SUCCESS, cbdata);
+        return;
+    }
+
     /* create the rollup object */
     myrollup = PMIX_NEW(pmix_inventory_rollup_t);
     if (NULL == myrollup) {
@@ -531,76 +608,130 @@ void pmix_pnet_base_deliver_inventory(pmix_info_t info[], size_t ninfo,
     return;
 }
 
-pmix_status_t pmix_pnet_base_harvest_envars(char **incvars, char **excvars,
-                                            pmix_list_t *ilist)
+pmix_status_t pmix_pnet_base_register_fabric(pmix_fabric_t *fabric,
+                                             const pmix_info_t directives[],
+                                             size_t ndirs,
+                                             pmix_op_cbfunc_t cbfunc, void *cbdata)
 {
-    int i, j;
-    size_t len;
-    pmix_kval_t *kv, *next;
-    char *cs_env, *string_key;
+    pmix_pnet_base_active_module_t *active;
+    pmix_status_t rc;
+    pmix_pnet_fabric_t *ft;
 
-    /* harvest envars to pass along */
-    for (j=0; NULL != incvars[j]; j++) {
-        len = strlen(incvars[j]);
-        if ('*' == incvars[j][len-1]) {
-            --len;
-        }
-        for (i = 0; NULL != environ[i]; ++i) {
-            if (0 == strncmp(environ[i], incvars[j], len)) {
-                cs_env = strdup(environ[i]);
-                kv = PMIX_NEW(pmix_kval_t);
-                if (NULL == kv) {
-                    free(cs_env);
-                    return PMIX_ERR_OUT_OF_RESOURCE;
+    /* ensure our fields of the fabric object are initialized */
+    fabric->info = NULL;
+    fabric->ninfo = 0;
+    fabric->module = NULL;
+
+    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
+
+    if (0 == pmix_list_get_size(&pmix_pnet_globals.actives)) {
+        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+        return PMIX_ERR_NOT_SUPPORTED;
+    }
+
+    /* scan across active modules until one returns success */
+    PMIX_LIST_FOREACH(active, &pmix_pnet_globals.actives, pmix_pnet_base_active_module_t) {
+        if (NULL != active->module->register_fabric) {
+            rc = active->module->register_fabric(fabric, directives, ndirs, cbfunc, cbdata);
+            if (PMIX_OPERATION_SUCCEEDED == rc) {
+                /* track this fabric so we can respond to remote requests */
+                ft = PMIX_NEW(pmix_pnet_fabric_t);
+                ft->index = fabric->index;
+                if (NULL != fabric->name) {
+                    ft->name = strdup(fabric->name);
                 }
-                kv->key = strdup(PMIX_SET_ENVAR);
-                kv->value = (pmix_value_t*)malloc(sizeof(pmix_value_t));
-                if (NULL == kv->value) {
-                    PMIX_RELEASE(kv);
-                    free(cs_env);
-                    return PMIX_ERR_OUT_OF_RESOURCE;
-                }
-                kv->value->type = PMIX_ENVAR;
-                string_key = strchr(cs_env, '=');
-                if (NULL == string_key) {
-                    free(cs_env);
-                    PMIX_RELEASE(kv);
-                    return PMIX_ERR_BAD_PARAM;
-                }
-                *string_key = '\0';
-                ++string_key;
-                pmix_output_verbose(5, pmix_pnet_base_framework.framework_output,
-                                    "pnet: adding envar %s", cs_env);
-                PMIX_ENVAR_LOAD(&kv->value->data.envar, cs_env, string_key, ':');
-                pmix_list_append(ilist, &kv->super);
-                free(cs_env);
+                ft->module = active->module;
+                pmix_list_append(&pmix_pnet_globals.fabrics, &ft->super);
+            } else if (PMIX_ERR_TAKE_NEXT_OPTION != rc) {
+                /* just return the result */
+                PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+                return rc;
             }
         }
     }
 
-    /* now check the exclusions and remove any that match */
-    if (NULL != excvars) {
-        for (j=0; NULL != excvars[j]; j++) {
-            len = strlen(excvars[j]);
-            if ('*' == excvars[j][len-1]) {
-                --len;
-            }
-            PMIX_LIST_FOREACH_SAFE(kv, next, ilist, pmix_kval_t) {
-                if (0 == strncmp(kv->value->data.envar.envar, excvars[j], len)) {
-                pmix_output_verbose(5, pmix_pnet_base_framework.framework_output,
-                                    "pnet: excluding envar %s", kv->value->data.envar.envar);
-                    pmix_list_remove_item(ilist, &kv->super);
-                    PMIX_RELEASE(kv);
-                }
-            }
-        }
-    }
-    return PMIX_SUCCESS;
+    /* unlock prior to return */
+    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
+
+    return PMIX_ERR_NOT_FOUND;
 }
 
-static pmix_status_t process_maps(char *nspace, char *nregex, char *pregex)
+pmix_status_t pmix_pnet_base_update_fabric(pmix_fabric_t *fabric)
 {
-    char **nodes, **procs, **ranks;
+    pmix_status_t rc = PMIX_SUCCESS;
+    pmix_pnet_fabric_t *active;
+    pmix_pnet_module_t *module = NULL;
+    pmix_pnet_fabric_t *ft;
+
+    /* protect against bozo input */
+    if (NULL == fabric) {
+        return PMIX_ERR_BAD_PARAM;
+    } else if (NULL == fabric->module) {
+        /* this might be a remote request, so look at the
+         * list of fabrics we have registered locally and
+         * see if we have one with the matching index */
+        PMIX_LIST_FOREACH(ft, &pmix_pnet_globals.fabrics, pmix_pnet_fabric_t) {
+            if (fabric->index == ft->index) {
+                module = ft->module;
+            } else if (NULL != fabric->name && NULL != ft->name
+                       && 0 == strcmp(ft->name, fabric->name)) {
+                module = ft->module;
+            }
+        }
+    } else {
+        active = (pmix_pnet_fabric_t*)fabric->module;
+        module = (pmix_pnet_module_t*)active->module;
+    }
+    if (NULL == module) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    if (NULL != module->update_fabric) {
+        rc = module->update_fabric(fabric);
+    }
+    return rc;
+}
+
+pmix_status_t pmix_pnet_base_deregister_fabric(pmix_fabric_t *fabric)
+{
+    pmix_status_t rc = PMIX_SUCCESS;
+    pmix_pnet_fabric_t *active;
+    pmix_pnet_module_t *module = NULL;
+    pmix_pnet_fabric_t *ft;
+
+    /* protect against bozo input */
+    if (NULL == fabric) {
+        return PMIX_ERR_BAD_PARAM;
+    } else if (NULL == fabric->module) {
+        /* this might be a remote request, so look at the
+         * list of fabrics we have registered locally and
+         * see if we have one with the matching index */
+        PMIX_LIST_FOREACH(ft, &pmix_pnet_globals.fabrics, pmix_pnet_fabric_t) {
+            if (fabric->index == ft->index) {
+                module = ft->module;
+            } else if (NULL != fabric->name && NULL != ft->name
+                       && 0 == strcmp(ft->name, fabric->name)) {
+                module = ft->module;
+            }
+        }
+    } else {
+        active = (pmix_pnet_fabric_t*)fabric->module;
+        module = (pmix_pnet_module_t*)active->module;
+    }
+    if (NULL == module) {
+        return PMIX_ERR_BAD_PARAM;
+    }
+
+    if (NULL != module->deregister_fabric) {
+        rc = module->deregister_fabric(fabric);
+    }
+    return rc;
+}
+
+
+static pmix_status_t process_maps(char *nspace, char **nodes, char **procs)
+{
+    char **ranks;
     pmix_status_t rc;
     size_t m, n;
     pmix_pnet_job_t *jptr, *job;
@@ -608,20 +739,10 @@ static pmix_status_t process_maps(char *nspace, char *nregex, char *pregex)
     pmix_pnet_local_procs_t *lp;
     bool needcheck;
 
-    PMIX_ACQUIRE_THREAD(&pmix_pnet_globals.lock);
-
-    /* parse the regex to get the argv array of node names */
-    if (PMIX_SUCCESS != (rc = pmix_preg.parse_nodes(nregex, &nodes))) {
+    /* bozo check */
+    if (pmix_argv_count(nodes) != pmix_argv_count(procs)) {
+        rc = PMIX_ERR_BAD_PARAM;
         PMIX_ERROR_LOG(rc);
-        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
-        return rc;
-    }
-
-    /* parse the regex to get the argv array of proc ranks on each node */
-    if (PMIX_SUCCESS != (rc = pmix_preg.parse_procs(pregex, &procs))) {
-        PMIX_ERROR_LOG(rc);
-        pmix_argv_free(nodes);
-        PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
         return rc;
     }
 
@@ -697,9 +818,5 @@ static pmix_status_t process_maps(char *nspace, char *nregex, char *pregex)
         pmix_argv_free(ranks);
     }
 
-    pmix_argv_free(nodes);
-    pmix_argv_free(procs);
-
-    PMIX_RELEASE_THREAD(&pmix_pnet_globals.lock);
     return PMIX_SUCCESS;
 }
