@@ -4,7 +4,7 @@
  *
  * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2018      Intel, Inc. All rights reserved.
+ * Copyright (c) 2018-2020 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,7 +26,8 @@
 #ifndef PMIX_PNET_H
 #define PMIX_PNET_H
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
+#include "include/pmix.h"
 
 #include "src/class/pmix_list.h"
 #include "src/mca/mca.h"
@@ -61,7 +62,7 @@ typedef void (*pmix_pnet_base_module_fini_fn_t)(void);
  * for forwarding to compute nodes, or allocation of static endpts
  */
 typedef pmix_status_t (*pmix_pnet_base_module_allocate_fn_t)(pmix_namespace_t *nptr,
-                                                             pmix_info_t *info,
+                                                             pmix_info_t info[], size_t ninfo,
                                                              pmix_list_t *ilist);
 
 /**
@@ -146,11 +147,34 @@ typedef pmix_status_t (*pmix_pnet_base_module_deliver_inventory_fn_t)(pmix_info_
                                                                       pmix_op_cbfunc_t cbfunc, void *cbdata);
 
 
+/* Register to provide cost information
+ *
+ * Scan the provided directives to identify if the module
+ * should service this request - directives could include
+ * ordered specification of fabric type or a direct request
+ * for a specific component
+ */
+typedef pmix_status_t (*pmix_pnet_base_module_register_fabric_fn_t)(pmix_fabric_t *fabric,
+                                                                    const pmix_info_t directives[],
+                                                                    size_t ndirs,
+                                                                    pmix_op_cbfunc_t cbfunc, void *cbdata);
+
+
+/* Update the fabric information */
+typedef pmix_status_t (*pmix_pnet_base_module_update_fabric_fn_t)(pmix_fabric_t *fabric);
+
+
+/* Deregister the fabric, giving the associated module a chance to cleanup */
+typedef pmix_status_t (*pmix_pnet_base_module_deregister_fabric_fn_t)(pmix_fabric_t *fabric);
+
 /**
- * Base structure for a PNET module
+ * Base structure for a PNET module. Each component should malloc a
+ * copy of the module structure for each fabric plane they support.
  */
 typedef struct {
     char *name;
+    /* provide a pointer to plane-specific metadata */
+    void *plane;
     /* init/finalize */
     pmix_pnet_base_module_init_fn_t                 init;
     pmix_pnet_base_module_fini_fn_t                 finalize;
@@ -162,6 +186,9 @@ typedef struct {
     pmix_pnet_base_module_dregister_nspace_fn_t     deregister_nspace;
     pmix_pnet_base_module_collect_inventory_fn_t    collect_inventory;
     pmix_pnet_base_module_deliver_inventory_fn_t    deliver_inventory;
+    pmix_pnet_base_module_register_fabric_fn_t      register_fabric;
+    pmix_pnet_base_module_update_fabric_fn_t        update_fabric;
+    pmix_pnet_base_module_deregister_fabric_fn_t    deregister_fabric;
 } pmix_pnet_module_t;
 
 
@@ -201,6 +228,9 @@ typedef struct {
     pmix_pnet_base_API_deregister_nspace_fn_t       deregister_nspace;
     pmix_pnet_base_API_collect_inventory_fn_t       collect_inventory;
     pmix_pnet_base_API_deliver_inventory_fn_t       deliver_inventory;
+    pmix_pnet_base_module_register_fabric_fn_t      register_fabric;
+    pmix_pnet_base_module_update_fabric_fn_t        update_fabric;
+    pmix_pnet_base_module_deregister_fabric_fn_t    deregister_fabric;
 } pmix_pnet_API_module_t;
 
 
