@@ -12,7 +12,7 @@
  * Copyright (c) 2008-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2010-2015 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
@@ -24,23 +24,27 @@
 
 /** @file **/
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
 #include "src/class/pmix_object.h"
 #include "src/client/pmix_client_ops.h"
+#include "src/common/pmix_attributes.h"
 #include "src/util/output.h"
 #include "src/util/keyval_parse.h"
 #include "src/util/show_help.h"
 #include "src/mca/base/base.h"
 #include "src/mca/base/pmix_mca_base_var.h"
 #include "src/mca/bfrops/base/base.h"
+#include "src/mca/pcompress/base/base.h"
 #include "src/mca/gds/base/base.h"
 #include "src/mca/pif/base/base.h"
 #include "src/mca/pinstalldirs/base/base.h"
+#include "src/mca/ploc/base/base.h"
 #include "src/mca/plog/base/base.h"
 #include "src/mca/pnet/base/base.h"
 #include "src/mca/preg/base/base.h"
 #include "src/mca/psec/base/base.h"
+#include "src/mca/psquash/base/base.h"
 #include "src/mca/ptl/base/base.h"
 #include PMIX_EVENT_HEADER
 
@@ -64,6 +68,11 @@ void pmix_rte_finalize(void)
         return;
     }
 
+    /* release the attribute support trackers */
+    pmix_release_registered_attrs();
+
+    /* close ploc */
+    (void)pmix_mca_base_framework_close(&pmix_ploc_base_framework);
 
     /* close plog */
     (void)pmix_mca_base_framework_close(&pmix_plog_base_framework);
@@ -80,6 +89,13 @@ void pmix_rte_finalize(void)
     /* close bfrops */
     (void)pmix_mca_base_framework_close(&pmix_bfrops_base_framework);
 
+    /* close the psquash framework */
+    pmix_psquash.finalize();
+    pmix_mca_base_framework_close(&pmix_psquash_base_framework);
+
+    /* close compress */
+    (void)pmix_mca_base_framework_close(&pmix_pcompress_base_framework);
+
     /* close GDS */
     (void)pmix_mca_base_framework_close(&pmix_gds_base_framework);
 
@@ -123,12 +139,12 @@ void pmix_rte_finalize(void)
     }
     PMIX_DESTRUCT(&pmix_globals.iof_requests);
     PMIX_LIST_DESTRUCT(&pmix_globals.stdin_targets);
-    free(pmix_globals.hostname);
+    if (NULL != pmix_globals.hostname) {
+        free(pmix_globals.hostname);
+    }
     PMIX_LIST_DESTRUCT(&pmix_globals.nspaces);
 
     /* now safe to release the event base */
-    if (!pmix_globals.external_evbase) {
-        (void)pmix_progress_thread_stop(NULL);
-    }
+    (void)pmix_progress_thread_stop(NULL);
 
 }
