@@ -14,7 +14,7 @@
  * Copyright (c) 2010-2016 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2011-2012 University of Houston. All rights reserved.
- * Copyright (c) 2016-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2016-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
  * Copyright (c) 2017      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
@@ -1327,11 +1327,13 @@ void pmix_info_do_config(bool want_all)
     char *debug;
     char *have_dl;
     char *symbol_visibility;
+    char *manpages;
 
     /* setup the strings that don't require allocations*/
     debug = PMIX_ENABLE_DEBUG ? "yes" : "no";
     have_dl = PMIX_HAVE_PDL_SUPPORT ? "yes" : "no";
     symbol_visibility = PMIX_HAVE_VISIBILITY ? "yes" : "no";
+    manpages = PMIX_ENABLE_MAN_PAGES ? "yes" : "no";
 
     /* output values */
     pmix_info_out("Configured by", "config:user", PMIX_CONFIGURE_USER);
@@ -1373,4 +1375,5 @@ void pmix_info_do_config(bool want_all)
     pmix_info_out("Internal debug support", "option:debug", debug);
     pmix_info_out("dl support", "option:dlopen", have_dl);
     pmix_info_out("Symbol vis. support", "options:visibility", symbol_visibility);
+    pmix_info_out("Manpages built", "options:man-pages", manpages);
 }
