@@ -15,7 +15,7 @@
  * Copyright (c) 2007-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
- * Copyright (c) 2014-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -32,7 +32,7 @@
  */
 
 #include "pmix_config.h"
-#include "pmix_common.h"
+#include "include/pmix_common.h"
 
 #include <stdio.h>
 #include <errno.h>
@@ -66,8 +66,8 @@
 #include "src/runtime/pmix_rte.h"
 
 #include "src/include/pmix_globals.h"
-#include <pmix_tool.h>
-#include <pmix.h>
+#include "include/pmix_tool.h"
+#include "include/pmix.h"
 
 typedef struct {
     pmix_lock_t lock;
@@ -263,6 +263,11 @@ main(int argc, char *argv[])
                 __FILE__, __LINE__, rc);
         return rc;
     }
+    if (PMIX_SUCCESS != (rc = pmix_pinstall_dirs_base_init(NULL, 0))) {
+        fprintf(stderr, "pmix_pinstalldirs_base_init() failed -- process will likely abort (%s:%d, returned %d instead of PMIX_SUCCESS)\n",
+                __FILE__, __LINE__, rc);
+        return rc;
+    }
 
     /* initialize the help system */
     pmix_show_help_init();
