@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2007-2012 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,7 +19,7 @@
  * $HEADER$
  */
 
-#include <src/include/pmix_config.h>
+#include "src/include/pmix_config.h"
 
 
 #ifdef HAVE_STRING_H
@@ -31,7 +31,7 @@
 #include <stdlib.h>
 #endif
 
-#include <pmix_common.h>
+#include "include/pmix_common.h"
 #include "src/include/pmix_globals.h"
 #include "src/util/error.h"
 
@@ -143,6 +143,8 @@ PMIX_EXPORT const char* PMIx_Error_string(pmix_status_t errnum)
         return "NOT-FOUND";
     case PMIX_ERR_NOT_SUPPORTED:
         return "NOT-SUPPORTED";
+    case PMIX_ERR_PARAM_VALUE_NOT_SUPPORTED:
+        return "PARAM-VALUE-NOT-SUPPORTED";
     case PMIX_ERR_NOT_IMPLEMENTED:
         return "NOT-IMPLEMENTED";
     case PMIX_ERR_COMM_FAILURE:
@@ -151,41 +153,44 @@ PMIX_EXPORT const char* PMIx_Error_string(pmix_status_t errnum)
         return "UNPACK-PAST-END";
     case PMIX_ERR_CONFLICTING_CLEANUP_DIRECTIVES:
         return "PMIX CONFLICTING CLEANUP DIRECTIVES";
+    case PMIX_ERR_PARTIAL_SUCCESS:
+        return "PARTIAL SUCCESS";
+    case PMIX_ERR_DUPLICATE_KEY:
+        return "DUPLICATE KEY";
+    case PMIX_ERR_EMPTY:
+        return "EMPTY";
+    case PMIX_PROCESS_SET_DEFINE:
+        return "PROCESS SET DEFINED";
+    case PMIX_ERR_IOF_FAILURE:
+        return "IOF FAILURE";
+    case PMIX_ERR_IOF_COMPLETE:
+        return "IOF COMPLETE";
 
-
-    case PMIX_ERR_LOST_CONNECTION_TO_SERVER:
-        return "LOST_CONNECTION_TO_SERVER";
-    case PMIX_ERR_LOST_PEER_CONNECTION:
-        return "LOST-PEER-CONNECTION";
-    case PMIX_ERR_LOST_CONNECTION_TO_CLIENT:
-        return "LOST-CONNECTION-TO-CLIENT";
-
-
+    case PMIX_ERR_LOST_CONNECTION:
+        return "LOST_CONNECTION";
+    case PMIX_ERR_EXISTS_OUTSIDE_SCOPE:
+        return "OUTSIDE-SCOPE";
     case PMIX_QUERY_PARTIAL_SUCCESS:
         return "QUERY-PARTIAL-SUCCESS";
-
-
     case PMIX_NOTIFY_ALLOC_COMPLETE:
         return "PMIX ALLOC OPERATION COMPLETE";
-
-
     case PMIX_JCTRL_CHECKPOINT:
         return "PMIX JOB CONTROL CHECKPOINT";
     case PMIX_JCTRL_CHECKPOINT_COMPLETE:
         return "PMIX JOB CONTROL CHECKPOINT COMPLETE";
     case PMIX_JCTRL_PREEMPT_ALERT:
         return "PMIX PRE-EMPTION ALERT";
-
-
     case PMIX_MONITOR_HEARTBEAT_ALERT:
         return "PMIX HEARTBEAT ALERT";
     case PMIX_MONITOR_FILE_ALERT:
         return "PMIX FILE MONITOR ALERT";
+    case PMIX_PROC_TERMINATED:
+        return "PROC-TERMINATED";
+    case PMIX_ERR_INVALID_TERMINATION:
+        return "INVALID-TERMINATION";
 
     case PMIX_ERR_EVENT_REGISTRATION:
         return "EVENT-REGISTRATION";
-    case PMIX_ERR_JOB_TERMINATED:
-        return "PMIX_ERR_JOB_TERMINATED";
     case PMIX_ERR_UPDATE_ENDPOINTS:
         return "UPDATE-ENDPOINTS";
     case PMIX_MODEL_DECLARED:
@@ -196,19 +201,60 @@ PMIX_EXPORT const char* PMIx_Error_string(pmix_status_t errnum)
         return "PROC-HAS-CONNECTED";
     case PMIX_CONNECT_REQUESTED:
         return "CONNECT-REQUESTED";
-    case PMIX_LAUNCH_DIRECTIVE:
-        return "LAUNCH-DIRECTIVE";
+    case PMIX_OPENMP_PARALLEL_ENTERED:
+        return "OPENMP-PARALLEL-ENTERED";
+    case PMIX_OPENMP_PARALLEL_EXITED:
+        return "OPENMP-PARALLEL-EXITED";
+
     case PMIX_LAUNCHER_READY:
         return "LAUNCHER-READY";
     case PMIX_OPERATION_IN_PROGRESS:
         return "OPERATION-IN-PROGRESS";
+    case PMIX_OPERATION_SUCCEEDED:
+        return "OPERATION-SUCCEEDED";
+    case PMIX_ERR_INVALID_OPERATION:
+        return "INVALID-OPERATION";
 
+    case PMIX_GROUP_INVITED:
+        return "GROUP-INVITED";
+    case PMIX_GROUP_LEFT:
+        return "GROUP-LEFT";
+    case PMIX_GROUP_INVITE_ACCEPTED:
+        return "GROUP-INVITE-ACCEPTED";
+    case PMIX_GROUP_INVITE_DECLINED:
+        return "GROUP-INVITE-DECLINED";
+    case PMIX_GROUP_INVITE_FAILED:
+        return "GROUP-INVITE-FAILED";
+    case PMIX_GROUP_MEMBERSHIP_UPDATE:
+        return "GROUP-MEMBERSHIP-UPDATE";
+    case PMIX_GROUP_CONSTRUCT_ABORT:
+        return "GROUP-CONSTRUCT-ABORT";
+    case PMIX_GROUP_CONSTRUCT_COMPLETE:
+        return "GROUP-CONSTRUCT-COMPLETE";
+    case PMIX_GROUP_LEADER_SELECTED:
+        return "GROUP-LEADER-SELECTED";
+    case PMIX_GROUP_LEADER_FAILED:
+        return "GROUP-LEADER-FAILED";
+    case PMIX_GROUP_CONTEXT_ID_ASSIGNED:
+        return "GROUP-CONTEXT-ID-ASSIGNED";
+    case PMIX_GROUP_MEMBER_FAILED:
+        return "GROUP-MEMBER-FAILED";
 
-    case PMIX_ERR_NODE_DOWN:
+    case PMIX_FABRIC_UPDATED:
+        return "FABRIC-UPDATED";
+    case PMIX_FABRIC_UPDATE_PENDING:
+        return "FABRIC-UPDATE-PENDING";
+    case PMIX_ERR_REPEAT_ATTR_REGISTRATION:
+        return "REPEAT-ATTRIBUTE-REGISTRATION";
+
+    case PMIX_EVENT_NODE_DOWN:
         return "NODE-DOWN";
-    case PMIX_ERR_NODE_OFFLINE:
+    case PMIX_EVENT_NODE_OFFLINE:
         return "NODE-OFFLINE";
-
+    case PMIX_EVENT_SYS_OTHER:
+        return "UNDEFINED-SYSTEM-EVENT";
+    case PMIX_EVENT_SYS_BASE:
+        return "SYSTEM BASE EVENT";
 
     case PMIX_EVENT_NO_ACTION_TAKEN:
         return "EVENT-NO-ACTION-TAKEN";
@@ -228,8 +274,8 @@ PMIX_EXPORT const char* PMIx_Error_string(pmix_status_t errnum)
         return "PMIX_ERR_VALUE_OUT_OF_BOUNDS";
     case PMIX_ERR_PERM:
         return "PMIX_ERR_PERM";
-    case PMIX_ERR_NETWORK_NOT_PARSEABLE:
-        return "PMIX_ERR_NETWORK_NOT_PARSEABLE";
+    case PMIX_ERR_FABRIC_NOT_PARSEABLE:
+        return "PMIX_ERR_FABRIC_NOT_PARSEABLE";
     case PMIX_ERR_FILE_OPEN_FAILURE:
         return "PMIX_ERR_FILE_OPEN_FAILURE";
     case PMIX_ERR_FILE_READ_FAILURE:
@@ -239,10 +285,51 @@ PMIX_EXPORT const char* PMIx_Error_string(pmix_status_t errnum)
     case PMIX_ERR_TEMP_UNAVAILABLE:
         return "PMIX TEMPORARILY UNAVAILABLE";
 
+    case PMIX_LAUNCH_COMPLETE:
+        return "PMIX LAUNCH COMPLETE";
 
     case PMIX_MAX_ERR_CONSTANT:
         return "PMIX_ERR_WILDCARD";
 
+    case PMIX_ERR_JOB_APP_NOT_EXECUTABLE:
+        return "APPLICATION NOT EXECUTABLE";
+    case PMIX_ERR_JOB_NO_EXE_SPECIFIED:
+        return "NO EXECUTABLE SPECIFIED";
+    case PMIX_ERR_JOB_FAILED_TO_MAP:
+        return "JOB FAILED TO MAP";
+    case PMIX_ERR_JOB_CANCELED:
+        return "JOB CANCELED";
+    case PMIX_ERR_JOB_FAILED_TO_LAUNCH:
+        return "JOB FAILED TO LAUNCH";
+    case PMIX_ERR_JOB_ABORTED:
+        return "JOB ABORTED";
+    case PMIX_ERR_JOB_KILLED_BY_CMD:
+        return "KILLED BY COMMAND";
+    case PMIX_ERR_JOB_ABORTED_BY_SIG:
+        return "ABORTED BY SIGNAL";
+    case PMIX_ERR_JOB_TERM_WO_SYNC:
+        return "TERMINATED WITHOUT SYNC";
+    case PMIX_ERR_JOB_SENSOR_BOUND_EXCEEDED:
+        return "SENSOR BOUND EXCEEDED";
+    case PMIX_ERR_JOB_NON_ZERO_TERM:
+        return "NON-ZERO TERMINATION";
+    case PMIX_ERR_JOB_ALLOC_FAILED:
+        return "FAILED TO OBTAIN ALLOCATION";
+    case PMIX_ERR_JOB_ABORTED_BY_SYS_EVENT:
+        return "JOB ABORTED BY SYSTEM EVENT";
+    case PMIX_ERR_PROC_TERM_WO_SYNC:
+        return "PROC TERMINATED WITHOUT SYNC";
+
+    case PMIX_EVENT_PROC_TERMINATED:
+        return "PROC TERMINATED";
+    case PMIX_EVENT_JOB_START:
+        return "JOB STARTED";
+    case PMIX_EVENT_JOB_END:
+        return "JOB ENDED";
+    case PMIX_EVENT_SESSION_START:
+        return "SESSION STARTED";
+    case PMIX_EVENT_SESSION_END:
+        return "SESSION ENDED";
 
     default:
         return "ERROR STRING NOT FOUND";
