@@ -16,7 +16,7 @@
  *                         reserved.
  * Copyright (c) 2015-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2016-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2016-2020 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,7 +25,7 @@
  */
 
 #include "pmix_config.h"
-#include "pmix_common.h"
+#include "include/pmix_common.h"
 
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -51,23 +51,7 @@
 #include <arpa/inet.h>
 #endif
 #ifdef HAVE_NET_IF_H
-#if defined(__APPLE__) && defined(_LP64)
-/* Apple engineering suggested using options align=power as a
-   workaround for a bug in OS X 10.4 (Tiger) that prevented ioctl(...,
-   SIOCGIFCONF, ...) from working properly in 64 bit mode on Power PC.
-   It turns out that the underlying issue is the size of struct
-   ifconf, which the kernel expects to be 12 and natural 64 bit
-   alignment would make 16.  The same bug appears in 64 bit mode on
-   Intel macs, but align=power is a no-op there, so instead, use the
-   pack pragma to instruct the compiler to pack on 4 byte words, which
-   has the same effect as align=power for our needs and works on both
-   Intel and Power PC Macs. */
-#pragma pack(push,4)
-#endif
 #include <net/if.h>
-#if defined(__APPLE__) && defined(_LP64)
-#pragma pack(pop)
-#endif
 #endif
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
@@ -218,7 +202,7 @@ int pmix_ifaddrtoname(const char* if_addr, char* if_name, int length)
                 memcpy (&ipv4, r->ai_addr, r->ai_addrlen);
 
                 if (inaddr->sin_addr.s_addr == ipv4.sin_addr.s_addr) {
-                    strncpy(if_name, intf->if_name, length);
+                    pmix_strncpy(if_name, intf->if_name, length-1);
                     freeaddrinfo (res);
                     return PMIX_SUCCESS;
                 }
@@ -226,7 +210,7 @@ int pmix_ifaddrtoname(const char* if_addr, char* if_name, int length)
             else {
                 if (IN6_ARE_ADDR_EQUAL(&((struct sockaddr_in6*) &intf->if_addr)->sin6_addr,
                     &((struct sockaddr_in6*) r->ai_addr)->sin6_addr)) {
-                    strncpy(if_name, intf->if_name, length);
+                    pmix_strncpy(if_name, intf->if_name, length-1);
                     freeaddrinfo (res);
                     return PMIX_SUCCESS;
                 }
@@ -493,7 +477,7 @@ int pmix_ifindextoname(int if_index, char* if_name, int length)
         intf != (pmix_pif_t*)pmix_list_get_end(&pmix_if_list);
         intf =  (pmix_pif_t*)pmix_list_get_next(intf)) {
         if (intf->if_index == if_index) {
-            strncpy(if_name, intf->if_name, length);
+            pmix_strncpy(if_name, intf->if_name, length-1);
             return PMIX_SUCCESS;
         }
     }
@@ -514,7 +498,7 @@ int pmix_ifkindextoname(int if_kindex, char* if_name, int length)
         intf != (pmix_pif_t*)pmix_list_get_end(&pmix_if_list);
         intf =  (pmix_pif_t*)pmix_list_get_next(intf)) {
         if (intf->if_kernel_index == if_kindex) {
-            strncpy(if_name, intf->if_name, length);
+            pmix_strncpy(if_name, intf->if_name, length-1);
             return PMIX_SUCCESS;
         }
     }
@@ -555,14 +539,14 @@ static int parse_ipv4_dots(const char *addr, uint32_t* net, int* dots)
         }
         /* did we read something sensible? */
         if( n[i] > 255 ) {
-            return PMIX_ERR_NETWORK_NOT_PARSEABLE;
+            return PMIX_ERR_FABRIC_NOT_PARSEABLE;
         }
         /* skip all the . */
         for( start = end; '\0' != *start; start++ )
             if( '.' != *start ) break;
     }
     *dots = i;
-    *net = PMIX_PIF_ASSEMBLE_NETWORK(n[0], n[1], n[2], n[3]);
+    *net = PMIX_PIF_ASSEMBLE_FABRIC(n[0], n[1], n[2], n[3]);
     return PMIX_SUCCESS;
 }
 
@@ -591,7 +575,7 @@ pmix_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
                 pval = strtol(ptr, NULL, 10);
                 if ((pval > 31) || (pval < 1)) {
                     pmix_output(0, "pmix_iftupletoaddr: unknown mask");
-                    return PMIX_ERR_NETWORK_NOT_PARSEABLE;
+                    return PMIX_ERR_FABRIC_NOT_PARSEABLE;
                 }
                 *mask = 0xFFFFFFFF << (32 - pval);
             }
@@ -616,7 +600,7 @@ pmix_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
                 *mask = 0xFF000000;
             } else {
                 pmix_output(0, "pmix_iftupletoaddr: unknown mask");
-                return PMIX_ERR_NETWORK_NOT_PARSEABLE;
+                return PMIX_ERR_FABRIC_NOT_PARSEABLE;
             }
         }
     }
