@@ -13,7 +13,7 @@
  *                         All rights reserved.
  * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2013-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
@@ -25,10 +25,10 @@
  *
  */
 
-#include <src/include/pmix_config.h>
-#include <pmix_server.h>
-#include <src/include/types.h>
-#include <src/include/pmix_globals.h>
+#include "src/include/pmix_config.h"
+#include "include/pmix_server.h"
+#include "src/include/types.h"
+#include "src/include/pmix_globals.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,15 +39,12 @@
 #include <errno.h>
 #include <signal.h>
 
-#if PMIX_HAVE_HWLOC
-#include <src/hwloc/hwloc-internal.h>
-#endif
-
 #include "src/class/pmix_list.h"
 #include "src/util/pmix_environ.h"
 #include "src/util/output.h"
 #include "src/util/printf.h"
 #include "src/util/argv.h"
+#include "src/runtime/pmix_progress_threads.h"
 
 #include "simptest.h"
 
@@ -198,15 +195,9 @@ static pmix_event_t handler;
 static pmix_list_t children;
 static bool istimeouttest = false;
 static mylock_t globallock;
-static bool nettest = false;
 static bool model = false;
 static bool xversion = false;
-static char *hostnames[] = {
-    "test000",
-    "test001",
-    "test002",
-    NULL
-};
+static pmix_event_base_t *simptest_evbase = NULL;
 
 static void set_namespace(int nprocs, char *nspace,
                           pmix_op_cbfunc_t cbfunc, myxfer_t *x);
@@ -258,11 +249,13 @@ static void infocbfunc(pmix_status_t status,
      * server */
     x = PMIX_NEW(myxfer_t);
     x->ninfo = ninfo;
-    PMIX_INFO_CREATE(x->info, x->ninfo);
+    if (0 < ninfo) {
+        PMIX_INFO_CREATE(x->info, x->ninfo);
+    }
     for (n=0; n < ninfo; n++) {
         PMIX_INFO_XFER(&x->info[n], &info[n]);
     }
-    PMIX_THREADSHIFT(x, dlcbfunc);
+    SIMPTEST_THREADSHIFT(x, dlcbfunc);
 
     if (NULL != release_fn) {
         release_fn(release_cbdata);
@@ -357,10 +350,6 @@ int main(int argc, char **argv)
     wait_tracker_t *child;
     pmix_info_t *info;
     size_t ninfo;
-    bool hwloc = false;
-#if PMIX_HAVE_HWLOC
-    char *hwloc_file = NULL;
-#endif
     mylock_t mylock;
     pmix_status_t code;
     sigset_t unblock;
@@ -389,36 +378,14 @@ int main(int argc, char **argv)
                 pmix_argv_append_nosize(&client_argv, argv[k]);
             }
             n += k;
-#if PMIX_HAVE_HWLOC
-        } else if (0 == strcmp("-hwloc", argv[n]) ||
-                   0 == strcmp("--hwloc", argv[n])) {
-            /* test hwloc support */
-            hwloc = true;
-        } else if (0 == strcmp("-hwloc-file", argv[n]) ||
-                   0 == strcmp("--hwloc-file", argv[n])) {
-            if (NULL == argv[n+1]) {
-                fprintf(stderr, "The --hwloc-file option requires an argument\n");
-                exit(1);
-            }
-            hwloc_file = strdup(argv[n+1]);
-            hwloc = true;
-            ++n;
-#endif
         } else if (0 == strcmp("-h", argv[n])) {
             /* print the options and exit */
             fprintf(stderr, "usage: simptest <options>\n");
             fprintf(stderr, "    -n N     Number of clients to run\n");
             fprintf(stderr, "    -e foo   Name of the client executable to run (default: simpclient\n");
             fprintf(stderr, "    -u       Enable legacy usock support\n");
-            fprintf(stderr, "    -hwloc   Test hwloc support\n");
-            fprintf(stderr, "    -hwloc-file FILE   Use file to import topology\n");
-            fprintf(stderr, "    -net-test  Test network endpt assignments\n");
             fprintf(stderr, "    -xversion  Cross-version test - simulate single node only\n");
             exit(0);
-        } else if (0 == strcmp("-net-test", argv[n]) ||
-                   0 == strcmp("--net-test", argv[n])) {
-            /* test network support */
-            nettest = true;
         }  else if (0 == strcmp("-model", argv[n]) ||
                    0 == strcmp("--model", argv[n])) {
             /* test network support */
@@ -430,11 +397,7 @@ int main(int argc, char **argv)
         }
     }
     if (NULL == executable) {
-        if (nettest) {
-            executable = strdup("./simpcoord");
-        } else {
-            executable = strdup("./simpclient");
-        }
+        executable = strdup("./simpclient");
     }
     /* check for executable existence and permissions */
     if (0 != access(executable, X_OK)) {
@@ -442,13 +405,6 @@ int main(int argc, char **argv)
         exit(1);
     }
 
-#if !PMIX_HAVE_HWLOC
-    if (hwloc) {
-        fprintf(stderr, "PMIx was not configured with HWLOC support - cannot continue\n");
-        exit(1);
-    }
-#endif
-
     fprintf(stderr, "Testing version %s\n", PMIx_Get_version());
 
     /* ensure that SIGCHLD is unblocked as we need to capture it */
@@ -467,48 +423,19 @@ int main(int argc, char **argv)
 
 
     /* setup the server library and tell it to support tool connections */
-#if PMIX_HAVE_HWLOC
-    if (hwloc) {
-#if HWLOC_API_VERSION < 0x20000
-        ninfo = 2;
-#else
-        ninfo = 3;
-#endif
-    } else {
-        ninfo = 1;
-    }
-#else
-    ninfo = 1;
-#endif
-
+    ninfo = 2;
     PMIX_INFO_CREATE(info, ninfo);
     PMIX_INFO_LOAD(&info[0], PMIX_SERVER_TOOL_SUPPORT, NULL, PMIX_BOOL);
-#if PMIX_HAVE_HWLOC
-    if (hwloc) {
-        if (NULL != hwloc_file) {
-            PMIX_INFO_LOAD(&info[1], PMIX_TOPOLOGY_FILE, hwloc_file, PMIX_STRING);
-        } else {
-            PMIX_INFO_LOAD(&info[1], PMIX_TOPOLOGY, NULL, PMIX_STRING);
-        }
-#if HWLOC_API_VERSION >= 0x20000
-        PMIX_INFO_LOAD(&info[2], PMIX_HWLOC_SHARE_TOPO, NULL, PMIX_BOOL);
-#endif
-    }
-#endif
-    if (nettest) {
-        /* set a known network configuration for the pnet/test component */
-        putenv("PMIX_MCA_pnet_test_planes=plane:d:3;plane:s:2;plane:d:5:2");
-        putenv("PMIX_MCA_pnet=test");
-    }
+    PMIX_INFO_LOAD(&info[1], PMIX_SERVER_SCHEDULER, NULL, PMIX_BOOL);
     if (PMIX_SUCCESS != (rc = PMIx_server_init(&mymodule, info, ninfo))) {
         fprintf(stderr, "Init failed with error %s\n", PMIx_Error_string(rc));
         return rc;
     }
     PMIX_INFO_FREE(info, ninfo);
-    if (nettest) {
-        unsetenv("PMIX_MCA_pnet");
-        unsetenv("PMIX_MCA_pnet_test_planes");
-    }
+
+    /* get our own event base */
+    simptest_evbase = pmix_progress_thread_init("simptest");
+    pmix_progress_thread_start("simptest");
 
     /* register the default errhandler */
     DEBUG_CONSTRUCT_LOCK(&mylock);
@@ -588,10 +515,6 @@ int main(int argc, char **argv)
             PMIx_server_finalize();
             return rc;
         }
-        /* add the hostname we want them to use */
-        if (!xversion) {
-            PMIX_SETENV(rc, "PMIX_HOSTNAME", hostnames[n % 3], &client_env);
-        }
 
         x = PMIX_NEW(myxfer_t);
         if (PMIX_SUCCESS != (rc = PMIx_server_register_client(&proc, myuid, mygid,
@@ -722,58 +645,20 @@ static void set_namespace(int nprocs, char *nspace,
     pmix_info_t *isv1, *isv2;
     myxfer_t cd, lock;
     pmix_status_t rc;
-    char **map[3] = {NULL, NULL, NULL};
-    char *peers[3] = {NULL, NULL, NULL};
     char tmp[50] , **agg = NULL;
 
-    if (xversion) {
-        /* everything on one node */
-        PMIx_generate_regex(pmix_globals.hostname, &regex);
-        for (m=0; m < nprocs; m++) {
-            snprintf(tmp, 50, "%d", m);
-            pmix_argv_append_nosize(&agg, tmp);
-            memset(tmp, 0, 50);
-        }
-        rks = pmix_argv_join(agg, ',');
-        pmix_argv_free(agg);
-        PMIx_generate_ppn(rks, &ppn);
-        free(rks);
-        nnodes = 1;
-    } else {
-        if (nprocs < 3) {
-            /* take only the number of hostnames equal to
-             * the number of procs */
-            for (m=0; m < nprocs; m++) {
-                pmix_argv_append_nosize(&agg, hostnames[m]);
-            }
-            ppn = pmix_argv_join(agg, ',');
-            pmix_argv_free(agg);
-            agg = NULL;
-            nnodes = nprocs;
-        } else {
-            nnodes = 3;
-            ppn = pmix_argv_join(hostnames, ',');
-        }
-        PMIx_generate_regex(ppn, &regex);
-        free(ppn);
-        /* compute the placement of the procs */
-        for (m=0; m < nprocs; m++) {
-            snprintf(tmp, 50, "%d", m);
-            pmix_argv_append_nosize(&map[m%3], tmp);
-            memset(tmp, 0, 50);
-        }
-        for (m=0; m < 3; m++) {
-            if (NULL != map[m]) {
-                peers[m] = pmix_argv_join(map[m], ',');
-                pmix_argv_append_nosize(&agg, peers[m]);
-                pmix_argv_free(map[m]);
-            }
-        }
-        rks = pmix_argv_join(agg, ';');
-        pmix_argv_free(agg);
-        PMIx_generate_ppn(rks, &ppn);
-        free(rks);
+    /* everything on one node */
+    PMIx_generate_regex(pmix_globals.hostname, &regex);
+    for (m=0; m < nprocs; m++) {
+        snprintf(tmp, 50, "%d", m);
+        pmix_argv_append_nosize(&agg, tmp);
+        memset(tmp, 0, 50);
     }
+    rks = pmix_argv_join(agg, ',');
+    pmix_argv_free(agg);
+    PMIx_generate_ppn(rks, &ppn);
+    free(rks);
+    nnodes = 1;
 
     x->ninfo = 1 + nprocs + nnodes;
     PMIX_INFO_CREATE(x->info, x->ninfo);
@@ -787,9 +672,9 @@ static void set_namespace(int nprocs, char *nspace,
         PMIX_DATA_ARRAY_CREATE(x->info[n].value.data.darray, 9, PMIX_INFO);
     }
     iptr = (pmix_info_t*)x->info[n].value.data.darray->array;
-    PMIX_INFO_LOAD(&iptr[0], PMIX_NODE_MAP, regex, PMIX_STRING);
+    PMIX_INFO_LOAD(&iptr[0], PMIX_NODE_MAP, regex, PMIX_REGEX);
     isv1 = &iptr[0];
-    PMIX_INFO_LOAD(&iptr[1], PMIX_PROC_MAP, ppn, PMIX_STRING);
+    PMIX_INFO_LOAD(&iptr[1], PMIX_PROC_MAP, ppn, PMIX_REGEX);
     isv2 = &iptr[1];
     PMIX_INFO_LOAD(&iptr[2], PMIX_JOB_SIZE, &nprocs, PMIX_UINT32);
     PMIX_INFO_LOAD(&iptr[3], PMIX_JOBID, "1234", PMIX_STRING);
@@ -809,14 +694,14 @@ static void set_namespace(int nprocs, char *nspace,
     PMIX_INFO_XFER(&iptr[0], isv1);
     PMIX_INFO_XFER(&iptr[1], isv2);
     PMIX_INFO_LOAD(&iptr[2], PMIX_SETUP_APP_ENVARS, NULL, PMIX_BOOL);
-    PMIX_LOAD_KEY(iptr[3].key, PMIX_ALLOC_NETWORK);
+    PMIX_LOAD_KEY(iptr[3].key, PMIX_ALLOC_FABRIC);
     iptr[3].value.type = PMIX_DATA_ARRAY;
     PMIX_DATA_ARRAY_CREATE(iptr[3].value.data.darray, 2, PMIX_INFO);
     ip = (pmix_info_t*)iptr[3].value.data.darray->array;
     asprintf(&rks, "%s.net", nspace);
-    PMIX_INFO_LOAD(&ip[0], PMIX_ALLOC_NETWORK_ID, rks, PMIX_STRING);
+    PMIX_INFO_LOAD(&ip[0], PMIX_ALLOC_FABRIC_ID, rks, PMIX_STRING);
     free(rks);
-    PMIX_INFO_LOAD(&ip[1], PMIX_ALLOC_NETWORK_SEC_KEY, NULL, PMIX_BOOL);
+    PMIX_INFO_LOAD(&ip[1], PMIX_ALLOC_FABRIC_SEC_KEY, NULL, PMIX_BOOL);
     PMIX_CONSTRUCT(&cd, myxfer_t);
     if (PMIX_SUCCESS != (rc = PMIx_server_setup_application(nspace, iptr, 4,
                                                              setup_cbfunc, &cd))) {
@@ -844,11 +729,7 @@ static void set_namespace(int nprocs, char *nspace,
         x->info[n].value.type = PMIX_DATA_ARRAY;
         PMIX_DATA_ARRAY_CREATE(x->info[n].value.data.darray, 3, PMIX_INFO);
         iptr = (pmix_info_t*)x->info[n].value.data.darray->array;
-        if (xversion) {
-            PMIX_INFO_LOAD(&iptr[0], PMIX_HOSTNAME, pmix_globals.hostname, PMIX_STRING);
-        } else {
-            PMIX_INFO_LOAD(&iptr[0], PMIX_HOSTNAME, hostnames[m % 3], PMIX_STRING);
-        }
+        PMIX_INFO_LOAD(&iptr[0], PMIX_HOSTNAME, pmix_globals.hostname, PMIX_STRING);
         PMIX_INFO_LOAD(&iptr[1], PMIX_NODEID, &m, PMIX_UINT32);
         PMIX_INFO_LOAD(&iptr[2], PMIX_NODE_SIZE, &nprocs, PMIX_UINT32);
         ++n;
@@ -887,11 +768,7 @@ static void set_namespace(int nprocs, char *nspace,
 
         (void)strncpy(info[k].key, PMIX_HOSTNAME, PMIX_MAX_KEYLEN);
         info[k].value.type = PMIX_STRING;
-        if (xversion) {
-            info[k].value.data.string = strdup(pmix_globals.hostname);
-        } else {
-            info[k].value.data.string = strdup(hostnames[m % 3]);
-        }
+        info[k].value.data.string = strdup(pmix_globals.hostname);
         ++k;
         /* move to next proc */
         ++n;
@@ -1017,11 +894,41 @@ static pmix_status_t fencenb_fn(const pmix_proc_t procs[], size_t nprocs,
     scd->ndata = ndata;
     scd->cbfunc.modexcbfunc = cbfunc;
     scd->cbdata = cbdata;
-    PMIX_THREADSHIFT(scd, fencbfn);
+    SIMPTEST_THREADSHIFT(scd, fencbfn);
     return PMIX_SUCCESS;
 }
 
 
+static void modex_resp(pmix_status_t status,
+                       char *data, size_t sz,
+                       void *cbdata)
+{
+    pmix_shift_caddy_t *scd = (pmix_shift_caddy_t*)cbdata;
+    scd->status = status;
+    scd->data = (char*)malloc(sz);
+    memcpy((char*)scd->data, data, sz);
+    scd->ndata = sz;
+    SIMPTEST_THREADSHIFT(scd, fencbfn);
+}
+
+static void dmdxfn(int sd, short args, void *cbdata)
+{
+    pmix_shift_caddy_t *scd = (pmix_shift_caddy_t*)cbdata;
+    pmix_proc_t proc;
+    pmix_status_t rc;
+
+    /* get the data */
+    PMIX_LOAD_PROCID(&proc, scd->pname.nspace, scd->pname.rank);
+    rc = PMIx_server_dmodex_request(&proc, modex_resp, scd);
+    if (PMIX_SUCCESS != rc) {
+        scd->status = rc;
+        scd->data = NULL;
+        scd->ndata = 0;
+        SIMPTEST_THREADSHIFT(scd, fencbfn);
+    }
+    return;
+}
+
 static pmix_status_t dmodex_fn(const pmix_proc_t *proc,
                      const pmix_info_t info[], size_t ninfo,
                      pmix_modex_cbfunc_t cbfunc, void *cbdata)
@@ -1034,10 +941,11 @@ static pmix_status_t dmodex_fn(const pmix_proc_t *proc,
     }
 
     scd = PMIX_NEW(pmix_shift_caddy_t);
-    scd->status = PMIX_ERR_NOT_FOUND;
+    scd->pname.nspace = strdup(proc->nspace);
+    scd->pname.rank = proc->rank;
     scd->cbfunc.modexcbfunc = cbfunc;
     scd->cbdata = cbdata;
-    PMIX_THREADSHIFT(scd, fencbfn);
+    SIMPTEST_THREADSHIFT(scd, dmdxfn);
 
     return PMIX_SUCCESS;
 }
@@ -1125,7 +1033,7 @@ static pmix_status_t lookup_fn(const pmix_proc_t *proc, char **keys,
         lk->n = n;
         lk->cbfunc = cbfunc;
         lk->cbdata = cbdata;
-        PMIX_THREADSHIFT(lk, lkcbfn);
+        SIMPTEST_THREADSHIFT(lk, lkcbfn);
     }
 
     return ret;
@@ -1283,7 +1191,7 @@ static pmix_status_t query_fn(pmix_proc_t *proct,
     qd.ndata = nqueries;
     qd.cbfunc = cbfunc;
     qd.cbdata = cbdata;
-    PMIX_THREADSHIFT(&qd, qfn);
+    SIMPTEST_THREADSHIFT(&qd, qfn);
     return PMIX_SUCCESS;
 }
 
@@ -1322,7 +1230,7 @@ static void log_fn(const pmix_proc_t *client,
 
     lg->cbfunc = cbfunc;
     lg->cbdata = cbdata;
-    PMIX_THREADSHIFT(lg, foobar);
+    SIMPTEST_THREADSHIFT(lg, foobar);
 }
 
 static pmix_status_t alloc_fn(const pmix_proc_t *client,
