@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2016-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2017 Mellanox Technologies, Inc.
  *                         All rights reserved.
  * $COPYRIGHT$
@@ -103,42 +103,42 @@ int test_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
             if (PMIX_SUCCESS != rc) {
                 TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
-                return rc;
+                exit(rc);
             }
 
             PUT(int, fence_num+my_rank, PMIX_GLOBAL, fence_num, put_ind++, params.use_same_keys);
             if (PMIX_SUCCESS != rc) {
                 TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
-                return rc;
+                exit(rc);
             }
 
             PUT(float, fence_num+1.1, PMIX_GLOBAL, fence_num, put_ind++, params.use_same_keys);
             if (PMIX_SUCCESS != rc) {
                 TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
-                return rc;
+                exit(rc);
             }
 
             PUT(uint32_t, fence_num+14, PMIX_GLOBAL, fence_num, put_ind++, params.use_same_keys);
             if (PMIX_SUCCESS != rc) {
                 TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
-                return rc;
+                exit(rc);
             }
 
             PUT(uint16_t, fence_num+15, PMIX_GLOBAL, fence_num, put_ind++, params.use_same_keys);
             if (PMIX_SUCCESS != rc) {
                 TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
-                return rc;
+                exit(rc);
             }
 
             /* Submit the data */
             if (PMIX_SUCCESS != (rc = PMIx_Commit())) {
                 TEST_ERROR(("%s:%d: PMIx_Commit failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
-                return rc;
+                exit(rc);
             }
 
             /* setup the fence */
@@ -146,8 +146,7 @@ int test_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
             PMIX_PROC_CREATE(pcs, npcs);
             i = 0;
             PMIX_LIST_FOREACH(p, desc->participants, participant_t) {
-                (void)strncpy(pcs[i].nspace, p->proc.nspace, PMIX_MAX_NSLEN);
-                pcs[i].rank = p->proc.rank;
+                PMIX_LOAD_PROCID(&pcs[i], p->proc.nspace, p->proc.rank);
                 i++;
             }
 
@@ -157,7 +156,7 @@ int test_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
                 TEST_ERROR(("%s:%d: PMIx_Fence failed: %d", my_nspace, my_rank, rc));
                 PMIX_LIST_DESTRUCT(&test_fences);
                 PMIX_PROC_FREE(pcs, npcs);
-                return rc;
+                exit(rc);
             }
 
             /* replace all items in the list with PMIX_RANK_WILDCARD rank by real ranks to get their data. */
@@ -169,14 +168,13 @@ int test_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
                     if (PMIX_SUCCESS != rc) {
                         TEST_ERROR(("%s:%d: Can't parse --ns-dist value in order to get ranks for namespace %s", my_nspace, my_rank, p->proc.nspace));
                         PMIX_LIST_DESTRUCT(&test_fences);
-                        return PMIX_ERROR;
+                        exit(PMIX_ERROR);
                     }
                     pmix_list_remove_item(desc->participants, (pmix_list_item_t*)p);
                     for (i = 0; i < nranks; i++) {
                         participant_t *prt;
                         prt = PMIX_NEW(participant_t);
-                        strncpy(prt->proc.nspace, ranks[i].nspace, strlen(ranks[i].nspace)+1);
-                        prt->proc.rank = ranks[i].rank;
+                        PMIX_LOAD_PROCID(&prt->proc, ranks[i].nspace, ranks[i].rank);
                         pmix_list_append(desc->participants, &prt->super);
                     }
                     PMIX_PROC_FREE(ranks, nranks);
@@ -189,38 +187,38 @@ int test_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
                 snprintf(sval, 500, "%d:%s:%d", fence_num, p->proc.nspace, p->proc.rank);
                 GET(string, sval, p->proc.nspace, p->proc.rank, fence_num, put_ind++, params.use_same_keys, 1, 0);
                 if (PMIX_SUCCESS != rc) {
-                    TEST_ERROR(("%s:%d: PMIx_Get failed (%d) from %s:%d", my_nspace, my_rank, rc, p->proc.nspace, p->proc.rank));
+                    TEST_ERROR(("%s:%d: PMIx_Get failed (%s) from %s:%d", my_nspace, my_rank, PMIx_Error_string(rc), p->proc.nspace, p->proc.rank));
                     PMIX_PROC_FREE(pcs, npcs);
                     PMIX_LIST_DESTRUCT(&test_fences);
-                    return rc;
+                    exit(rc);
                 }
                 GET(int, (int)(fence_num+p->proc.rank), p->proc.nspace, p->proc.rank, fence_num, put_ind++, params.use_same_keys, 0, 0);
                 if (PMIX_SUCCESS != rc) {
                     TEST_ERROR(("%s:%d: PMIx_Get failed (%d) from %s:%d", my_nspace, my_rank, rc, p->proc.nspace, p->proc.rank));
                     PMIX_PROC_FREE(pcs, npcs);
                     PMIX_LIST_DESTRUCT(&test_fences);
-                    return rc;
+                    exit(rc);
                 }
                 GET(float, fence_num+1.1, p->proc.nspace, p->proc.rank, fence_num, put_ind++, params.use_same_keys, 1, 0);
                 if (PMIX_SUCCESS != rc) {
                     TEST_ERROR(("%s:%d: PMIx_Get failed (%d) from %s:%d", my_nspace, my_rank, rc, p->proc.nspace, p->proc.rank));
                     PMIX_PROC_FREE(pcs, npcs);
                     PMIX_LIST_DESTRUCT(&test_fences);
-                    return rc;
+                    exit(rc);
                 }
                 GET(uint32_t, (uint32_t)fence_num+14, p->proc.nspace, p->proc.rank, fence_num, put_ind++, params.use_same_keys, 0, 0);
                 if (PMIX_SUCCESS != rc) {
                     TEST_ERROR(("%s:%d: PMIx_Get failed (%d) from %s:%d", my_nspace, my_rank, rc, p->proc.nspace, p->proc.rank));
                     PMIX_PROC_FREE(pcs, npcs);
                     PMIX_LIST_DESTRUCT(&test_fences);
-                    return rc;
+                    exit(rc);
                 }
                 GET(uint16_t, fence_num+15, p->proc.nspace, p->proc.rank, fence_num, put_ind++, params.use_same_keys, 1, 0);
                 if (PMIX_SUCCESS != rc) {
                     TEST_ERROR(("%s:%d: PMIx_Get failed (%d) from %s:%d", my_nspace, my_rank, rc, p->proc.nspace, p->proc.rank));
                     PMIX_PROC_FREE(pcs, npcs);
                     PMIX_LIST_DESTRUCT(&test_fences);
-                    return rc;
+                    exit(rc);
                 }
             }
             /* barrier across participating processes to prevent putting new values with the same key
@@ -246,35 +244,35 @@ static int get_local_peers(char *my_nspace, int my_rank, pmix_rank_t **_peers, p
 
     (void)strncpy(proc.nspace, my_nspace, PMIX_MAX_NSLEN);
     proc.rank = PMIX_RANK_WILDCARD;
-    /* get number of neighbours on this node */
+    /* get number of neighbors on this node */
     if (PMIX_SUCCESS != (rc = PMIx_Get(&proc, PMIX_LOCAL_SIZE, NULL, 0, &val))) {
         TEST_ERROR(("%s:%d: PMIx_Get local peer # failed: %d", my_nspace, my_rank, rc));
-        return rc;
+        exit(rc);
     }
     if (NULL == val) {
         TEST_ERROR(("%s:%d: PMIx_Get local peer # returned NULL value", my_nspace, my_rank));
-        return PMIX_ERROR;
+        exit(PMIX_ERROR);
     }
 
     if (val->type != PMIX_UINT32  ) {
         TEST_ERROR(("%s:%d: local peer # attribute value type mismatch,"
                 " want %d get %d(%d)",
                 my_nspace, my_rank, PMIX_UINT32, val->type));
-        return PMIX_ERROR;
+        exit(PMIX_ERROR);
     }
     npeers = val->data.uint32;
     peers = malloc(sizeof(pmix_rank_t) * npeers);
 
-    /* get ranks of neighbours on this node */
+    /* get ranks of neighbors on this node */
     if (PMIX_SUCCESS != (rc = PMIx_Get(&proc, PMIX_LOCAL_PEERS, NULL, 0, &val))) {
         TEST_ERROR(("%s:%d: PMIx_Get local peers failed: %d", my_nspace, my_rank, rc));
         free(peers);
-        return rc;
+        exit(rc);
     }
     if (NULL == val) {
         TEST_ERROR(("%s:%d: PMIx_Get local peers returned NULL value", my_nspace, my_rank));
         free(peers);
-        return PMIX_ERROR;
+        exit(PMIX_ERROR);
     }
 
     if (val->type != PMIX_STRING  ) {
@@ -282,7 +280,7 @@ static int get_local_peers(char *my_nspace, int my_rank, pmix_rank_t **_peers, p
                 " want %d get %d(%d)",
                 my_nspace, my_rank, PMIX_UINT32, val->type));
         free(peers);
-        return PMIX_ERROR;
+        exit(PMIX_ERROR);
     }
 
     *count = 0;
@@ -293,7 +291,7 @@ static int get_local_peers(char *my_nspace, int my_rank, pmix_rank_t **_peers, p
             TEST_ERROR(("%s:%d: Bad peer ranks number: should be %d, actual %d (%s)",
                 my_nspace, my_rank, npeers, *count, val->data.string));
             free(peers);
-            return PMIX_ERROR;
+            exit(PMIX_ERROR);
         }
         token = strtok_r(str, ",", &sptr);
         str = NULL;
@@ -302,7 +300,7 @@ static int get_local_peers(char *my_nspace, int my_rank, pmix_rank_t **_peers, p
             if( *eptr != '\0' ){
                 TEST_ERROR(("%s:%d: Bad peer ranks string", my_nspace, my_rank));
                 free(peers);
-                return PMIX_ERROR;
+                exit(PMIX_ERROR);
             }
         }
 
@@ -312,7 +310,7 @@ static int get_local_peers(char *my_nspace, int my_rank, pmix_rank_t **_peers, p
         TEST_ERROR(("%s:%d: Bad peer ranks number: should be %d, actual %d (%s)",
                 my_nspace, my_rank, npeers, *count, val->data.string));
         free(peers);
-        return PMIX_ERROR;
+        exit(PMIX_ERROR);
     }
     *_peers = peers;
     return PMIX_SUCCESS;
@@ -327,6 +325,7 @@ int test_job_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
     pmix_value_t value;
     pmix_value_t *val = &value;
     pmix_proc_t proc;
+    pmix_info_t info;
 
     (void)strncpy(proc.nspace, my_nspace, PMIX_MAX_NSLEN);
     proc.rank = my_rank;
@@ -335,104 +334,115 @@ int test_job_fence(test_params params, char *my_nspace, pmix_rank_t my_rank)
         PUT(int, 12340 + i, PMIX_LOCAL, 100, i, 0);
         if (PMIX_SUCCESS != rc) {
             TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
-            return rc;
+            exit(rc);
         }
 
         (void)snprintf(sval, 50, "%s:%d", my_nspace, my_rank);
         PUT(string, sval, PMIX_REMOTE, 101, i, 0);
         if (PMIX_SUCCESS != rc) {
             TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
-            return PMIX_ERROR;
+            exit(PMIX_ERROR);
         }
 
         PUT(float, (float)12.15 + i, PMIX_GLOBAL, 102, i, 0);
         if (PMIX_SUCCESS != rc) {
             TEST_ERROR(("%s:%d: PMIx_Put failed: %d", my_nspace, my_rank, rc));
-            return PMIX_ERROR;
+            exit(PMIX_ERROR);
         }
     }
 
     /* Submit the data */
     if (PMIX_SUCCESS != (rc = PMIx_Commit())) {
         TEST_ERROR(("%s:%d: PMIx_Commit failed: %d", my_nspace, my_rank, rc));
-        return PMIX_ERROR;
+        exit(PMIX_ERROR);
     }
 
     /* Perform a fence if was requested */
     FENCE(!params.nonblocking, params.collect, NULL, 0);
     if (PMIX_SUCCESS != rc) {
         TEST_ERROR(("%s:%d: PMIx_Fence failed: %d", my_nspace, my_rank, rc));
-        return rc;
+        exit(rc);
     }
 
     if (PMIX_SUCCESS != (rc = get_local_peers(my_nspace, my_rank, &peers, &npeers))) {
-        return PMIX_ERROR;
+        exit(rc);
     }
 
     /* Check the predefined output */
     for (i=0; i < (int)params.ns_size; i++) {
-
         for (j=0; j < 3; j++) {
-
             int local = 0;
             pmix_rank_t k;
             for(k=0; k<npeers; k++){
                 if( peers[k] == i+params.base_rank){
                     local = 1;
+                    break;
                 }
             }
             if( local ){
                 GET(int, (12340+j), my_nspace, i+params.base_rank, 100, j, 0, 0, 0);
                 if (PMIX_SUCCESS != rc) {
                     TEST_ERROR(("%s:%d: PMIx_Get failed: %s", my_nspace, my_rank, PMIx_Error_string(rc)));
-                    return PMIX_ERROR;
+                    exit(rc);
                 }
 
                 snprintf(sval, 50, "%s:%d", my_nspace, i+params.base_rank);
                 GET(string, sval, my_nspace, i+params.base_rank, 101, j, 0, 1, 1);
                 if (PMIX_SUCCESS == rc && (i+params.base_rank) != my_rank ) {
                     TEST_ERROR(("%s:%d: PMIx_Get of remote key on local proc", my_nspace, my_rank));
-                    return PMIX_ERROR;
+                    exit(PMIX_ERROR);
                 }
             } else {
                 GET(int, (12340+j), my_nspace, i+params.base_rank, 100, j, 0, 0, 1);
                 if (PMIX_SUCCESS == rc && (i+params.base_rank) != my_rank) {
                     TEST_ERROR(("%s:%d: PMIx_Get of local key on the remote proc", my_nspace, my_rank));
-                    return PMIX_ERROR;
+                    exit(PMIX_ERROR);
                 }
 
                 snprintf(sval, 50, "%s:%d", my_nspace, i+params.base_rank);
                 GET(string, sval, my_nspace, i+params.base_rank, 101, j, 0, 1, 0);
                 if (PMIX_SUCCESS != rc) {
                     TEST_ERROR(("%s:%d: PMIx_Get failed (%d)", my_nspace, my_rank, rc));
-                    return PMIX_ERROR;
+                    exit(PMIX_ERROR);
                 }
             }
 
             GET(float, (float)12.15 + j, my_nspace, i+params.base_rank, 102, j, 0, 0, 0);
             if (PMIX_SUCCESS != rc) {
                 TEST_ERROR(("%s:%d: PMIx_Get failed (%d)", my_nspace, my_rank, rc));
-                return PMIX_ERROR;
+                exit(PMIX_ERROR);
             }
         }
 
-        /* ask for a non-existent key */
-        proc.rank = i+params.base_rank;
-        if (PMIX_SUCCESS == (rc = PMIx_Get(&proc, "foobar", NULL, 0, &val))) {
+      cleanout:
+        TEST_VERBOSE(("%s:%d: rank %d is OK", my_nspace, my_rank, i+params.base_rank));
+    }
+
+    /* ask for a non-existent key */
+    if (0 == my_rank) {
+        proc.rank = 1+params.base_rank;
+        j = 1;
+        PMIX_INFO_LOAD(&info, PMIX_TIMEOUT, &j, PMIX_INT);
+        PMIX_INFO_REQUIRED(&info);
+        if (PMIX_SUCCESS == (rc = PMIx_Get(&proc, "foobar", &info, 1, &val))) {
             TEST_ERROR(("%s:%d: PMIx_Get returned success instead of failure",
                         my_nspace, my_rank));
-            return PMIX_ERROR;
+            exit(PMIX_ERROR);
+        }
+        if (PMIX_ERR_NOT_SUPPORTED == rc) {
+            goto cleanout;
         }
         if (PMIX_ERR_NOT_FOUND != rc && PMIX_ERR_PROC_ENTRY_NOT_FOUND != rc) {
             TEST_ERROR(("%s:%d [ERROR]: PMIx_Get returned %s instead of not_found",
                         my_nspace, my_rank, PMIx_Error_string(rc)));
-            return PMIX_ERROR;
+            exit(PMIX_ERROR);
         }
         if (NULL != val) {
             TEST_ERROR(("%s:%d [ERROR]: PMIx_Get did not return NULL value", my_nspace, my_rank));
-            return PMIX_ERROR;
+            exit(PMIX_ERROR);
         }
-        TEST_VERBOSE(("%s:%d: rank %d is OK", my_nspace, my_rank, i+params.base_rank));
     }
+
+    free(peers);
     return PMIX_SUCCESS;
 }
