@@ -2,8 +2,8 @@
  * Copyright (c) 2015-2019 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2018 Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2016      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -30,7 +30,6 @@
 #include "server_callbacks.h"
 
 int my_server_id = 0;
-int test_fail = 0;
 
 server_info_t *my_server_info = NULL;
 pmix_list_t *server_list = NULL;
@@ -41,7 +40,7 @@ static void sdes(server_info_t *s)
     close(s->rd_fd);
     close(s->wr_fd);
     if (s->evread) {
-        event_del(s->evread);
+        pmix_event_del(s->evread);
     }
     s->evread = NULL;
     if (NULL != s->hostname) {
@@ -188,7 +187,7 @@ static void set_namespace(int local_size, int univ_size,
             return;
         }
         free(tmp);
-        PMIX_INFO_LOAD(&info[4], PMIX_NODE_MAP, regex, PMIX_STRING);
+        PMIX_INFO_LOAD(&info[4], PMIX_NODE_MAP, regex, PMIX_REGEX);
     }
 
     /* generate the global proc map - if we have two
@@ -221,7 +220,7 @@ static void set_namespace(int local_size, int univ_size,
     }
     PMIx_generate_ppn(ranks, &ppn);
     free(ranks);
-    PMIX_INFO_LOAD(&info[5], PMIX_PROC_MAP, ppn, PMIX_STRING);
+    PMIX_INFO_LOAD(&info[5], PMIX_PROC_MAP, ppn, PMIX_REGEX);
     free(ppn);
 
     pmix_strncpy(info[6].key, PMIX_JOB_SIZE, PMIX_MAX_KEYLEN);
@@ -925,16 +924,14 @@ int server_init(test_params *params)
         server_info_t *server;
         PMIX_LIST_FOREACH(server, server_list, server_info_t) {
             server->evread = pmix_event_new(pmix_globals.evbase, server->rd_fd,
-                              EV_READ|EV_PERSIST, server_read_cb, server);
+                                            EV_READ|EV_PERSIST, server_read_cb, server);
             pmix_event_add(server->evread, NULL);
         }
     }
 
-#if 0
     /* register the errhandler */
     PMIx_Register_event_handler(NULL, 0, NULL, 0,
                                 errhandler, errhandler_reg_callbk, NULL);
-#endif
 
     /* setup to see sigchld on the forked tests */
     pmix_event_assign(&handler, pmix_globals.evbase, SIGCHLD,
@@ -953,12 +950,11 @@ error:
     return rc;
 }
 
-int server_finalize(test_params *params)
+int server_finalize(test_params *params, int local_fail)
 {
     int rc = PMIX_SUCCESS;
-    int total_ret = 0;
+    int total_ret = local_fail;
 
-    total_ret = test_fail;
     if (0 != (rc = server_barrier())) {
         total_ret++;
         goto exit;
@@ -970,15 +966,16 @@ int server_finalize(test_params *params)
     }
 
     if (params->nservers && 0 == my_server_id) {
-        int ret;
         /* wait for all servers are finished */
-        ret = srv_wait_all(10.0);
-        if (!pmix_list_is_empty(server_list)) {
-            total_ret += ret;
-        }
+        total_ret += srv_wait_all(10.0);
         PMIX_LIST_RELEASE(server_list);
         TEST_VERBOSE(("SERVER %d FINALIZE PID:%d with status %d",
-                    my_server_id, getpid(), ret));
+                        my_server_id, getpid(), total_ret));
+        if (0 == total_ret) {
+            TEST_OUTPUT(("Test finished OK!"));
+        } else {
+            rc = PMIX_ERROR;
+        }
     }
     PMIX_LIST_RELEASE(server_nspace);
 
@@ -988,11 +985,6 @@ int server_finalize(test_params *params)
         total_ret += rc;
         goto exit;
     }
-    if (0 == total_ret) {
-        TEST_OUTPUT(("Test finished OK!"));
-    } else {
-        TEST_OUTPUT(("Test FAILED!"));
-    }
 
 exit:
     return total_ret;
